import{G as c,I as l,Sd as j,U as a,W as u,X as o,ea as s,ga as d,h as p,ia as h,na as g,oa as m,ta as b}from"./chunk-XJKQ67XR.js";import{e as f}from"./chunk-SZAW6ZBW.js";var I,v=f(()=>{g();m();j();I=(()=>{class i{constructor(){this.longOperationRunningSubject=new p({visible:!1,indicatorType:"standard"}),this.longOperationRunning=this.longOperationRunningSubject.asObservable()}showIndicator(e="standard"){this.longOperationRunningSubject.next({visible:!0,indicatorType:e})}hideIndicator(e="standard"){this.longOperationRunningSubject.next({visible:!1,indicatorType:e})}indicate(e,t=1e3){return n=>n.pipe(s(1),this.prepare(r=>e.next(r),t),o(()=>e.next(!1)))}observe(e="standard",t=1e3){return n=>n.pipe(s(1),this.prepare(r=>this.longOperationRunningSubject.next({visible:r,indicatorType:e}),t),o(()=>this.hideIndicator(e)))}prepare(e,t=1e3){return n=>(l(c(t).pipe(a(!0),h(n)),n.pipe(a(!1))).pipe(d(!1),u()).subscribe(r=>e(r)),n)}static{this.\u0275fac=function(t){return new(t||i)}}static{this.\u0275prov=b({token:i,factory:i.\u0275fac,providedIn:"root"})}}return i})()});export{I as a,v as b};
