import{e}from"./chunk-SZAW6ZBW.js";var o,t=e(()=>{o={production:!0,hmr:!1,cloudUrl:"https://cloud.devolutions.net/",dvlsUrl:"",i18nPath:"prod/",version:"3.0.1-1002",location:"",applicationName:"",siteKey:{devolutionsSend:"0x4AAAAAAA-tGoR6cNBD8o8X"},openSearchConfigurations:{url:"https://telemetry2.devolutions.net:9200",username:"DVLS_2024.1",password:"??/$w_Nz9.jtGWxx",datasourceFeatureEventIndex:"feature_events"}}});export{o as a,t as b};
