import{c as M,e as w}from"./chunk-SNKEPRYU.js";import{C as T,E as D,a as f,f as k,m as _}from"./chunk-K7KAWBRR.js";import{Hc as b,Mb as d,Od as m,Rb as u,Sd as C,Xa as r,Ya as l,_a as g,kc as p,kd as v,sa as i,xc as h,yc as c}from"./chunk-XJKQ67XR.js";import{e as B}from"./chunk-SZAW6ZBW.js";var R,y=B(()=>{C();D();w();C();D();R=(()=>{class o{constructor(){this.options=l([]),this.disabled=l(!1),this.change=r(),this.value=d(null),this.cvaDisabled=g(!1),this.isDisabled=m(()=>this.disabled()||this.cvaDisabled()),this.onCvaChange=()=>{},this.onCvaTouched=()=>{}}onNgModelChange(e){this.value.set(e),this.onCvaChange(e)}markTouched(){this.onCvaTouched()}writeValue(e){this.value.set(e)}registerOnChange(e){this.onCvaChange=e}registerOnTouched(e){this.onCvaTouched=e}setDisabledState(e){this.cvaDisabled.set(e)}static{this.\u0275fac=function(n){return new(n||o)}}static{this.\u0275cmp=u({type:o,selectors:[["dvl-dashboard-toggle-button"]],hostAttrs:[1,"dvl-dashboard-toggle"],inputs:{options:[1,"options"],disabled:[1,"disabled"],value:[1,"value"]},outputs:{change:"change",value:"valueChange"},features:[v([{provide:f,useExisting:i(()=>o),multi:!0}])],decls:1,vars:3,consts:[["styleClass","tabs-lookalike",3,"ngModelChange","onChange","blur","options","disabled","ngModel"]],template:function(n,t){n&1&&(h(0,"p-selectButton",0),b("ngModelChange",function(a){return t.onNgModelChange(a)})("onChange",function(a){return t.change.emit(a)})("blur",function(){return t.markTouched()}),c()),n&2&&p("options",t.options())("disabled",t.isDisabled())("ngModel",t.value())},dependencies:[M,T,k,_],styles:[`.dvl-dashboard-toggle .p-selectbutton .tabs-lookalike.p-togglebutton{border-width:0 0 2px 0;border-radius:0;font-weight:600;font-size:14px;letter-spacing:-.02em;--p-togglebutton-color: var(--label-input-color);--p-togglebutton-background: transparent;--p-togglebutton-border-color: transparent;--p-togglebutton-padding: inherit 0}.dvl-dashboard-toggle .p-selectbutton .tabs-lookalike.p-togglebutton.p-togglebutton-checked{--p-togglebutton-checked-border-color: var(--connection-view-link-color);--p-togglebutton-checked-color: var(--connection-view-link-color);--p-togglebutton-checked-background: transparent}.dvl-dashboard-toggle .p-selectbutton .tabs-lookalike.p-togglebutton:hover{--p-togglebutton-hover-color: inherit;--p-togglebutton-hover-background: transparent;--p-togglebutton-hover-border-color: transparent}.dvl-dashboard-toggle .p-selectbutton .tabs-lookalike.p-togglebutton:not(:last-child){margin-right:20px}
`],encapsulation:2})}}return o})()});export{R as a,y as b};
