import{c as n,d as h}from"./chunk-CUIDVWNH.js";import{e as i}from"./chunk-SZAW6ZBW.js";var r,u=i(()=>{r=function(e){return e[e.Default=0]="Default",e[e.My=1]="My",e[e.Favourite=2]="Favourite",e[e.All=3]="All",e}(r||{});(function(e){function t(){return"JiraFilterEndpoint"}e.getClass=t;function o(l){let s="";return typeof e.getClass=="function"&&(s=e.getClass()),s+"_"+e[l]}e.getEnumKey=o})(r||(r={}))});var a,m=i(()=>{h();u();a=class e{toQueryString(){let t=[];return this.jiraFilterEndpoint!==null&&this.jiraFilterEndpoint.toString()&&t.push("jiraFilterEndpoint="+encodeURIComponent(this.jiraFilterEndpoint.toString())),this.serverUri!==null&&this.serverUri.toString()&&t.push("serverUri="+encodeURIComponent(this.serverUri.toString())),this.userName!==null&&this.userName.toString()&&t.push("userName="+encodeURIComponent(this.userName.toString())),this.apiKey!==null&&this.apiKey.toString()&&t.push("apiKey="+encodeURIComponent(this.apiKey.toString())),t.join("&")}constructor(t=null){this.clone=()=>new e(this),this.jiraFilterEndpoint=n.manageDefaultValue(t,"jiraFilterEndpoint",r.Default),this.serverUri=n.manageDefaultValue(t,"serverUri",""),this.userName=n.manageDefaultValue(t,"userName",""),this.apiKey=n.manageDefaultValue(t,"apiKey","")}get jiraFilterEndpointKey(){return r.getEnumKey(this.jiraFilterEndpoint)}}});export{r as a,u as b,a as c,m as d};
