import{c as i,d as s}from"./chunk-CUIDVWNH.js";import{e as n}from"./chunk-SZAW6ZBW.js";var a,d=n(()=>{s();a=class e{get externalId(){return this.sid}set externalId(t){this.sid=t}toQueryString(){let t=[];return this.distinguishedName!==null&&this.distinguishedName.toString()&&t.push("distinguishedName="+encodeURIComponent(this.distinguishedName.toString())),this.domainName!==null&&this.domainName.toString()&&t.push("domainName="+encodeURIComponent(this.domainName.toString())),this.name!==null&&this.name.toString()&&t.push("name="+encodeURIComponent(this.name.toString())),this.sid!==null&&this.sid.toString()&&t.push("sid="+encodeURIComponent(this.sid.toString())),t.join("&")}constructor(t=null){this.clone=()=>new e(this),this.distinguishedName=i.manageDefaultValue(t,"distinguishedName",""),this.domainName=i.manageDefaultValue(t,"domainName",""),this.name=i.manageDefaultValue(t,"name",""),this.sid=i.manageDefaultValue(t,"sid","")}}});var o,h=n(()=>{s();o=class e{get externalId(){return this.sid}set externalId(t){this.sid=t}toQueryString(){let t=[];return this.name!==null&&this.name.toString()&&t.push("name="+encodeURIComponent(this.name.toString())),this.sid!==null&&this.sid.toString()&&t.push("sid="+encodeURIComponent(this.sid.toString())),t.join("&")}constructor(t=null){this.clone=()=>new e(this),this.name=i.manageDefaultValue(t,"name",""),this.sid=i.manageDefaultValue(t,"sid","")}}});export{a,d as b,o as c,h as d};
