import{Sd as n,g as r,na as u,ta as o}from"./chunk-XJKQ67XR.js";import{e as c}from"./chunk-SZAW6ZBW.js";var b,a=c(()=>{u();n();b=(()=>{class t{constructor(){this.showDialogSource=new r,this.resultSource=new r,this.showDialog$=this.showDialogSource.asObservable(),this.result$=this.resultSource.asObservable()}show(){return this.showDialogSource.next(),new Promise(e=>{let s=this.result$.subscribe(i=>{e(i),s.unsubscribe()})})}handleResult(e){this.resultSource.next(e)}static{this.\u0275fac=function(s){return new(s||t)}}static{this.\u0275prov=o({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{b as a,a as b};
