import{a as Ot,c as Gt,d as It,f as Nt,g as Tt,i as jt}from"./chunk-LEYQOZ37.js";import{a as Ct,c as Zt}from"./chunk-DM2XHT3H.js";import{E as At,a as We}from"./chunk-K7KAWBRR.js";import{a as mt,c as Qt}from"./chunk-XUUJ7NBF.js";import{A as ft,C as gt,U as yt,d as be,da as bt,e as dt,i as X,j as ut,k as ee,l as ht,m as Ht,na as Kt,oa as vt,qa as Pt,ra as te,ta as ve,va as Bt,wa as xt,y as _t,ya as qt}from"./chunk-QH7CG3K4.js";import{c as Ue,d as Te,g as we,i as Se,j as Ve,l as ke,s as Rt}from"./chunk-IS75LVR4.js";import{$ as _e,Ca as ct,D as tt,Ea as pt,H as it,Ha as fe,Ia as E,Ja as ge,La as ye,Q as J,U as nt,Y as Fe,Z as ot,aa as Q,b as Ee,ba as lt,d as Ye,e as Je,fa as Le,na as st,u as H,v as k,wa as at,x as Xe,xa as rt,y as et}from"./chunk-5YPL7P3G.js";import{D as Y,F as je,K as me,m as N,o as Ne,p as j,s as U,t as W}from"./chunk-HJI5FNS6.js";import{$c as q,Ac as w,Bb as c,Bc as S,Bd as L,Cc as C,Dc as I,Gb as R,Hc as v,Ic as a,Jc as Qe,Ka as _,Kc as Ke,La as f,Ld as g,Md as Z,Oa as K,Od as G,Pd as Ge,Rb as P,Rc as y,Sb as re,Sc as V,Sd as he,Tc as d,Ua as b,Uc as u,Va as ae,Vb as B,Xb as p,Yc as A,_a as D,_c as $,ad as Oe,ed as Pe,fd as Be,gd as qe,jc as T,kc as r,kd as pe,lc as He,ld as de,md as O,nc as ce,nd as Ie,oc as z,od as ue,sa as Ae,sc as Ce,ta as oe,ua as le,ud as Ze,xc as h,yc as m,za as se,zc as F}from"./chunk-XJKQ67XR.js";import{a as M,b as $e,e as Re}from"./chunk-SZAW6ZBW.js";function li(t,s){t&1&&C(0)}function si(t,s){if(t&1){let e=I();h(0,"div",3,1),v("click",function(i){_(e);let o=a(2);return f(o.onOverlayContentClick(i))})("@overlayContentAnimation.start",function(i){_(e);let o=a(2);return f(o.onOverlayContentAnimationStart(i))})("@overlayContentAnimation.done",function(i){_(e);let o=a(2);return f(o.onOverlayContentAnimationDone(i))}),Ke(2),p(3,li,1,0,"ng-container",4),m()}if(t&2){let e=a(2);z(e.contentStyleClass),r("ngStyle",e.contentStyle)("ngClass","p-overlay-content")("@overlayContentAnimation",O(11,ii,ue(7,ti,e.showTransitionOptions,e.hideTransitionOptions,e.transformOptions[e.modal?e.overlayResponsiveDirection:"default"]))),c(3),r("ngTemplateOutlet",e.contentTemplate||e._contentTemplate)("ngTemplateOutletContext",O(15,oi,O(13,ni,e.overlayMode)))}}function ai(t,s){if(t&1){let e=I();h(0,"div",3,0),v("click",function(){_(e);let i=a();return f(i.onOverlayClick())}),p(2,si,4,17,"div",2),m()}if(t&2){let e=a();z(e.styleClass),r("ngStyle",e.style)("ngClass",Ze(5,ei,[e.modal,e.modal&&e.overlayResponsiveDirection==="center",e.modal&&e.overlayResponsiveDirection==="top",e.modal&&e.overlayResponsiveDirection==="top-start",e.modal&&e.overlayResponsiveDirection==="top-end",e.modal&&e.overlayResponsiveDirection==="bottom",e.modal&&e.overlayResponsiveDirection==="bottom-start",e.modal&&e.overlayResponsiveDirection==="bottom-end",e.modal&&e.overlayResponsiveDirection==="left",e.modal&&e.overlayResponsiveDirection==="left-start",e.modal&&e.overlayResponsiveDirection==="left-end",e.modal&&e.overlayResponsiveDirection==="right",e.modal&&e.overlayResponsiveDirection==="right-start",e.modal&&e.overlayResponsiveDirection==="right-end"])),c(2),r("ngIf",e.visible)}}var wt,Jt,Xt,ei,ti,ii,ni,oi,ri,St,ci,pi,Me,go,Vt=Re(()=>{Rt();me();me();he();he();rt();ye();ye();ut();Ht();Bt();dt();wt=["content"],Jt=["overlay"],Xt=["*"],ei=(t,s,e,n,i,o,l,x,ze,ne,xe,Dt,zt,$t)=>({"p-overlay p-component":!0,"p-overlay-modal p-overlay-mask p-overlay-mask-enter":t,"p-overlay-center":s,"p-overlay-top":e,"p-overlay-top-start":n,"p-overlay-top-end":i,"p-overlay-bottom":o,"p-overlay-bottom-start":l,"p-overlay-bottom-end":x,"p-overlay-left":ze,"p-overlay-left-start":ne,"p-overlay-left-end":xe,"p-overlay-right":Dt,"p-overlay-right-start":zt,"p-overlay-right-end":$t}),ti=(t,s,e)=>({showTransitionParams:t,hideTransitionParams:s,transform:e}),ii=t=>({value:"visible",params:t}),ni=t=>({mode:t}),oi=t=>({$implicit:t});ri=({dt:t})=>`
.p-overlay {
    position: absolute;
    top: 0;
}

.p-overlay-modal {
    display: flex;
    align-items: center;
    justify-content: center;
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
}

.p-overlay-content {
    transform-origin: inherit;
}

.p-overlay-modal > .p-overlay-content {
    z-index: 1;
    width: 90%;
}

/* Position */
/* top */
.p-overlay-top {
    align-items: flex-start;
}
.p-overlay-top-start {
    align-items: flex-start;
    justify-content: flex-start;
}
.p-overlay-top-end {
    align-items: flex-start;
    justify-content: flex-end;
}

/* bottom */
.p-overlay-bottom {
    align-items: flex-end;
}
.p-overlay-bottom-start {
    align-items: flex-end;
    justify-content: flex-start;
}
.p-overlay-bottom-end {
    align-items: flex-end;
    justify-content: flex-end;
}

/* left */
.p-overlay-left {
    justify-content: flex-start;
}
.p-overlay-left-start {
    justify-content: flex-start;
    align-items: flex-start;
}
.p-overlay-left-end {
    justify-content: flex-start;
    align-items: flex-end;
}

/* right */
.p-overlay-right {
    justify-content: flex-end;
}
.p-overlay-right-start {
    justify-content: flex-end;
    align-items: flex-start;
}
.p-overlay-right-end {
    justify-content: flex-end;
    align-items: flex-end;
}
`,St=(()=>{class t extends be{name="overlay";theme=ri;static \u0275fac=(()=>{let e;return function(i){return(e||(e=K(t)))(i||t)}})();static \u0275prov=oe({token:t,factory:t.\u0275fac})}return t})(),ci=Ve([we({transform:"{{transform}}",opacity:0}),Te("{{showTransitionParams}}")]),pi=Ve([Te("{{hideTransitionParams}}",we({transform:"{{transform}}",opacity:0}))]),Me=(()=>{class t extends X{overlayService;zone;get visible(){return this._visible}set visible(e){this._visible=e,this._visible&&!this.modalVisible&&(this.modalVisible=!0)}get mode(){return this._mode||this.overlayOptions?.mode}set mode(e){this._mode=e}get style(){return te.merge(this._style,this.modal?this.overlayResponsiveOptions?.style:this.overlayOptions?.style)}set style(e){this._style=e}get styleClass(){return te.merge(this._styleClass,this.modal?this.overlayResponsiveOptions?.styleClass:this.overlayOptions?.styleClass)}set styleClass(e){this._styleClass=e}get contentStyle(){return te.merge(this._contentStyle,this.modal?this.overlayResponsiveOptions?.contentStyle:this.overlayOptions?.contentStyle)}set contentStyle(e){this._contentStyle=e}get contentStyleClass(){return te.merge(this._contentStyleClass,this.modal?this.overlayResponsiveOptions?.contentStyleClass:this.overlayOptions?.contentStyleClass)}set contentStyleClass(e){this._contentStyleClass=e}get target(){let e=this._target||this.overlayOptions?.target;return e===void 0?"@prev":e}set target(e){this._target=e}get appendTo(){return this._appendTo||this.overlayOptions?.appendTo}set appendTo(e){this._appendTo=e}get autoZIndex(){let e=this._autoZIndex||this.overlayOptions?.autoZIndex;return e===void 0?!0:e}set autoZIndex(e){this._autoZIndex=e}get baseZIndex(){let e=this._baseZIndex||this.overlayOptions?.baseZIndex;return e===void 0?0:e}set baseZIndex(e){this._baseZIndex=e}get showTransitionOptions(){let e=this._showTransitionOptions||this.overlayOptions?.showTransitionOptions;return e===void 0?".12s cubic-bezier(0, 0, 0.2, 1)":e}set showTransitionOptions(e){this._showTransitionOptions=e}get hideTransitionOptions(){let e=this._hideTransitionOptions||this.overlayOptions?.hideTransitionOptions;return e===void 0?".1s linear":e}set hideTransitionOptions(e){this._hideTransitionOptions=e}get listener(){return this._listener||this.overlayOptions?.listener}set listener(e){this._listener=e}get responsive(){return this._responsive||this.overlayOptions?.responsive}set responsive(e){this._responsive=e}get options(){return this._options}set options(e){this._options=e}visibleChange=new b;onBeforeShow=new b;onShow=new b;onBeforeHide=new b;onHide=new b;onAnimationStart=new b;onAnimationDone=new b;overlayViewChild;contentViewChild;contentTemplate;templates;_contentTemplate;_visible=!1;_mode;_style;_styleClass;_contentStyle;_contentStyleClass;_target;_appendTo;_autoZIndex;_baseZIndex;_showTransitionOptions;_hideTransitionOptions;_listener;_responsive;_options;modalVisible=!1;isOverlayClicked=!1;isOverlayContentClicked=!1;scrollHandler;documentClickListener;documentResizeListener;_componentStyle=se(St);documentKeyboardListener;window;transformOptions={default:"scaleY(0.8)",center:"scale(0.7)",top:"translate3d(0px, -100%, 0px)","top-start":"translate3d(0px, -100%, 0px)","top-end":"translate3d(0px, -100%, 0px)",bottom:"translate3d(0px, 100%, 0px)","bottom-start":"translate3d(0px, 100%, 0px)","bottom-end":"translate3d(0px, 100%, 0px)",left:"translate3d(-100%, 0px, 0px)","left-start":"translate3d(-100%, 0px, 0px)","left-end":"translate3d(-100%, 0px, 0px)",right:"translate3d(100%, 0px, 0px)","right-start":"translate3d(100%, 0px, 0px)","right-end":"translate3d(100%, 0px, 0px)"};get modal(){if(je(this.platformId))return this.mode==="modal"||this.overlayResponsiveOptions&&this.document.defaultView?.matchMedia(this.overlayResponsiveOptions.media?.replace("@media","")||`(max-width: ${this.overlayResponsiveOptions.breakpoint})`).matches}get overlayMode(){return this.mode||(this.modal?"modal":"overlay")}get overlayOptions(){return M(M({},this.config?.overlayOptions),this.options)}get overlayResponsiveOptions(){return M(M({},this.overlayOptions?.responsive),this.responsive)}get overlayResponsiveDirection(){return this.overlayResponsiveOptions?.direction||"center"}get overlayEl(){return this.overlayViewChild?.nativeElement}get contentEl(){return this.contentViewChild?.nativeElement}get targetEl(){return it(this.target,this.el?.nativeElement)}constructor(e,n){super(),this.overlayService=e,this.zone=n}ngAfterContentInit(){this.templates?.forEach(e=>{switch(e.getType()){case"content":this._contentTemplate=e.template;break;default:this._contentTemplate=e.template;break}})}show(e,n=!1){this.onVisibleChange(!0),this.handleEvents("onShow",{overlay:e||this.overlayEl,target:this.targetEl,mode:this.overlayMode}),n&&k(this.targetEl),this.modal&&Ee(this.document?.body,"p-overflow-hidden")}hide(e,n=!1){if(this.visible)this.onVisibleChange(!1),this.handleEvents("onHide",{overlay:e||this.overlayEl,target:this.targetEl,mode:this.overlayMode}),n&&k(this.targetEl),this.modal&&Ye(this.document?.body,"p-overflow-hidden");else return}alignOverlay(){!this.modal&&ee.alignOverlay(this.overlayEl,this.targetEl,this.appendTo)}onVisibleChange(e){this._visible=e,this.visibleChange.emit(e)}onOverlayClick(){this.isOverlayClicked=!0}onOverlayContentClick(e){this.overlayService.add({originalEvent:e,target:this.targetEl}),this.isOverlayContentClicked=!0}onOverlayContentAnimationStart(e){switch(e.toState){case"visible":this.handleEvents("onBeforeShow",{overlay:this.overlayEl,target:this.targetEl,mode:this.overlayMode}),this.autoZIndex&&ve.set(this.overlayMode,this.overlayEl,this.baseZIndex+this.config?.zIndex[this.overlayMode]),ee.appendOverlay(this.overlayEl,this.appendTo==="body"?this.document.body:this.appendTo,this.appendTo),this.alignOverlay();break;case"void":this.handleEvents("onBeforeHide",{overlay:this.overlayEl,target:this.targetEl,mode:this.overlayMode}),this.modal&&Ee(this.overlayEl,"p-overlay-mask-leave");break}this.handleEvents("onAnimationStart",e)}onOverlayContentAnimationDone(e){let n=this.overlayEl||e.element.parentElement;switch(e.toState){case"visible":this.visible&&(this.show(n,!0),this.bindListeners());break;case"void":if(!this.visible){this.hide(n,!0),this.modalVisible=!1,this.unbindListeners(),ee.appendOverlay(this.overlayEl,this.targetEl,this.appendTo),ve.clear(n),this.cd.markForCheck();break}}this.handleEvents("onAnimationDone",e)}handleEvents(e,n){this[e].emit(n),this.options&&this.options[e]&&this.options[e](n),this.config?.overlayOptions&&(this.config?.overlayOptions)[e]&&(this.config?.overlayOptions)[e](n)}bindListeners(){this.bindScrollListener(),this.bindDocumentClickListener(),this.bindDocumentResizeListener(),this.bindDocumentKeyboardListener()}unbindListeners(){this.unbindScrollListener(),this.unbindDocumentClickListener(),this.unbindDocumentResizeListener(),this.unbindDocumentKeyboardListener()}bindScrollListener(){this.scrollHandler||(this.scrollHandler=new ht(this.targetEl,e=>{(this.listener?this.listener(e,{type:"scroll",mode:this.overlayMode,valid:!0}):!0)&&this.hide(e,!0)})),this.scrollHandler.bindScrollListener()}unbindScrollListener(){this.scrollHandler&&this.scrollHandler.unbindScrollListener()}bindDocumentClickListener(){this.documentClickListener||(this.documentClickListener=this.renderer.listen(this.document,"click",e=>{let i=!(this.targetEl&&(this.targetEl.isSameNode(e.target)||!this.isOverlayClicked&&this.targetEl.contains(e.target)))&&!this.isOverlayContentClicked;(this.listener?this.listener(e,{type:"outside",mode:this.overlayMode,valid:e.which!==3&&i}):i)&&this.hide(e),this.isOverlayClicked=this.isOverlayContentClicked=!1}))}unbindDocumentClickListener(){this.documentClickListener&&(this.documentClickListener(),this.documentClickListener=null)}bindDocumentResizeListener(){this.documentResizeListener||(this.documentResizeListener=this.renderer.listen(this.document.defaultView,"resize",e=>{(this.listener?this.listener(e,{type:"resize",mode:this.overlayMode,valid:!J()}):!J())&&this.hide(e,!0)}))}unbindDocumentResizeListener(){this.documentResizeListener&&(this.documentResizeListener(),this.documentResizeListener=null)}bindDocumentKeyboardListener(){this.documentKeyboardListener||this.zone.runOutsideAngular(()=>{this.documentKeyboardListener=this.renderer.listen(this.document.defaultView,"keydown",e=>{if(this.overlayOptions.hideOnEscape===!1||e.code!=="Escape")return;(this.listener?this.listener(e,{type:"keydown",mode:this.overlayMode,valid:!J()}):!J())&&this.zone.run(()=>{this.hide(e,!0)})})})}unbindDocumentKeyboardListener(){this.documentKeyboardListener&&(this.documentKeyboardListener(),this.documentKeyboardListener=null)}ngOnDestroy(){this.hide(this.overlayEl,!0),this.overlayEl&&(ee.appendOverlay(this.overlayEl,this.targetEl,this.appendTo),ve.clear(this.overlayEl)),this.scrollHandler&&(this.scrollHandler.destroy(),this.scrollHandler=null),this.unbindListeners(),super.ngOnDestroy()}static \u0275fac=function(n){return new(n||t)(R(pt),R(ae))};static \u0275cmp=P({type:t,selectors:[["p-overlay"]],contentQueries:function(n,i,o){if(n&1&&(y(o,wt,4),y(o,fe,4)),n&2){let l;d(l=u())&&(i.contentTemplate=l.first),d(l=u())&&(i.templates=l)}},viewQuery:function(n,i){if(n&1&&(V(Jt,5),V(wt,5)),n&2){let o;d(o=u())&&(i.overlayViewChild=o.first),d(o=u())&&(i.contentViewChild=o.first)}},inputs:{visible:"visible",mode:"mode",style:"style",styleClass:"styleClass",contentStyle:"contentStyle",contentStyleClass:"contentStyleClass",target:"target",appendTo:"appendTo",autoZIndex:"autoZIndex",baseZIndex:"baseZIndex",showTransitionOptions:"showTransitionOptions",hideTransitionOptions:"hideTransitionOptions",listener:"listener",responsive:"responsive",options:"options"},outputs:{visibleChange:"visibleChange",onBeforeShow:"onBeforeShow",onShow:"onShow",onBeforeHide:"onBeforeHide",onHide:"onHide",onAnimationStart:"onAnimationStart",onAnimationDone:"onAnimationDone"},features:[pe([St]),B],ngContentSelectors:Xt,decls:1,vars:1,consts:[["overlay",""],["content",""],[3,"ngStyle","class","ngClass","click",4,"ngIf"],[3,"click","ngStyle","ngClass"],[4,"ngTemplateOutlet","ngTemplateOutletContext"]],template:function(n,i){n&1&&(Qe(),p(0,ai,3,20,"div",2)),n&2&&r("ngIf",i.modalVisible)},dependencies:[Y,N,j,W,U,E],encapsulation:2,data:{animation:[Ue("overlayContentAnimation",[Se(":enter",[ke(ci)]),Se(":leave",[ke(pi)])])]},changeDetection:0})}return t})(),go=(()=>{class t{static \u0275fac=function(n){return new(n||t)};static \u0275mod=re({type:t});static \u0275inj=le({imports:[Me,E,E]})}return t})()});function ui(t,s){t&1&&F(0,"CheckIcon",5)}function hi(t,s){t&1&&F(0,"BlankIcon",6)}function mi(t,s){if(t&1&&(w(0),p(1,ui,1,0,"CheckIcon",3)(2,hi,1,0,"BlankIcon",4),S()),t&2){let e=a();c(),r("ngIf",e.selected),c(),r("ngIf",!e.selected)}}function _i(t,s){if(t&1&&(h(0,"span"),$(1),m()),t&2){let e,n=a();c(),q((e=n.label)!==null&&e!==void 0?e:"empty")}}function fi(t,s){t&1&&C(0)}function Pi(t,s){if(t&1&&(w(0),$(1),S()),t&2){let e=a(2);c(),q(e.label()==="p-emptylabel"?"\xA0":e.label())}}function Bi(t,s){if(t&1&&C(0,24),t&2){let e=a(2);r("ngTemplateOutlet",e.selectedItemTemplate||e._selectedItemTemplate)("ngTemplateOutletContext",O(2,De,e.selectedOption))}}function qi(t,s){if(t&1&&(h(0,"span"),$(1),m()),t&2){let e=a(3);c(),q(e.label()==="p-emptylabel"?"\xA0":e.label())}}function Zi(t,s){if(t&1&&p(0,qi,2,1,"span",18),t&2){let e=a(2);r("ngIf",e.isSelectedOptionEmpty())}}function Gi(t,s){if(t&1){let e=I();h(0,"span",22,3),v("focus",function(i){_(e);let o=a();return f(o.onInputFocus(i))})("blur",function(i){_(e);let o=a();return f(o.onInputBlur(i))})("keydown",function(i){_(e);let o=a();return f(o.onKeyDown(i))}),p(2,Pi,2,1,"ng-container",20)(3,Bi,1,4,"ng-container",23)(4,Zi,1,1,"ng-template",null,4,L),m()}if(t&2){let e,n=A(5),i=a();r("ngClass",i.inputClass)("pTooltip",i.tooltip)("tooltipPosition",i.tooltipPosition)("positionStyle",i.tooltipPositionStyle)("tooltipStyleClass",i.tooltipStyleClass)("pAutoFocus",i.autofocus),T("aria-disabled",i.disabled)("id",i.inputId)("aria-label",i.ariaLabel||(i.label()==="p-emptylabel"?void 0:i.label()))("aria-labelledby",i.ariaLabelledBy)("aria-haspopup","listbox")("aria-expanded",(e=i.overlayVisible)!==null&&e!==void 0?e:!1)("aria-controls",i.overlayVisible?i.id+"_list":null)("tabindex",i.disabled?-1:i.tabindex)("aria-activedescendant",i.focused?i.focusedOptionId:void 0)("aria-required",i.required)("required",i.required),c(2),r("ngIf",!i.selectedItemTemplate&&!i._selectedItemTemplate)("ngIfElse",n),c(),r("ngIf",(i.selectedItemTemplate||i._selectedItemTemplate)&&!i.isSelectedOptionEmpty())}}function Ni(t,s){if(t&1){let e=I();h(0,"input",25,5),v("input",function(i){_(e);let o=a();return f(o.onEditableInput(i))})("keydown",function(i){_(e);let o=a();return f(o.onKeyDown(i))})("focus",function(i){_(e);let o=a();return f(o.onInputFocus(i))})("blur",function(i){_(e);let o=a();return f(o.onInputBlur(i))}),m()}if(t&2){let e=a();r("ngClass",e.inputClass)("disabled",e.disabled)("pAutoFocus",e.autofocus),T("id",e.inputId)("maxlength",e.maxlength)("placeholder",e.modelValue()===void 0||e.modelValue()===null?e.placeholder():void 0)("aria-label",e.ariaLabel||(e.label()==="p-emptylabel"?void 0:e.label()))("aria-activedescendant",e.focused?e.focusedOptionId:void 0)}}function ji(t,s){if(t&1){let e=I();h(0,"TimesIcon",27),v("click",function(i){_(e);let o=a(2);return f(o.clear(i))}),m()}t&2&&T("data-pc-section","clearicon")}function Ui(t,s){}function Wi(t,s){t&1&&p(0,Ui,0,0,"ng-template")}function Yi(t,s){if(t&1){let e=I();h(0,"span",27),v("click",function(i){_(e);let o=a(2);return f(o.clear(i))}),p(1,Wi,1,0,null,28),m()}if(t&2){let e=a(2);T("data-pc-section","clearicon"),c(),r("ngTemplateOutlet",e.clearIconTemplate||e._clearIconTemplate)("ngTemplateOutletContext",de(3,Hi))}}function Ji(t,s){if(t&1&&(w(0),p(1,ji,1,1,"TimesIcon",26)(2,Yi,2,4,"span",26),S()),t&2){let e=a();c(),r("ngIf",!e.clearIconTemplate&&!e._clearIconTemplate),c(),r("ngIf",e.clearIconTemplate||e._clearIconTemplate)}}function Xi(t,s){t&1&&C(0)}function en(t,s){if(t&1&&(w(0),p(1,Xi,1,0,"ng-container",29),S()),t&2){let e=a(2);c(),r("ngTemplateOutlet",e.loadingIconTemplate||e._loadingIconTemplate)}}function tn(t,s){if(t&1&&F(0,"span",32),t&2){let e=a(3);r("ngClass","p-select-loading-icon pi-spin "+e.loadingIcon)}}function nn(t,s){t&1&&F(0,"span",33),t&2&&z("p-select-loading-icon pi pi-spinner pi-spin")}function on(t,s){if(t&1&&(w(0),p(1,tn,1,1,"span",30)(2,nn,1,2,"span",31),S()),t&2){let e=a(2);c(),r("ngIf",e.loadingIcon),c(),r("ngIf",!e.loadingIcon)}}function ln(t,s){if(t&1&&(w(0),p(1,en,2,1,"ng-container",18)(2,on,3,2,"ng-container",18),S()),t&2){let e=a();c(),r("ngIf",e.loadingIconTemplate||e._loadingIconTemplate),c(),r("ngIf",!e.loadingIconTemplate&&!e._loadingIconTemplate)}}function sn(t,s){if(t&1&&F(0,"span",37),t&2){let e=a(3);r("ngClass",e.dropdownIcon)}}function an(t,s){t&1&&F(0,"ChevronDownIcon",38),t&2&&r("styleClass","p-select-dropdown-icon")}function rn(t,s){if(t&1&&(w(0),p(1,sn,1,1,"span",35)(2,an,1,1,"ChevronDownIcon",36),S()),t&2){let e=a(2);c(),r("ngIf",e.dropdownIcon),c(),r("ngIf",!e.dropdownIcon)}}function cn(t,s){}function pn(t,s){t&1&&p(0,cn,0,0,"ng-template")}function dn(t,s){if(t&1&&(h(0,"span",39),p(1,pn,1,0,null,28),m()),t&2){let e=a(2);c(),r("ngTemplateOutlet",e.dropdownIconTemplate||e._dropdownIconTemplate)("ngTemplateOutletContext",de(2,Qi))}}function un(t,s){if(t&1&&p(0,rn,3,2,"ng-container",18)(1,dn,2,3,"span",34),t&2){let e=a();r("ngIf",!e.dropdownIconTemplate&&!e._dropdownIconTemplate),c(),r("ngIf",e.dropdownIconTemplate||e._dropdownIconTemplate)}}function hn(t,s){t&1&&C(0)}function mn(t,s){t&1&&C(0)}function _n(t,s){if(t&1&&(w(0),p(1,mn,1,0,"ng-container",28),S()),t&2){let e=a(3);c(),r("ngTemplateOutlet",e.filterTemplate||e._filterTemplate)("ngTemplateOutletContext",O(2,Ft,e.filterOptions))}}function fn(t,s){t&1&&F(0,"SearchIcon")}function gn(t,s){}function yn(t,s){t&1&&p(0,gn,0,0,"ng-template")}function bn(t,s){if(t&1&&(h(0,"span"),p(1,yn,1,0,null,29),m()),t&2){let e=a(4);c(),r("ngTemplateOutlet",e.filterIconTemplate||e._filterIconTemplate)}}function vn(t,s){if(t&1){let e=I();h(0,"p-iconfield")(1,"input",46,10),v("input",function(i){_(e);let o=a(3);return f(o.onFilterInputChange(i))})("keydown",function(i){_(e);let o=a(3);return f(o.onFilterKeyDown(i))})("blur",function(i){_(e);let o=a(3);return f(o.onFilterBlur(i))}),m(),h(3,"p-inputicon"),p(4,fn,1,0,"SearchIcon",18)(5,bn,2,1,"span",18),m()()}if(t&2){let e=a(3);c(),r("pSize",e.size)("value",e._filterValue()||"")("variant",e.variant),T("placeholder",e.filterPlaceholder)("aria-owns",e.id+"_list")("aria-label",e.ariaFilterLabel)("aria-activedescendant",e.focusedOptionId),c(3),r("ngIf",!e.filterIconTemplate&&!e._filterIconTemplate),c(),r("ngIf",e.filterIconTemplate||e._filterIconTemplate)}}function xn(t,s){if(t&1){let e=I();h(0,"div",45),v("click",function(i){return _(e),f(i.stopPropagation())}),p(1,_n,2,4,"ng-container",20)(2,vn,6,9,"ng-template",null,9,L),m()}if(t&2){let e=A(3),n=a(2);c(),r("ngIf",n.filterTemplate||n._filterTemplate)("ngIfElse",e)}}function Cn(t,s){t&1&&C(0)}function On(t,s){if(t&1&&p(0,Cn,1,0,"ng-container",28),t&2){let e=s.$implicit,n=s.options;a(2);let i=A(9);r("ngTemplateOutlet",i)("ngTemplateOutletContext",Ie(2,Lt,e,n))}}function In(t,s){t&1&&C(0)}function Tn(t,s){if(t&1&&p(0,In,1,0,"ng-container",28),t&2){let e=s.options,n=a(4);r("ngTemplateOutlet",n.loaderTemplate||n._loaderTemplate)("ngTemplateOutletContext",O(2,Ft,e))}}function wn(t,s){t&1&&(w(0),p(1,Tn,1,4,"ng-template",null,12,L),S())}function Sn(t,s){if(t&1){let e=I();h(0,"p-scroller",47,11),v("onLazyLoad",function(i){_(e);let o=a(2);return f(o.onLazyLoad.emit(i))}),p(2,On,1,5,"ng-template",null,2,L)(4,wn,3,0,"ng-container",18),m()}if(t&2){let e=a(2);ce(O(8,ie,e.scrollHeight)),r("items",e.visibleOptions())("itemSize",e.virtualScrollItemSize||e._itemSize)("autoSize",!0)("lazy",e.lazy)("options",e.virtualScrollOptions),c(4),r("ngIf",e.loaderTemplate||e._loaderTemplate)}}function Vn(t,s){t&1&&C(0)}function kn(t,s){if(t&1&&(w(0),p(1,Vn,1,0,"ng-container",28),S()),t&2){a();let e=A(9),n=a();c(),r("ngTemplateOutlet",e)("ngTemplateOutletContext",Ie(3,Lt,n.visibleOptions(),de(2,Ki)))}}function En(t,s){if(t&1&&(h(0,"span"),$(1),m()),t&2){let e=a(2).$implicit,n=a(3);c(),q(n.getOptionGroupLabel(e.optionGroup))}}function Fn(t,s){t&1&&C(0)}function Ln(t,s){if(t&1&&(w(0),h(1,"li",51),p(2,En,2,1,"span",18)(3,Fn,1,0,"ng-container",28),m(),S()),t&2){let e=a(),n=e.$implicit,i=e.index,o=a().options,l=a(2);c(),r("ngStyle",O(5,ie,o.itemSize+"px")),T("id",l.id+"_"+l.getOptionIndex(i,o)),c(),r("ngIf",!l.groupTemplate&&!l._groupTemplate),c(),r("ngTemplateOutlet",l.groupTemplate||l._groupTemplate)("ngTemplateOutletContext",O(7,De,n.optionGroup))}}function Mn(t,s){if(t&1){let e=I();w(0),h(1,"p-selectItem",52),v("onClick",function(i){_(e);let o=a().$implicit,l=a(3);return f(l.onOptionSelect(i,o))})("onMouseEnter",function(i){_(e);let o=a().index,l=a().options,x=a(2);return f(x.onOptionMouseEnter(i,x.getOptionIndex(o,l)))}),m(),S()}if(t&2){let e=a(),n=e.$implicit,i=e.index,o=a().options,l=a(2);c(),r("id",l.id+"_"+l.getOptionIndex(i,o))("option",n)("checkmark",l.checkmark)("selected",l.isSelected(n))("label",l.getOptionLabel(n))("disabled",l.isOptionDisabled(n))("template",l.itemTemplate||l._itemTemplate)("focused",l.focusedOptionIndex()===l.getOptionIndex(i,o))("ariaPosInset",l.getAriaPosInset(l.getOptionIndex(i,o)))("ariaSetSize",l.ariaSetSize)}}function Dn(t,s){if(t&1&&p(0,Ln,4,9,"ng-container",18)(1,Mn,2,10,"ng-container",18),t&2){let e=s.$implicit,n=a(3);r("ngIf",n.isOptionGroup(e)),c(),r("ngIf",!n.isOptionGroup(e))}}function zn(t,s){if(t&1&&$(0),t&2){let e=a(4);Oe(" ",e.emptyFilterMessageLabel," ")}}function $n(t,s){t&1&&C(0,null,14)}function Rn(t,s){if(t&1&&p(0,$n,2,0,"ng-container",29),t&2){let e=a(4);r("ngTemplateOutlet",e.emptyFilterTemplate||e._emptyFilterTemplate||e.emptyTemplate||e._emptyTemplate)}}function An(t,s){if(t&1&&(h(0,"li",53),p(1,zn,1,1)(2,Rn,1,1,"ng-container"),m()),t&2){let e=a().options,n=a(2);r("ngStyle",O(2,ie,e.itemSize+"px")),c(),Ce(!n.emptyFilterTemplate&&!n._emptyFilterTemplate&&!n.emptyTemplate?1:2)}}function Hn(t,s){if(t&1&&$(0),t&2){let e=a(4);Oe(" ",e.emptyMessageLabel," ")}}function Qn(t,s){t&1&&C(0,null,15)}function Kn(t,s){if(t&1&&p(0,Qn,2,0,"ng-container",29),t&2){let e=a(4);r("ngTemplateOutlet",e.emptyTemplate||e._emptyTemplate)}}function Pn(t,s){if(t&1&&(h(0,"li",53),p(1,Hn,1,1)(2,Kn,1,1,"ng-container"),m()),t&2){let e=a().options,n=a(2);r("ngStyle",O(2,ie,e.itemSize+"px")),c(),Ce(!n.emptyTemplate&&!n._emptyTemplate?1:2)}}function Bn(t,s){if(t&1&&(h(0,"ul",48,13),p(2,Dn,2,2,"ng-template",49)(3,An,3,4,"li",50)(4,Pn,3,4,"li",50),m()),t&2){let e=s.$implicit,n=s.options,i=a(2);ce(n.contentStyle),r("ngClass",n.contentStyleClass),T("id",i.id+"_list")("aria-label",i.listLabel),c(2),r("ngForOf",e),c(),r("ngIf",i.filterValue&&i.isEmpty()),c(),r("ngIf",!i.filterValue&&i.isEmpty())}}function qn(t,s){t&1&&C(0)}function Zn(t,s){if(t&1){let e=I();h(0,"div",40)(1,"span",41,6),v("focus",function(i){_(e);let o=a();return f(o.onFirstHiddenFocus(i))}),m(),p(3,hn,1,0,"ng-container",29)(4,xn,4,2,"div",42),h(5,"div",43),p(6,Sn,5,10,"p-scroller",44)(7,kn,2,6,"ng-container",18)(8,Bn,5,8,"ng-template",null,7,L),m(),p(10,qn,1,0,"ng-container",29),h(11,"span",41,8),v("focus",function(i){_(e);let o=a();return f(o.onLastHiddenFocus(i))}),m()()}if(t&2){let e=a();z(e.panelStyleClass),r("ngClass","p-select-overlay p-component")("ngStyle",e.panelStyle),c(),T("tabindex",0)("data-p-hidden-accessible",!0)("data-p-hidden-focusable",!0),c(2),r("ngTemplateOutlet",e.headerTemplate||e._headerTemplate),c(),r("ngIf",e.filter),c(),He("max-height",e.virtualScroll?"auto":e.scrollHeight||"auto"),c(),r("ngIf",e.virtualScroll),c(),r("ngIf",!e.virtualScroll),c(3),r("ngTemplateOutlet",e.footerTemplate||e._footerTemplate),c(),T("tabindex",0)("data-p-hidden-accessible",!0)("data-p-hidden-focusable",!0)}}var ie,di,De,gi,yi,bi,vi,xi,kt,Ci,Oi,Ii,Ti,wi,Si,Vi,ki,Ei,Fi,Li,Mi,Di,zi,$i,Ri,Ai,Hi,Qi,Ft,Lt,Ki,Gn,Nn,Et,jn,Un,Mt,No,Wn=Re(()=>{me();me();he();he();At();rt();ye();ye();Qt();ut();Gt();Kt();Nt();Zt();Vt();Pt();jt();qt();dt();ie=t=>({height:t}),di=(t,s,e)=>({"p-select-option":!0,"p-select-option-selected":t,"p-disabled":s,"p-focus":e}),De=t=>({$implicit:t});gi=["item"],yi=["group"],bi=["loader"],vi=["selectedItem"],xi=["header"],kt=["filter"],Ci=["footer"],Oi=["emptyfilter"],Ii=["empty"],Ti=["dropdownicon"],wi=["loadingicon"],Si=["clearicon"],Vi=["filtericon"],ki=["onicon"],Ei=["officon"],Fi=["cancelicon"],Li=["focusInput"],Mi=["editableInput"],Di=["items"],zi=["scroller"],$i=["overlay"],Ri=["firstHiddenFocusableEl"],Ai=["lastHiddenFocusableEl"],Hi=()=>({class:"p-select-clear-icon"}),Qi=()=>({class:"p-select-dropdown-icon"}),Ft=t=>({options:t}),Lt=(t,s)=>({$implicit:t,options:s}),Ki=()=>({});Gn=({dt:t})=>`
.p-select {
    display: inline-flex;
    cursor: pointer;
    position: relative;
    user-select: none;
    background: ${t("select.background")};
    border: 1px solid ${t("select.border.color")};
    transition: background ${t("select.transition.duration")}, color ${t("select.transition.duration")}, border-color ${t("select.transition.duration")},
        outline-color ${t("select.transition.duration")}, box-shadow ${t("select.transition.duration")};
    border-radius: ${t("select.border.radius")};
    outline-color: transparent;
    box-shadow: ${t("select.shadow")};
}

.p-select.ng-invalid.ng-dirty {
    border-color: ${t("select.invalid.border.color")};
}

.p-select:not(.p-disabled):hover {
    border-color: ${t("select.hover.border.color")};
}

.p-select:not(.p-disabled).p-focus {
    border-color: ${t("select.focus.border.color")};
    box-shadow: ${t("select.focus.ring.shadow")};
    outline: ${t("select.focus.ring.width")} ${t("select.focus.ring.style")} ${t("select.focus.ring.color")};
    outline-offset: ${t("select.focus.ring.offset")};
}

.p-select.p-variant-filled {
    background: ${t("select.filled.background")};
}

.p-select.p-variant-filled:not(.p-disabled):hover {
    background: ${t("select.filled.hover.background")};
}

.p-select.p-variant-filled.p-focus {
    background: ${t("select.filled.focus.background")};
}

.p-select.p-disabled {
    opacity: 1;
    background: ${t("select.disabled.background")};
}

.p-select-clear-icon {
    position: absolute;
    top: 50%;
    margin-top: -0.5rem;
    color: ${t("select.clear.icon.color")};
    inset-inline-end: ${t("select.dropdown.width")};
}

.p-select-dropdown {
    display: flex;
    align-items: center;
    justify-content: center;
    flex-shrink: 0;
    background: transparent;
    color: ${t("select.dropdown.color")};
    width: ${t("select.dropdown.width")};
    border-start-end-radius: ${t("select.border.radius")};
    border-end-end-radius: ${t("select.border.radius")};
}

.p-select-label {
    display: block;
    white-space: nowrap;
    overflow: hidden;
    flex: 1 1 auto;
    width: 1%;
    padding: ${t("select.padding.y")} ${t("select.padding.x")};
    text-overflow: ellipsis;
    cursor: pointer;
    color: ${t("select.color")};
    background: transparent;
    border: 0 none;
    outline: 0 none;
}

.p-select-label.p-placeholder {
    color: ${t("select.placeholder.color")};
}

.p-select.ng-invalid.ng-dirty .p-select-label.p-placeholder {
    color: ${t("select.invalid.placeholder.color")};
}

.p-select:has(.p-select-clear-icon) .p-select-label {
    padding-inline-end: calc(1rem + ${t("select.padding.x")});
}

.p-select.p-disabled .p-select-label {
    color: ${t("select.disabled.color")};
}

.p-select-label-empty {
    overflow: hidden;
    opacity: 0;
}

input.p-select-label {
    cursor: default;
}

.p-select .p-select-overlay {
    min-width: 100%;
}

.p-select-overlay {
    cursor: default;
    background: ${t("select.overlay.background")};
    color: ${t("select.overlay.color")};
    border: 1px solid ${t("select.overlay.border.color")};
    border-radius: ${t("select.overlay.border.radius")};
    box-shadow: ${t("select.overlay.shadow")};
}

.p-select-header {
    padding: ${t("select.list.header.padding")};
}

.p-select-filter {
    width: 100%;
}

.p-select-list-container {
    overflow: auto;
}

.p-select-option-group {
    cursor: auto;
    margin: 0;
    padding: ${t("select.option.group.padding")};
    background: ${t("select.option.group.background")};
    color: ${t("select.option.group.color")};
    font-weight: ${t("select.option.group.font.weight")};
}

.p-select-list {
    margin: 0;
    padding: 0;
    list-style-type: none;
    padding: ${t("select.list.padding")};
    gap: ${t("select.list.gap")};
    display: flex;
    flex-direction: column;
}

.p-select-option {
    cursor: pointer;
    font-weight: normal;
    white-space: nowrap;
    position: relative;
    overflow: hidden;
    display: flex;
    align-items: center;
    padding: ${t("select.option.padding")};
    border: 0 none;
    color: ${t("select.option.color")};
    background: transparent;
    transition: background ${t("select.transition.duration")}, color ${t("select.transition.duration")}, border-color ${t("select.transition.duration")},
            box-shadow ${t("select.transition.duration")}, outline-color ${t("select.transition.duration")};
    border-radius: ${t("select.option.border.radius")};
}

.p-select-option:not(.p-select-option-selected):not(.p-disabled).p-focus {
    background: ${t("select.option.focus.background")};
    color: ${t("select.option.focus.color")};
}

.p-select-option.p-select-option-selected {
    background: ${t("select.option.selected.background")};
    color: ${t("select.option.selected.color")};
}

.p-select-option.p-select-option-selected.p-focus {
    background: ${t("select.option.selected.focus.background")};
    color: ${t("select.option.selected.focus.color")};
}

.p-select-option-check-icon {
    position: relative;
    margin-inline-start: ${t("select.checkmark.gutter.start")};
    margin-inline-end: ${t("select.checkmark.gutter.end")};
    color: ${t("select.checkmark.color")};
}

.p-select-empty-message {
    padding: ${t("select.empty.message.padding")};
}

.p-select-fluid {
    display: flex;
}

.p-select-sm .p-select-label {
    font-size: ${t("select.sm.font.size")};
    padding-block: ${t("select.sm.padding.y")};
    padding-inline: ${t("select.sm.padding.x")};
}

.p-select-sm .p-select-dropdown .p-icon {
    font-size: ${t("select.sm.font.size")};
    width: ${t("select.sm.font.size")};
    height: ${t("select.sm.font.size")};
}

.p-select-lg .p-select-label {
    font-size: ${t("select.lg.font.size")};
    padding-block: ${t("select.lg.padding.y")};
    padding-inline: ${t("select.lg.padding.x")};
}

.p-select-lg .p-select-dropdown .p-icon {
    font-size: ${t("select.lg.font.size")};
    width: ${t("select.lg.font.size")};
    height: ${t("select.lg.font.size")};
}
`,Nn={root:({instance:t})=>["p-select p-component p-inputwrapper",{"p-disabled":t.disabled,"p-variant-filled":t.variant==="filled"||t.config.inputVariant()==="filled"||t.config.inputStyle()==="filled","p-focus":t.focused,"p-inputwrapper-filled":t.modelValue()!==void 0&&t.modelValue()!==null,"p-inputwrapper-focus":t.focused||t.overlayVisible,"p-select-open":t.overlayVisible,"p-select-fluid":t.hasFluid,"p-select-sm p-inputfield-sm":t.size==="small","p-select-lg p-inputfield-lg":t.size==="large"}],label:({instance:t,props:s})=>["p-select-label",{"p-placeholder":!s.editable&&t.label===s.placeholder,"p-select-label-empty":!s.editable&&!t.$slots.value&&(t.label==="p-emptylabel"||t.label.length===0)}],clearIcon:"p-select-clear-icon",dropdown:"p-select-dropdown",loadingicon:"p-select-loading-icon",dropdownIcon:"p-select-dropdown-icon",overlay:"p-select-overlay p-component",header:"p-select-header",pcFilter:"p-select-filter",listContainer:"p-select-list-container",list:"p-select-list",optionGroup:"p-select-option-group",optionGroupLabel:"p-select-option-group-label",option:({instance:t,props:s,state:e,option:n,focusedOption:i})=>["p-select-option",{"p-select-option-selected":t.isSelected(n)&&s.highlightOnSelect,"p-focus":e.focusedOptionIndex===i,"p-disabled":t.isOptionDisabled(n)}],optionLabel:"p-select-option-label",optionCheckIcon:"p-select-option-check-icon",optionBlankIcon:"p-select-option-blank-icon",emptyMessage:"p-select-empty-message"},Et=(()=>{class t extends be{name="select";theme=Gn;classes=Nn;static \u0275fac=(()=>{let e;return function(i){return(e||(e=K(t)))(i||t)}})();static \u0275prov=oe({token:t,factory:t.\u0275fac})}return t})(),jn={provide:We,useExisting:Ae(()=>Mt),multi:!0},Un=(()=>{class t extends X{id;option;selected;focused;label;disabled;visible;itemSize;ariaPosInset;ariaSetSize;template;checkmark;onClick=new b;onMouseEnter=new b;onOptionClick(e){this.onClick.emit(e)}onOptionMouseEnter(e){this.onMouseEnter.emit(e)}static \u0275fac=(()=>{let e;return function(i){return(e||(e=K(t)))(i||t)}})();static \u0275cmp=P({type:t,selectors:[["p-selectItem"]],inputs:{id:"id",option:"option",selected:[2,"selected","selected",g],focused:[2,"focused","focused",g],label:"label",disabled:[2,"disabled","disabled",g],visible:[2,"visible","visible",g],itemSize:[2,"itemSize","itemSize",Z],ariaPosInset:"ariaPosInset",ariaSetSize:"ariaSetSize",template:"template",checkmark:[2,"checkmark","checkmark",g]},outputs:{onClick:"onClick",onMouseEnter:"onMouseEnter"},features:[B],decls:4,vars:22,consts:[["role","option","pRipple","",3,"click","mouseenter","id","ngStyle","ngClass"],[4,"ngIf"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],["styleClass","p-select-option-check-icon",4,"ngIf"],["styleClass","p-select-option-blank-icon",4,"ngIf"],["styleClass","p-select-option-check-icon"],["styleClass","p-select-option-blank-icon"]],template:function(n,i){n&1&&(h(0,"li",0),v("click",function(l){return i.onOptionClick(l)})("mouseenter",function(l){return i.onOptionMouseEnter(l)}),p(1,mi,3,2,"ng-container",1)(2,_i,2,1,"span",1)(3,fi,1,0,"ng-container",2),m()),n&2&&(r("id",i.id)("ngStyle",O(14,ie,i.itemSize+"px"))("ngClass",ue(16,di,i.selected&&!i.checkmark,i.disabled,i.focused)),T("aria-label",i.label)("aria-setsize",i.ariaSetSize)("aria-posinset",i.ariaPosInset)("aria-selected",i.selected)("data-p-focused",i.focused)("data-p-highlight",i.selected)("data-p-disabled",i.disabled),c(),r("ngIf",i.checkmark),c(),r("ngIf",!i.template),c(),r("ngTemplateOutlet",i.template)("ngTemplateOutletContext",O(20,De,i.option)))},dependencies:[Y,N,j,W,U,E,vt,ft,_t],encapsulation:2})}return t})(),Mt=(()=>{class t extends X{zone;filterService;id;scrollHeight="200px";filter;name;style;panelStyle;styleClass;panelStyleClass;readonly;required;editable;appendTo;tabindex=0;set placeholder(e){this._placeholder.set(e)}get placeholder(){return this._placeholder.asReadonly()}loadingIcon;filterPlaceholder;filterLocale;variant;inputId;dataKey;filterBy;filterFields;autofocus;resetFilterOnHide=!1;checkmark=!1;dropdownIcon;loading=!1;optionLabel;optionValue;optionDisabled;optionGroupLabel="label";optionGroupChildren="items";autoDisplayFirst=!0;group;showClear;emptyFilterMessage="";emptyMessage="";lazy=!1;virtualScroll;virtualScrollItemSize;virtualScrollOptions;size;overlayOptions;ariaFilterLabel;ariaLabel;ariaLabelledBy;filterMatchMode="contains";maxlength;tooltip="";tooltipPosition="right";tooltipPositionStyle="absolute";tooltipStyleClass;focusOnHover=!0;selectOnFocus=!1;autoOptionFocus=!1;autofocusFilter=!0;fluid;get disabled(){return this._disabled}set disabled(e){e&&(this.focused=!1,this.overlayVisible&&this.hide()),this._disabled=e,this.cd.destroyed||this.cd.detectChanges()}get itemSize(){return this._itemSize}set itemSize(e){this._itemSize=e,console.log("The itemSize property is deprecated, use virtualScrollItemSize property instead.")}_itemSize;get autoZIndex(){return this._autoZIndex}set autoZIndex(e){this._autoZIndex=e,console.log("The autoZIndex property is deprecated since v14.2.0, use overlayOptions property instead.")}_autoZIndex;get baseZIndex(){return this._baseZIndex}set baseZIndex(e){this._baseZIndex=e,console.log("The baseZIndex property is deprecated since v14.2.0, use overlayOptions property instead.")}_baseZIndex;get showTransitionOptions(){return this._showTransitionOptions}set showTransitionOptions(e){this._showTransitionOptions=e,console.log("The showTransitionOptions property is deprecated since v14.2.0, use overlayOptions property instead.")}_showTransitionOptions;get hideTransitionOptions(){return this._hideTransitionOptions}set hideTransitionOptions(e){this._hideTransitionOptions=e,console.log("The hideTransitionOptions property is deprecated since v14.2.0, use overlayOptions property instead.")}_hideTransitionOptions;get filterValue(){return this._filterValue()}set filterValue(e){setTimeout(()=>{this._filterValue.set(e)})}get options(){return this._options()}set options(e){ot(e,this._options())||this._options.set(e)}onChange=new b;onFilter=new b;onFocus=new b;onBlur=new b;onClick=new b;onShow=new b;onHide=new b;onClear=new b;onLazyLoad=new b;_componentStyle=se(Et);filterViewChild;focusInputViewChild;editableInputViewChild;itemsViewChild;scroller;overlayViewChild;firstHiddenFocusableElementOnOverlay;lastHiddenFocusableElementOnOverlay;get hasFluid(){let n=this.el.nativeElement.closest("p-fluid");return this.fluid||!!n}get hostClass(){return this._componentStyle.classes.root({instance:this}).map(n=>typeof n=="string"?n:Object.keys(n).filter(i=>n[i]).join(" ")).join(" ")+" "+this.styleClass}get hostStyle(){return this.style}_disabled;itemsWrapper;itemTemplate;groupTemplate;loaderTemplate;selectedItemTemplate;headerTemplate;filterTemplate;footerTemplate;emptyFilterTemplate;emptyTemplate;dropdownIconTemplate;loadingIconTemplate;clearIconTemplate;filterIconTemplate;onIconTemplate;offIconTemplate;cancelIconTemplate;templates;_itemTemplate;_selectedItemTemplate;_headerTemplate;_filterTemplate;_footerTemplate;_emptyFilterTemplate;_emptyTemplate;_groupTemplate;_loaderTemplate;_dropdownIconTemplate;_loadingIconTemplate;_clearIconTemplate;_filterIconTemplate;_cancelIconTemplate;_onIconTemplate;_offIconTemplate;filterOptions;_options=D(null);_placeholder=D(void 0);modelValue=D(null);value;onModelChange=()=>{};onModelTouched=()=>{};hover;focused;overlayVisible;optionsChanged;panel;dimensionsUpdated;hoveredItem;selectedOptionUpdated;_filterValue=D(null);searchValue;searchIndex;searchTimeout;previousSearchChar;currentSearchChar;preventModelTouched;focusedOptionIndex=D(-1);labelId;listId;clicked=D(!1);get emptyMessageLabel(){return this.emptyMessage||this.config.getTranslation(ge.EMPTY_MESSAGE)}get emptyFilterMessageLabel(){return this.emptyFilterMessage||this.config.getTranslation(ge.EMPTY_FILTER_MESSAGE)}get isVisibleClearIcon(){return this.modelValue()!=null&&this.hasSelectedOption()&&this.showClear&&!this.disabled}get listLabel(){return this.config.getTranslation(ge.ARIA).listLabel}get rootClass(){return this._componentStyle.classes.root({instance:this})}get inputClass(){let e=this.label();return{"p-select-label":!0,"p-placeholder":this.placeholder()&&e===this.placeholder(),"p-select-label-empty":!this.editable&&!this.selectedItemTemplate&&(e==null||e==="p-emptylabel"||e.length===0)}}get panelClass(){return{"p-dropdown-panel p-component":!0,"p-input-filled":this.config.inputStyle()==="filled"||this.config.inputVariant()==="filled","p-ripple-disabled":this.config.ripple()===!1}}get focusedOptionId(){return this.focusedOptionIndex()!==-1?`${this.id}_${this.focusedOptionIndex()}`:null}visibleOptions=G(()=>{let e=this.getAllVisibleAndNonVisibleOptions();if(this._filterValue()){let i=!(this.filterBy||this.optionLabel)&&!this.filterFields&&!this.optionValue?this.options.filter(o=>o.label?o.label.toString().toLowerCase().indexOf(this._filterValue().toLowerCase().trim())!==-1:o.toString().toLowerCase().indexOf(this._filterValue().toLowerCase().trim())!==-1):this.filterService.filter(e,this.searchFields(),this._filterValue().trim(),this.filterMatchMode,this.filterLocale);if(this.group){let o=this.options||[],l=[];return o.forEach(x=>{let ne=this.getOptionGroupChildren(x).filter(xe=>i.includes(xe));ne.length>0&&l.push($e(M({},x),{[typeof this.optionGroupChildren=="string"?this.optionGroupChildren:"items"]:[...ne]}))}),this.flatOptions(l)}return i}return e});label=G(()=>{let e=this.getAllVisibleAndNonVisibleOptions(),n=e.findIndex(i=>this.isOptionValueEqualsModelValue(i));return n!==-1?this.getOptionLabel(e[n]):this.placeholder()||"p-emptylabel"});filled=G(()=>typeof this.modelValue()=="string"?!!this.modelValue():this.label()!=="p-emptylabel"&&this.modelValue()!==void 0&&this.modelValue()!==null);selectedOption;editableInputValue=G(()=>this.getOptionLabel(this.selectedOption)||this.modelValue()||"");constructor(e,n){super(),this.zone=e,this.filterService=n,Ge(()=>{let i=this.modelValue(),o=this.visibleOptions();if(o&&_e(o)){let l=this.findSelectedOptionIndex();(l!==-1||i===void 0||typeof i=="string"&&i.length===0||this.isModelValueNotSet()||this.editable)&&(this.selectedOption=o[l])}Fe(o)&&(i===void 0||this.isModelValueNotSet())&&_e(this.selectedOption)&&(this.selectedOption=null),i!==void 0&&this.editable&&this.updateEditableLabel(),this.cd.markForCheck()})}isModelValueNotSet(){return this.modelValue()===null&&!this.isOptionValueEqualsModelValue(this.selectedOption)}getAllVisibleAndNonVisibleOptions(){return this.group?this.flatOptions(this.options):this.options||[]}ngOnInit(){super.ngOnInit(),this.id=this.id||at("pn_id_"),this.autoUpdateModel(),this.filterBy&&(this.filterOptions={filter:e=>this.onFilterInputChange(e),reset:()=>this.resetFilter()})}ngAfterContentInit(){this.templates.forEach(e=>{switch(e.getType()){case"item":this._itemTemplate=e.template;break;case"selectedItem":this._selectedItemTemplate=e.template;break;case"header":this._headerTemplate=e.template;break;case"filter":this._filterTemplate=e.template;break;case"footer":this._footerTemplate=e.template;break;case"emptyfilter":this._emptyFilterTemplate=e.template;break;case"empty":this._emptyTemplate=e.template;break;case"group":this._groupTemplate=e.template;break;case"loader":this._loaderTemplate=e.template;break;case"dropdownicon":this._dropdownIconTemplate=e.template;break;case"loadingicon":this._loadingIconTemplate=e.template;break;case"clearicon":this._clearIconTemplate=e.template;break;case"filtericon":this._filterIconTemplate=e.template;break;case"cancelicon":this._cancelIconTemplate=e.template;break;case"onicon":this._onIconTemplate=e.template;break;case"officon":this._offIconTemplate=e.template;break;default:this._itemTemplate=e.template;break}})}ngAfterViewChecked(){if(this.optionsChanged&&this.overlayVisible&&(this.optionsChanged=!1,this.zone.runOutsideAngular(()=>{setTimeout(()=>{this.overlayViewChild&&this.overlayViewChild.alignOverlay()},1)})),this.selectedOptionUpdated&&this.itemsWrapper){let e=H(this.overlayViewChild?.overlayViewChild?.nativeElement,"li.p-select-option-selected");e&&nt(this.itemsWrapper,e),this.selectedOptionUpdated=!1}}flatOptions(e){return(e||[]).reduce((n,i,o)=>{n.push({optionGroup:i,group:!0,index:o});let l=this.getOptionGroupChildren(i);return l&&l.forEach(x=>n.push(x)),n},[])}autoUpdateModel(){if(this.selectOnFocus&&this.autoOptionFocus&&!this.hasSelectedOption()&&(this.focusedOptionIndex.set(this.findFirstFocusedOptionIndex()),this.onOptionSelect(null,this.visibleOptions()[this.focusedOptionIndex()],!1)),this.autoDisplayFirst&&(this.modelValue()===null||this.modelValue()===void 0)&&!this.placeholder()){let e=this.findFirstOptionIndex();this.onOptionSelect(null,this.visibleOptions()[e],!1,!0)}}onOptionSelect(e,n,i=!0,o=!1){if(!this.isSelected(n)){let l=this.getOptionValue(n);this.updateModel(l,e),this.focusedOptionIndex.set(this.findSelectedOptionIndex()),o===!1&&this.onChange.emit({originalEvent:e,value:l})}i&&this.hide(!0)}onOptionMouseEnter(e,n){this.focusOnHover&&this.changeFocusedOptionIndex(e,n)}updateModel(e,n){this.value=e,this.onModelChange(e),this.modelValue.set(e),this.selectedOptionUpdated=!0}writeValue(e){this.filter&&this.resetFilter(),this.value=e,this.allowModelChange()&&this.onModelChange(e),this.modelValue.set(this.value),this.updateEditableLabel(),this.cd.markForCheck()}allowModelChange(){return!!this.modelValue()&&!this.placeholder()&&(this.modelValue()===void 0||this.modelValue()===null)&&!this.editable&&this.options&&this.options.length}isSelected(e){return this.isOptionValueEqualsModelValue(e)}isOptionValueEqualsModelValue(e){return this.isValidOption(e)&&lt(this.modelValue(),this.getOptionValue(e),this.equalityKey())}ngAfterViewInit(){super.ngAfterViewInit(),this.editable&&this.updateEditableLabel(),this.updatePlaceHolderForFloatingLabel()}updatePlaceHolderForFloatingLabel(){let e=this.el.nativeElement.parentElement,n=e?.classList.contains("p-float-label");if(e&&n&&!this.selectedOption){let i=e.querySelector("label");i&&this._placeholder.set(i.textContent)}}updateEditableLabel(){this.editableInputViewChild&&(this.editableInputViewChild.nativeElement.value=this.getOptionLabel(this.selectedOption)||this.modelValue()||"")}clearEditableLabel(){this.editableInputViewChild&&(this.editableInputViewChild.nativeElement.value="")}getOptionIndex(e,n){return this.virtualScrollerDisabled?e:n&&n.getItemOptions(e).index}getOptionLabel(e){return this.optionLabel!==void 0&&this.optionLabel!==null?Q(e,this.optionLabel):e&&e.label!==void 0?e.label:e}getOptionValue(e){return this.optionValue&&this.optionValue!==null?Q(e,this.optionValue):!this.optionLabel&&e&&e.value!==void 0?e.value:e}isSelectedOptionEmpty(){return Fe(this.selectedOption)}isOptionDisabled(e){return this.getOptionValue(this.modelValue())===this.getOptionValue(e)||this.getOptionLabel(this.modelValue()===this.getOptionLabel(e))&&e.disabled===!1?!1:this.optionDisabled?Q(e,this.optionDisabled):e&&e.disabled!==void 0?e.disabled:!1}getOptionGroupLabel(e){return this.optionGroupLabel!==void 0&&this.optionGroupLabel!==null?Q(e,this.optionGroupLabel):e&&e.label!==void 0?e.label:e}getOptionGroupChildren(e){return this.optionGroupChildren!==void 0&&this.optionGroupChildren!==null?Q(e,this.optionGroupChildren):e.items}getAriaPosInset(e){return(this.optionGroupLabel?e-this.visibleOptions().slice(0,e).filter(n=>this.isOptionGroup(n)).length:e)+1}get ariaSetSize(){return this.visibleOptions().filter(e=>!this.isOptionGroup(e)).length}resetFilter(){this._filterValue.set(null),this.filterViewChild&&this.filterViewChild.nativeElement&&(this.filterViewChild.nativeElement.value="")}registerOnChange(e){this.onModelChange=e}registerOnTouched(e){this.onModelTouched=e}setDisabledState(e){this.disabled=e,this.cd.markForCheck()}onContainerClick(e){this.disabled||this.readonly||this.loading||(this.focusInputViewChild?.nativeElement.focus({preventScroll:!0}),!(e.target.tagName==="INPUT"||e.target.getAttribute("data-pc-section")==="clearicon"||e.target.closest('[data-pc-section="clearicon"]'))&&((!this.overlayViewChild||!this.overlayViewChild.el.nativeElement.contains(e.target))&&(this.overlayVisible?this.hide(!0):this.show(!0)),this.onClick.emit(e),this.clicked.set(!0),this.cd.detectChanges()))}isEmpty(){return!this._options()||this.visibleOptions()&&this.visibleOptions().length===0}onEditableInput(e){let n=e.target.value;this.searchValue="",!this.searchOptions(e,n)&&this.focusedOptionIndex.set(-1),this.onModelChange(n),this.updateModel(n||null,e),setTimeout(()=>{this.onChange.emit({originalEvent:e,value:n})},1),!this.overlayVisible&&_e(n)&&this.show()}show(e){this.overlayVisible=!0,this.focusedOptionIndex.set(this.focusedOptionIndex()!==-1?this.focusedOptionIndex():this.autoOptionFocus?this.findFirstFocusedOptionIndex():this.editable?-1:this.findSelectedOptionIndex()),e&&k(this.focusInputViewChild?.nativeElement),this.cd.markForCheck()}onOverlayAnimationStart(e){if(e.toState==="visible"){if(this.itemsWrapper=H(this.overlayViewChild?.overlayViewChild?.nativeElement,this.virtualScroll?".p-scroller":".p-select-list-container"),this.virtualScroll&&this.scroller?.setContentEl(this.itemsViewChild?.nativeElement),this.options&&this.options.length)if(this.virtualScroll){let n=this.modelValue()?this.focusedOptionIndex():-1;n!==-1&&this.scroller?.scrollToIndex(n)}else{let n=H(this.itemsWrapper,".p-select-option.p-select-option-selected");n&&n.scrollIntoView({block:"nearest",inline:"nearest"})}this.filterViewChild&&this.filterViewChild.nativeElement&&(this.preventModelTouched=!0,this.autofocusFilter&&!this.editable&&this.filterViewChild.nativeElement.focus()),this.onShow.emit(e)}e.toState==="void"&&(this.itemsWrapper=null,this.onModelTouched(),this.onHide.emit(e))}hide(e){this.overlayVisible=!1,this.focusedOptionIndex.set(-1),this.clicked.set(!1),this.searchValue="",this.overlayOptions?.mode==="modal"&&Je(),this.filter&&this.resetFilterOnHide&&this.resetFilter(),e&&(this.focusInputViewChild&&k(this.focusInputViewChild?.nativeElement),this.editable&&this.editableInputViewChild&&k(this.editableInputViewChild?.nativeElement)),this.cd.markForCheck()}onInputFocus(e){if(this.disabled)return;this.focused=!0;let n=this.focusedOptionIndex()!==-1?this.focusedOptionIndex():this.overlayVisible&&this.autoOptionFocus?this.findFirstFocusedOptionIndex():-1;this.focusedOptionIndex.set(n),this.overlayVisible&&this.scrollInView(this.focusedOptionIndex()),this.onFocus.emit(e)}onInputBlur(e){this.focused=!1,this.onBlur.emit(e),this.preventModelTouched||this.onModelTouched(),this.preventModelTouched=!1}onKeyDown(e,n=!1){if(!(this.disabled||this.readonly||this.loading)){switch(e.code){case"ArrowDown":this.onArrowDownKey(e);break;case"ArrowUp":this.onArrowUpKey(e,this.editable);break;case"ArrowLeft":case"ArrowRight":this.onArrowLeftKey(e,this.editable);break;case"Delete":this.onDeleteKey(e);break;case"Home":this.onHomeKey(e,this.editable);break;case"End":this.onEndKey(e,this.editable);break;case"PageDown":this.onPageDownKey(e);break;case"PageUp":this.onPageUpKey(e);break;case"Space":this.onSpaceKey(e,n);break;case"Enter":case"NumpadEnter":this.onEnterKey(e);break;case"Escape":this.onEscapeKey(e);break;case"Tab":this.onTabKey(e);break;case"Backspace":this.onBackspaceKey(e,this.editable);break;case"ShiftLeft":case"ShiftRight":break;default:!e.metaKey&&st(e.key)&&(!this.overlayVisible&&this.show(),!this.editable&&this.searchOptions(e,e.key));break}this.clicked.set(!1)}}onFilterKeyDown(e){switch(e.code){case"ArrowDown":this.onArrowDownKey(e);break;case"ArrowUp":this.onArrowUpKey(e,!0);break;case"ArrowLeft":case"ArrowRight":this.onArrowLeftKey(e,!0);break;case"Home":this.onHomeKey(e,!0);break;case"End":this.onEndKey(e,!0);break;case"Enter":case"NumpadEnter":this.onEnterKey(e,!0);break;case"Escape":this.onEscapeKey(e);break;case"Tab":this.onTabKey(e,!0);break;default:break}}onFilterBlur(e){this.focusedOptionIndex.set(-1)}onArrowDownKey(e){if(!this.overlayVisible)this.show(),this.editable&&this.changeFocusedOptionIndex(e,this.findSelectedOptionIndex());else{let n=this.focusedOptionIndex()!==-1?this.findNextOptionIndex(this.focusedOptionIndex()):this.clicked()?this.findFirstOptionIndex():this.findFirstFocusedOptionIndex();this.changeFocusedOptionIndex(e,n)}e.preventDefault(),e.stopPropagation()}changeFocusedOptionIndex(e,n){if(this.focusedOptionIndex()!==n&&(this.focusedOptionIndex.set(n),this.scrollInView(),this.selectOnFocus)){let i=this.visibleOptions()[n];this.onOptionSelect(e,i,!1)}}get virtualScrollerDisabled(){return!this.virtualScroll}scrollInView(e=-1){let n=e!==-1?`${this.id}_${e}`:this.focusedOptionId;if(this.itemsViewChild&&this.itemsViewChild.nativeElement){let i=H(this.itemsViewChild.nativeElement,`li[id="${n}"]`);i?i.scrollIntoView&&i.scrollIntoView({block:"nearest",inline:"nearest"}):this.virtualScrollerDisabled||setTimeout(()=>{this.virtualScroll&&this.scroller?.scrollToIndex(e!==-1?e:this.focusedOptionIndex())},0)}}hasSelectedOption(){return this.modelValue()!==void 0}isValidSelectedOption(e){return this.isValidOption(e)&&this.isSelected(e)}equalityKey(){return this.optionValue?null:this.dataKey}findFirstFocusedOptionIndex(){let e=this.findSelectedOptionIndex();return e<0?this.findFirstOptionIndex():e}findFirstOptionIndex(){return this.visibleOptions().findIndex(e=>this.isValidOption(e))}findSelectedOptionIndex(){return this.hasSelectedOption()?this.visibleOptions().findIndex(e=>this.isValidSelectedOption(e)):-1}findNextOptionIndex(e){let n=e<this.visibleOptions().length-1?this.visibleOptions().slice(e+1).findIndex(i=>this.isValidOption(i)):-1;return n>-1?n+e+1:e}findPrevOptionIndex(e){let n=e>0?Le(this.visibleOptions().slice(0,e),i=>this.isValidOption(i)):-1;return n>-1?n:e}findLastOptionIndex(){return Le(this.visibleOptions(),e=>this.isValidOption(e))}findLastFocusedOptionIndex(){let e=this.findSelectedOptionIndex();return e<0?this.findLastOptionIndex():e}isValidOption(e){return e!=null&&!(this.isOptionDisabled(e)||this.isOptionGroup(e))}isOptionGroup(e){return this.optionGroupLabel!==void 0&&this.optionGroupLabel!==null&&e.optionGroup!==void 0&&e.optionGroup!==null&&e.group}onArrowUpKey(e,n=!1){if(e.altKey&&!n){if(this.focusedOptionIndex()!==-1){let i=this.visibleOptions()[this.focusedOptionIndex()];this.onOptionSelect(e,i)}this.overlayVisible&&this.hide()}else{let i=this.focusedOptionIndex()!==-1?this.findPrevOptionIndex(this.focusedOptionIndex()):this.clicked()?this.findLastOptionIndex():this.findLastFocusedOptionIndex();this.changeFocusedOptionIndex(e,i),!this.overlayVisible&&this.show()}e.preventDefault(),e.stopPropagation()}onArrowLeftKey(e,n=!1){n&&this.focusedOptionIndex.set(-1)}onDeleteKey(e){this.showClear&&(this.clear(e),e.preventDefault())}onHomeKey(e,n=!1){if(n){let i=e.currentTarget;e.shiftKey?i.setSelectionRange(0,i.value.length):(i.setSelectionRange(0,0),this.focusedOptionIndex.set(-1))}else this.changeFocusedOptionIndex(e,this.findFirstOptionIndex()),!this.overlayVisible&&this.show();e.preventDefault()}onEndKey(e,n=!1){if(n){let i=e.currentTarget;if(e.shiftKey)i.setSelectionRange(0,i.value.length);else{let o=i.value.length;i.setSelectionRange(o,o),this.focusedOptionIndex.set(-1)}}else this.changeFocusedOptionIndex(e,this.findLastOptionIndex()),!this.overlayVisible&&this.show();e.preventDefault()}onPageDownKey(e){this.scrollInView(this.visibleOptions().length-1),e.preventDefault()}onPageUpKey(e){this.scrollInView(0),e.preventDefault()}onSpaceKey(e,n=!1){!this.editable&&!n&&this.onEnterKey(e)}onEnterKey(e,n=!1){if(!this.overlayVisible)this.focusedOptionIndex.set(-1),this.onArrowDownKey(e);else{if(this.focusedOptionIndex()!==-1){let i=this.visibleOptions()[this.focusedOptionIndex()];this.onOptionSelect(e,i)}!n&&this.hide()}e.preventDefault()}onEscapeKey(e){this.overlayVisible&&this.hide(!0),e.preventDefault(),e.stopPropagation()}onTabKey(e,n=!1){if(!n)if(this.overlayVisible&&this.hasFocusableElements())k(e.shiftKey?this.lastHiddenFocusableElementOnOverlay.nativeElement:this.firstHiddenFocusableElementOnOverlay.nativeElement),e.preventDefault();else{if(this.focusedOptionIndex()!==-1&&this.overlayVisible){let i=this.visibleOptions()[this.focusedOptionIndex()];this.onOptionSelect(e,i)}this.overlayVisible&&this.hide(this.filter)}e.stopPropagation()}onFirstHiddenFocus(e){let n=e.relatedTarget===this.focusInputViewChild?.nativeElement?et(this.overlayViewChild.el?.nativeElement,":not(.p-hidden-focusable)"):this.focusInputViewChild?.nativeElement;k(n)}onLastHiddenFocus(e){let n=e.relatedTarget===this.focusInputViewChild?.nativeElement?tt(this.overlayViewChild?.overlayViewChild?.nativeElement,':not([data-p-hidden-focusable="true"])'):this.focusInputViewChild?.nativeElement;k(n)}hasFocusableElements(){return Xe(this.overlayViewChild.overlayViewChild.nativeElement,':not([data-p-hidden-focusable="true"])').length>0}onBackspaceKey(e,n=!1){n&&!this.overlayVisible&&this.show()}searchFields(){return this.filterBy?.split(",")||this.filterFields||[this.optionLabel]}searchOptions(e,n){this.searchValue=(this.searchValue||"")+n;let i=-1,o=!1;return i=this.visibleOptions().findIndex(l=>this.isOptionMatched(l)),i!==-1&&(o=!0),i===-1&&this.focusedOptionIndex()===-1&&(i=this.findFirstFocusedOptionIndex()),i!==-1&&setTimeout(()=>{this.changeFocusedOptionIndex(e,i)}),this.searchTimeout&&clearTimeout(this.searchTimeout),this.searchTimeout=setTimeout(()=>{this.searchValue="",this.searchTimeout=null},500),o}isOptionMatched(e){return this.isValidOption(e)&&this.getOptionLabel(e).toString().toLocaleLowerCase(this.filterLocale).startsWith(this.searchValue.toLocaleLowerCase(this.filterLocale))}onFilterInputChange(e){let n=e.target.value;this._filterValue.set(n),this.focusedOptionIndex.set(-1),this.onFilter.emit({originalEvent:e,filter:this._filterValue()}),!this.virtualScrollerDisabled&&this.scroller.scrollToIndex(0),setTimeout(()=>{this.overlayViewChild.alignOverlay()}),this.cd.markForCheck()}applyFocus(){this.editable?H(this.el.nativeElement,".p-dropdown-label.p-inputtext").focus():k(this.focusInputViewChild?.nativeElement)}focus(){this.applyFocus()}clear(e){this.updateModel(null,e),this.clearEditableLabel(),this.onModelTouched(),this.onChange.emit({originalEvent:e,value:this.value}),this.onClear.emit(e),this.resetFilter()}static \u0275fac=function(n){return new(n||t)(R(ae),R(ct))};static \u0275cmp=P({type:t,selectors:[["p-select"]],contentQueries:function(n,i,o){if(n&1&&(y(o,gi,4),y(o,yi,4),y(o,bi,4),y(o,vi,4),y(o,xi,4),y(o,kt,4),y(o,Ci,4),y(o,Oi,4),y(o,Ii,4),y(o,Ti,4),y(o,wi,4),y(o,Si,4),y(o,Vi,4),y(o,ki,4),y(o,Ei,4),y(o,Fi,4),y(o,fe,4)),n&2){let l;d(l=u())&&(i.itemTemplate=l.first),d(l=u())&&(i.groupTemplate=l.first),d(l=u())&&(i.loaderTemplate=l.first),d(l=u())&&(i.selectedItemTemplate=l.first),d(l=u())&&(i.headerTemplate=l.first),d(l=u())&&(i.filterTemplate=l.first),d(l=u())&&(i.footerTemplate=l.first),d(l=u())&&(i.emptyFilterTemplate=l.first),d(l=u())&&(i.emptyTemplate=l.first),d(l=u())&&(i.dropdownIconTemplate=l.first),d(l=u())&&(i.loadingIconTemplate=l.first),d(l=u())&&(i.clearIconTemplate=l.first),d(l=u())&&(i.filterIconTemplate=l.first),d(l=u())&&(i.onIconTemplate=l.first),d(l=u())&&(i.offIconTemplate=l.first),d(l=u())&&(i.cancelIconTemplate=l.first),d(l=u())&&(i.templates=l)}},viewQuery:function(n,i){if(n&1&&(V(kt,5),V(Li,5),V(Mi,5),V(Di,5),V(zi,5),V($i,5),V(Ri,5),V(Ai,5)),n&2){let o;d(o=u())&&(i.filterViewChild=o.first),d(o=u())&&(i.focusInputViewChild=o.first),d(o=u())&&(i.editableInputViewChild=o.first),d(o=u())&&(i.itemsViewChild=o.first),d(o=u())&&(i.scroller=o.first),d(o=u())&&(i.overlayViewChild=o.first),d(o=u())&&(i.firstHiddenFocusableElementOnOverlay=o.first),d(o=u())&&(i.lastHiddenFocusableElementOnOverlay=o.first)}},hostVars:5,hostBindings:function(n,i){n&1&&v("click",function(l){return i.onContainerClick(l)}),n&2&&(T("id",i.id),ce(i.hostStyle),z(i.hostClass))},inputs:{id:"id",scrollHeight:"scrollHeight",filter:[2,"filter","filter",g],name:"name",style:"style",panelStyle:"panelStyle",styleClass:"styleClass",panelStyleClass:"panelStyleClass",readonly:[2,"readonly","readonly",g],required:[2,"required","required",g],editable:[2,"editable","editable",g],appendTo:"appendTo",tabindex:[2,"tabindex","tabindex",Z],placeholder:"placeholder",loadingIcon:"loadingIcon",filterPlaceholder:"filterPlaceholder",filterLocale:"filterLocale",variant:"variant",inputId:"inputId",dataKey:"dataKey",filterBy:"filterBy",filterFields:"filterFields",autofocus:[2,"autofocus","autofocus",g],resetFilterOnHide:[2,"resetFilterOnHide","resetFilterOnHide",g],checkmark:[2,"checkmark","checkmark",g],dropdownIcon:"dropdownIcon",loading:[2,"loading","loading",g],optionLabel:"optionLabel",optionValue:"optionValue",optionDisabled:"optionDisabled",optionGroupLabel:"optionGroupLabel",optionGroupChildren:"optionGroupChildren",autoDisplayFirst:[2,"autoDisplayFirst","autoDisplayFirst",g],group:[2,"group","group",g],showClear:[2,"showClear","showClear",g],emptyFilterMessage:"emptyFilterMessage",emptyMessage:"emptyMessage",lazy:[2,"lazy","lazy",g],virtualScroll:[2,"virtualScroll","virtualScroll",g],virtualScrollItemSize:[2,"virtualScrollItemSize","virtualScrollItemSize",Z],virtualScrollOptions:"virtualScrollOptions",size:"size",overlayOptions:"overlayOptions",ariaFilterLabel:"ariaFilterLabel",ariaLabel:"ariaLabel",ariaLabelledBy:"ariaLabelledBy",filterMatchMode:"filterMatchMode",maxlength:[2,"maxlength","maxlength",Z],tooltip:"tooltip",tooltipPosition:"tooltipPosition",tooltipPositionStyle:"tooltipPositionStyle",tooltipStyleClass:"tooltipStyleClass",focusOnHover:[2,"focusOnHover","focusOnHover",g],selectOnFocus:[2,"selectOnFocus","selectOnFocus",g],autoOptionFocus:[2,"autoOptionFocus","autoOptionFocus",g],autofocusFilter:[2,"autofocusFilter","autofocusFilter",g],fluid:[2,"fluid","fluid",g],disabled:"disabled",itemSize:"itemSize",autoZIndex:"autoZIndex",baseZIndex:"baseZIndex",showTransitionOptions:"showTransitionOptions",hideTransitionOptions:"hideTransitionOptions",filterValue:"filterValue",options:"options"},outputs:{onChange:"onChange",onFilter:"onFilter",onFocus:"onFocus",onBlur:"onBlur",onClick:"onClick",onShow:"onShow",onHide:"onHide",onClear:"onClear",onLazyLoad:"onLazyLoad"},features:[pe([jn,Et]),B],decls:11,vars:15,consts:[["elseBlock",""],["overlay",""],["content",""],["focusInput",""],["defaultPlaceholder",""],["editableInput",""],["firstHiddenFocusableEl",""],["buildInItems",""],["lastHiddenFocusableEl",""],["builtInFilterElement",""],["filter",""],["scroller",""],["loader",""],["items",""],["emptyFilter",""],["empty",""],["role","combobox",3,"ngClass","pTooltip","tooltipPosition","positionStyle","tooltipStyleClass","pAutoFocus","focus","blur","keydown",4,"ngIf"],["type","text","aria-haspopup","listbox",3,"ngClass","disabled","pAutoFocus","input","keydown","focus","blur",4,"ngIf"],[4,"ngIf"],["role","button","aria-label","dropdown trigger","aria-haspopup","listbox",1,"p-select-dropdown"],[4,"ngIf","ngIfElse"],[3,"visibleChange","onAnimationStart","onHide","visible","options","target","appendTo","autoZIndex","baseZIndex","showTransitionOptions","hideTransitionOptions"],["role","combobox",3,"focus","blur","keydown","ngClass","pTooltip","tooltipPosition","positionStyle","tooltipStyleClass","pAutoFocus"],[3,"ngTemplateOutlet","ngTemplateOutletContext",4,"ngIf"],[3,"ngTemplateOutlet","ngTemplateOutletContext"],["type","text","aria-haspopup","listbox",3,"input","keydown","focus","blur","ngClass","disabled","pAutoFocus"],["class","p-select-clear-icon",3,"click",4,"ngIf"],[1,"p-select-clear-icon",3,"click"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],[4,"ngTemplateOutlet"],["aria-hidden","true",3,"ngClass",4,"ngIf"],["aria-hidden","true",3,"class",4,"ngIf"],["aria-hidden","true",3,"ngClass"],["aria-hidden","true"],["class","p-select-dropdown-icon",4,"ngIf"],["class","p-select-dropdown-icon",3,"ngClass",4,"ngIf"],[3,"styleClass",4,"ngIf"],[1,"p-select-dropdown-icon",3,"ngClass"],[3,"styleClass"],[1,"p-select-dropdown-icon"],[3,"ngClass","ngStyle"],["role","presentation",1,"p-hidden-accessible","p-hidden-focusable",3,"focus"],["class","p-select-header",3,"click",4,"ngIf"],[1,"p-select-list-container"],[3,"items","style","itemSize","autoSize","lazy","options","onLazyLoad",4,"ngIf"],[1,"p-select-header",3,"click"],["pInputText","","type","text","role","searchbox","autocomplete","off",1,"p-select-filter",3,"input","keydown","blur","pSize","value","variant"],[3,"onLazyLoad","items","itemSize","autoSize","lazy","options"],["role","listbox",1,"p-select-list",3,"ngClass"],["ngFor","",3,"ngForOf"],["class","p-select-empty-message","role","option",3,"ngStyle",4,"ngIf"],["role","option",1,"p-select-option-group",3,"ngStyle"],[3,"onClick","onMouseEnter","id","option","checkmark","selected","label","disabled","template","focused","ariaPosInset","ariaSetSize"],["role","option",1,"p-select-empty-message",3,"ngStyle"]],template:function(n,i){if(n&1){let o=I();p(0,Gi,6,20,"span",16)(1,Ni,2,8,"input",17)(2,Ji,3,2,"ng-container",18),h(3,"div",19),p(4,ln,3,2,"ng-container",20)(5,un,2,2,"ng-template",null,0,L),m(),h(7,"p-overlay",21,1),qe("visibleChange",function(x){return _(o),Be(i.overlayVisible,x)||(i.overlayVisible=x),f(x)}),v("onAnimationStart",function(x){return _(o),f(i.onOverlayAnimationStart(x))})("onHide",function(){return _(o),f(i.hide())}),p(9,Zn,13,17,"ng-template",null,2,L),m()}if(n&2){let o,l=A(6);r("ngIf",!i.editable),c(),r("ngIf",i.editable),c(),r("ngIf",i.isVisibleClearIcon),c(),T("aria-expanded",(o=i.overlayVisible)!==null&&o!==void 0?o:!1)("data-pc-section","trigger"),c(),r("ngIf",i.loading)("ngIfElse",l),c(3),Pe("visible",i.overlayVisible),r("options",i.overlayOptions)("target","@parent")("appendTo",i.appendTo)("autoZIndex",i.autoZIndex)("baseZIndex",i.baseZIndex)("showTransitionOptions",i.showTransitionOptions)("hideTransitionOptions",i.hideTransitionOptions)}},dependencies:[Y,N,Ne,j,W,U,Un,Me,xt,mt,bt,gt,yt,Ct,Ot,It,Tt,E],encapsulation:2,changeDetection:0})}return t})(),No=(()=>{class t{static \u0275fac=function(n){return new(n||t)};static \u0275mod=re({type:t});static \u0275inj=le({imports:[Mt,E,E]})}return t})()});export{Me as a,go as b,Vt as c,Mt as d,No as e,Wn as f};
