import{a as o,b as d}from"./chunk-JGUUVYPT.js";import{a as n,b as p}from"./chunk-DXKIJFOO.js";import{a as i,c}from"./chunk-NUDGNJBM.js";import"./chunk-B64ADSFN.js";import{a,b as m}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-ZSHAELBT.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import{Yb as e,Zb as s}from"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import"./chunk-XJKQ67XR.js";import{e as r}from"./chunk-SZAW6ZBW.js";var A,v=r(()=>{m();d();p();c();s();A=[{path:"",loadComponent:()=>import("./chunk-F44DDTLS.js").then(t=>t.ConfigurationComponent),canActivate:[a,o],children:[{loadComponent:()=>import("./chunk-BXI7WC6Q.js").then(t=>t.GeneralSettingsComponent),path:"general-settings",canActivate:[i],data:{permissions:[e.ServerSettingsGeneralView],editPermission:e.ServerSettingsGeneralEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-A4X5HV7F.js").then(t=>t.TwoFactorConfigurationComponent),path:"two-factor",canActivate:[i],data:{permissions:[e.ServerSettingsMfaView],editPermission:e.ServerSettingsMfaEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-JJBNWM4G.js").then(t=>t.SecurityConfigurationComponent),path:"security",canActivate:[i],data:{permissions:[e.ServerSettingsSecurityView],editPermission:e.ServerSettingsSecurityEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-JVZFQIDZ.js").then(t=>t.GeoIpConfigurationComponent),path:"geo-ip",canActivate:[i],data:{permissions:[e.ServerSettingsGeoIPView],editPermission:e.ServerSettingsGeoIPEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-HOZC4EWF.js").then(t=>t.ConfigurationLoggingComponent),path:"logging",canActivate:[i],data:{permissions:[e.ServerSettingsLogsView],editPermission:e.ServerSettingsLogsEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-BATF5IUZ.js").then(t=>t.FeaturesConfigurationComponent),path:"features",canActivate:[i],data:{permissions:[e.ServerSettingsFeaturesView],editPermission:e.ServerSettingsFeaturesEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-C3AL7STJ.js").then(t=>t.AdvancedConfigurationComponent),path:"advanced",canActivate:[i],data:{permissions:[e.ServerSettingsAdvancedView],editPermission:e.ServerSettingsAdvancedEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-6VZRNTOZ.js").then(t=>t.ApprovalConfigurationComponent),path:"approval",canActivate:[i],data:{permissions:[e.ServerSettingsApprovalsView],editPermission:e.ServerSettingsApprovalsEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-VLGAFAJM.js").then(t=>t.EmailConfigurationComponent),path:"email",canActivate:[i],data:{permissions:[e.ServerSettingsEmailView],editPermission:e.ServerSettingsEmailEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-OTOQCMFL.js").then(t=>t.AuthenticationComponent),path:"authentication",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-5RUV2D6S.js").then(t=>t.AzureComponent),path:"authentication/office365",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-LZN7Q2ZX.js").then(t=>t.OktaComponent),path:"authentication/okta",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-VHDYCWMB.js").then(t=>t.PingOneComponent),path:"authentication/pingone",canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{path:"",loadComponent:()=>import("./chunk-YRH3CRQG.js").then(t=>t.ConfigurationDashboardComponent)},{path:"authentication/multi-domain",loadComponent:()=>import("./chunk-C5RNRD4R.js").then(t=>t.MultiDomainListComponent),canActivate:[i],data:{permissions:[e.ServerSettingsAuthenticationView],editPermission:e.ServerSettingsAuthenticationEdit},canDeactivate:[n]},{loadComponent:()=>import("./chunk-BUUSKIVO.js").then(t=>t.SecurityPolicyComponent),path:"security-policies",canActivate:[i],data:{permissions:[e.SecurityPoliciesView]}},{path:"user-behavior-analytics",loadComponent:()=>import("./chunk-WD4WSGGY.js").then(t=>t.UserBehaviorAnalyticsComponent),canActivate:[i],data:{permissions:[e.UserBehaviorAnalyticsConfigurationView]}}]}]});v();export{A as CONFIGURATION_ROUTES};
