import{a as p,b as f}from"./chunk-4FQJH7SK.js";import"./chunk-EXSN6QWV.js";import"./chunk-INYFUTXB.js";import{p as l,v as c}from"./chunk-FNS7DN6T.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Rb as n,Sd as a,Vb as o,xc as i,yc as r,zc as s}from"./chunk-XJKQ67XR.js";import{e as u}from"./chunk-SZAW6ZBW.js";var M,d=u(()=>{f();c();a();M=(()=>{class t extends p{constructor(){super()}getComponentTitle(){return"lblConfiguration"}static{this.\u0275fac=function(e){return new(e||t)}}static{this.\u0275cmp=n({type:t,selectors:[["ng-component"]],features:[o],decls:2,vars:0,consts:[[1,"full-height","full-width"]],template:function(e,g){e&1&&(i(0,"div",0),s(1,"router-outlet"),r())},dependencies:[l],styles:["[_nghost-%COMP%]   ul[_ngcontent-%COMP%]{list-style-type:none;padding:0}[_nghost-%COMP%]   .page-content[_ngcontent-%COMP%]{width:82%;display:flex}"]})}}return t})()});d();export{M as ConfigurationComponent};
