import{c as t,d as r}from"./chunk-CUIDVWNH.js";import{e as s}from"./chunk-SZAW6ZBW.js";var i,o=s(()=>{r();i=class n{toQueryString(){let e=[];return this.pageSize!==null&&this.pageSize.toString()&&e.push("pageSize="+encodeURIComponent(this.pageSize.toString())),this.nextPageToken!==null&&this.nextPageToken.toString()&&e.push("nextPageToken="+encodeURIComponent(this.nextPageToken.toString())),this.filter!==null&&this.filter.toString()&&e.push("filter="+encodeURIComponent(this.filter.toString())),this.sortField!==null&&this.sortField.toString()&&e.push("sortField="+encodeURIComponent(this.sortField.toString())),this.sortOrder!==null&&this.sortOrder.toString()&&e.push("sortOrder="+encodeURIComponent(this.sortOrder.toString())),e.join("&")}constructor(e=null){this.clone=()=>new n(this),this.pageSize=t.manageDefaultValue(e,"pageSize",0),this.nextPageToken=t.manageDefaultValue(e,"nextPageToken",""),this.filter=t.manageDefaultValue(e,"filter",""),this.sortField=t.manageDefaultValue(e,"sortField",""),this.sortOrder=t.manageDefaultValue(e,"sortOrder",0)}}});var l,h=s(()=>{r();o();l=class n extends i{toQueryString(){let e=[];return e.push(super.toQueryString()),this.hideExistingUsers!==null&&this.hideExistingUsers.toString()&&e.push("hideExistingUsers="+encodeURIComponent(this.hideExistingUsers.toString())),e.join("&")}constructor(e=null){super(e),this.clone=()=>new n(this),this.hideExistingUsers=t.manageDefaultValue(e,"hideExistingUsers",!1)}}});export{i as a,o as b,l as c,h as d};
