import{a as o,b as p}from"./chunk-GWI5PXEB.js";import{a as m,c as s}from"./chunk-NUDGNJBM.js";import{a as i,b as n}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-ZSHAELBT.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import{Yb as t,Zb as e}from"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import"./chunk-XJKQ67XR.js";import{e as a}from"./chunk-SZAW6ZBW.js";var A,d=a(()=>{n();p();s();e();A=[{path:"",children:[{path:"",canActivate:[i,o,m],data:{permissions:[t.PamSettingsJitTemplatesView]},loadComponent:()=>import("./chunk-7KQXINNM.js").then(r=>r.SshJitTemplatesDashboardComponent)}]}]});d();export{A as SSH_JIT_TEMPLATES_ROUTES};
