import{e as n}from"./chunk-SZAW6ZBW.js";var e,C=n(()=>{e=function(r){return r[r.Cancel=0]="Cancel",r[r.Ignore=1]="Ignore",r[r.IgnoreAll=2]="IgnoreAll",r[r.Overwrite=3]="Overwrite",r[r.OverwriteAll=4]="OverwriteAll",r[r.Add=5]="Add",r[r.AddAll=6]="AddAll",r}(e||{});(function(r){function d(){return"DuplicateAction"}r.getClass=d;function s(f){let g="";return typeof r.getClass=="function"&&(g=r.getClass()),g+"_"+r[f]}r.getEnumKey=s})(e||(e={}))});export{e as a,C as b};
