import{a as Q,b as j,c as k}from"./chunk-FEY6ZSY7.js";import{Ha as I,La as N}from"./chunk-5YPL7P3G.js";import{K as B,t as W}from"./chunk-HJI5FNS6.js";import{Bb as i,Cc as u,Ic as l,Jc as D,Kc as S,Ld as y,Rb as h,Rc as E,Sd as $,Tc as O,Uc as w,Xb as r,_c as C,ad as f,kc as d,ld as v,md as x,nc as _,nd as g,sc as m,tc as M,vc as F,wc as b,xc as c,yc as a}from"./chunk-XJKQ67XR.js";import{e as A}from"./chunk-SZAW6ZBW.js";function J(e,o){if(e&1&&u(0,4),e&2){let t=l().$implicit,n=l(2);d("ngTemplateOutlet",n.itemTemplate.template)("ngTemplateOutletContext",g(2,q,t,t))}}function K(e,o){if(e&1&&(c(0,"div"),C(1),a()),e&2){let t=l(2).$implicit,n=l(2);_(x(3,R,n.summaryFontWeight||"600")),i(),f(" ",t.summary," ")}}function L(e,o){if(e&1&&(c(0,"div"),C(1),a()),e&2){let t=l(2).$implicit,n=l(2);_(x(3,V,n.detailFontWeight||"normal")),i(),f(" ",t.detail," ")}}function U(e,o){if(e&1&&(c(0,"div",5),r(1,K,2,5,"div",7)(2,L,2,5,"div",7),a()),e&2){let t=l().$implicit;i(),m(t.summary?1:-1),i(),m(t.detail?2:-1)}}function X(e,o){if(e&1&&(c(0,"div",8),u(1,4),a()),e&2){let t=l().$implicit,n=l(2);_(v(4,z)),i(),d("ngTemplateOutlet",n.endTemplate.template)("ngTemplateOutletContext",g(5,q,t,t))}}function Y(e,o){if(e&1&&(c(0,"div",2)(1,"p-message",3),r(2,J,1,5,"ng-container",4)(3,U,3,2,"div",5),a(),r(4,X,2,8,"div",6),a()),e&2){let t,n,s=o.$implicit,p=l(2);i(),d("severity",(t=s.severity)!==null&&t!==void 0?t:p.severity)("escape",p.escape)("closable",(n=s.closable)!==null&&n!==void 0?n:p.closable)("styleClass",p.messageStyleClass),i(),m(p.itemTemplate?2:3),i(2),m(p.endTemplate?4:-1)}}function Z(e,o){if(e&1&&(c(0,"div",0),F(1,Y,5,6,"div",2,M),a()),e&2){let t=l();i(),b(t.value)}}function ee(e,o){if(e&1&&u(0,12),e&2){let t=l(2);d("ngTemplateOutlet",t.itemTemplate.template)}}function te(e,o){e&1&&S(0)}function ne(e,o){if(e&1&&(c(0,"div"),u(1,12),a()),e&2){let t=l(2);_(v(3,H)),i(),d("ngTemplateOutlet",t.endTemplate.template)}}function ie(e,o){if(e&1&&(c(0,"div",1)(1,"p-message",9)(2,"div",10)(3,"div",11),r(4,ee,1,1,"ng-container",12)(5,te,1,0),a(),r(6,ne,2,4,"div",7),a()()()),e&2){let t=l();i(),d("severity",t.severity)("escape",t.escape)("closable",t.closable)("styleClass",t.messageStyleClass),i(),_(v(8,G)),i(2),m(t.itemTemplate?4:5),i(2),m(t.endTemplate?6:-1)}}var P,q,R,V,z,G,H,ve,le=A(()=>{$();B();k();N();$();k();P=["*"],q=(e,o)=>({$implicit:e,message:o}),R=e=>({"font-weight":e,color:"var(--text-400)"}),V=e=>({"font-weight":e,color:"var(--text-300)"}),z=()=>({"padding-right":"1.25rem"}),G=()=>({width:"100%"}),H=()=>({"margin-left":"1rem"});ve=(()=>{class e{constructor(){this.value=null,this.severity="info",this.escape=!0,this.closable=!1}get itemTemplate(){if(this.templates)return this.templates.find(t=>t.name==="content")??this.templates.find(t=>t.name==="message")??this.templates.find(t=>!t.name)}get endTemplate(){if(this.templates)return this.templates.find(t=>t.name==="end"||t.name==="actions")}hasContent(){return!!this.itemTemplate||!!this.endTemplate}static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275cmp=h({type:e,selectors:[["p-messages"],["dvl-messages"]],contentQueries:function(n,s,p){if(n&1&&E(p,I,4),n&2){let T;O(T=w())&&(s.templates=T)}},inputs:{value:"value",severity:"severity",escape:[2,"escape","escape",y],closable:[2,"closable","closable",y],messageStyleClass:"messageStyleClass",summaryFontWeight:"summaryFontWeight",detailFontWeight:"detailFontWeight"},ngContentSelectors:P,decls:2,vars:2,consts:[[1,"flex","flex-col","w-full","gap-2"],[1,"flex","flex-col","w-full"],[1,"flex","items-center","w-full","relative"],[1,"flex-1",3,"severity","escape","closable","styleClass"],[3,"ngTemplateOutlet","ngTemplateOutletContext"],[1,"flex","flex-col","gap-1.5"],[1,"absolute","right-0","top-0","bottom-0","flex","items-center",3,"style"],[3,"style"],[1,"absolute","right-0","top-0","bottom-0","flex","items-center"],[3,"severity","escape","closable","styleClass"],[1,"flex","items-center"],[1,"flex-1"],[3,"ngTemplateOutlet"]],template:function(n,s){n&1&&(D(),r(0,Z,3,0,"div",0)(1,ie,7,9,"div",1)),n&2&&(m(s.value&&s.value.length>0?0:-1),i(),m(s.value===null||s.value===void 0?1:-1))},dependencies:[W,j,Q],encapsulation:2})}}return e})()});export{ve as a,le as b};
