import{a as z,b as J,c as K}from"./chunk-HL5YYMVR.js";import{a as H,b as Z,c as q}from"./chunk-DM2XHT3H.js";import{C as O,E as k,b as B,f as V,m as W}from"./chunk-K7KAWBRR.js";import{a as N,b as te}from"./chunk-CMGJXTS5.js";import{a as D,b as $}from"./chunk-3AUKOF45.js";import{b as L,c as ee}from"./chunk-IBUDDWQO.js";import{K as Y,m as E}from"./chunk-HJI5FNS6.js";import{$c as C,Bb as a,Dc as I,Gb as M,Hc as m,Ic as F,Ka as w,La as T,R as f,Rb as P,Sd as j,Ua as v,Vb as U,W as S,Xb as A,_c as h,ed as _,fd as y,g as d,gd as b,ia as g,kc as o,na as R,oa as X,oc as x,sc as G,vd as c,wd as u,xc as s,yc as i}from"./chunk-XJKQ67XR.js";import{e as Q}from"./chunk-SZAW6ZBW.js";function re(n,ie){if(n&1){let t=I();s(0,"i",6),m("click",function(){w(t);let e=F();return T(e.clearFilter())}),i()}}var we,ae=Q(()=>{j();$();R();X();Y();te();q();k();K();j();ee();q();k();K();we=(()=>{class n extends D{constructor(t){super(),this.translatorService=t,this.parametersChange=new v,this.filterSubject=new d,this.parametersChangedSubject=new d,this.userTypeOptions=[{label:this.translatorService.translate("lblUser"),value:"User"},{label:this.translatorService.translate("lblUserGroup"),value:"UserGroup"},{label:this.translatorService.translate("lblApplication"),value:"Application"}]}ngOnInit(){this.subscribeToFilterValue(),this.subscribeToParametersChange()}onFilterChange(t){this.filterSubject.next(t.target.value)}clearFilter(){this.parameters.filter="",this.onParametersChange()}clearUserType(){this.parameters.userType=null,this.onParametersChange()}onParametersChange(){this.parametersChangedSubject.next()}subscribeToParametersChange(){this.parametersChangedSubject.pipe(f(500),g(this.destroyed$)).subscribe(()=>this.parametersChange.emit(this.parameters))}subscribeToFilterValue(){this.filterSubject.pipe(S(),f(500),g(this.destroyed$)).subscribe(t=>{this.parameters.filter=t,this.onParametersChange()})}static{this.\u0275fac=function(l){return new(l||n)(M(L))}}static{this.\u0275cmp=P({type:n,selectors:[["dvls-gateway-user-access-parameters"]],inputs:{isScheduling:"isScheduling",parameters:"parameters"},outputs:{parametersChange:"parametersChange"},features:[U],decls:14,vars:20,consts:[[2,"padding-bottom","15px"],[3,"ngClass"],[1,"clear-icon-parent"],["pInputText","","id","filter",1,"form-control",3,"ngModelChange","input","ngModel"],[1,"pi","pi-times","filter-clear-icon"],["styleClass","form-control",3,"ngModelChange","onChange","onClear","options","showHeader","defaultLabel","ngModel","baseZIndex","showClear"],[1,"pi","pi-times","filter-clear-icon",3,"click"]],template:function(l,e){l&1&&(s(0,"div",0)(1,"div",1)(2,"label"),h(3),c(4,"translate"),i(),s(5,"div",2)(6,"input",3),b("ngModelChange",function(r){return y(e.parameters.filter,r)||(e.parameters.filter=r),r}),m("input",function(r){return e.onFilterChange(r)}),i(),A(7,re,1,0,"i",4),i()(),s(8,"div",1)(9,"label"),h(10),c(11,"translate"),i(),s(12,"p-multiSelect",5),c(13,"translate"),b("ngModelChange",function(r){return y(e.parameters.userType,r)||(e.parameters.userType=r),r}),m("onChange",function(){return e.onParametersChange()})("onClear",function(){return e.clearUserType()}),i()()()),l&2&&(x(e.isScheduling?"form parameters-form flex flex-col gap-4":"form parameters-form flex flex-row gap-4"),a(),o("ngClass",e.isScheduling?"form-group w-full":"form-group flex-1"),a(2),C(u(4,14,"lblFilter")),a(3),_("ngModel",e.parameters.filter),a(),G(e.parameters.filter?7:-1),a(),o("ngClass",e.isScheduling?"form-group w-full":"form-group flex-1"),a(2),C(u(11,16,"lblUserType")),a(2),o("options",e.userTypeOptions)("showHeader",!1)("defaultLabel",u(13,18,"lblSelect")),_("ngModel",e.parameters.userType),o("baseZIndex",3200)("showClear",!0))},dependencies:[E,N,Z,H,O,B,V,W,J,z],styles:["[_nghost-%COMP%]     .clear-icon-parent{position:relative}"]})}}return n})()});export{we as a,ae as b};
