import{a as d,b as D}from"./chunk-4FQJH7SK.js";import{a as r,b as R}from"./chunk-EXSN6QWV.js";import{V as c,W as f,j as h,k as l}from"./chunk-INYFUTXB.js";import{J as o,Sd as m,Tb as n,Ua as t,Vb as a,ia as s,oa as u}from"./chunk-XJKQ67XR.js";import{e as p}from"./chunk-SZAW6ZBW.js";var A,S=p(()=>{m();l();R();D();u();f();m();A=(()=>{class i extends d{constructor(){super(),this.onAdd=new t,this.onChangePasswordRow=new t,this.onRefresh=new t,this.onViewRow=new t,this.onEditRow=new t,this.onDeleteRow=new t,this.onDuplicateRow=new t,this.storageService=r.injector.get(h),this.currentUserService=r.injector.get(c)}ngAfterContentInit(){this.currentUserService.webSettingsChanged.pipe(s(this.destroyed$),o(e=>!!e)).subscribe(e=>{this.defaultPageSize=e.defaultPageSize,this.dateFormatToDisplay=this.currentUserService.displayDateFormat,this.timeFormatToDisplay=this.currentUserService.displayTimeFormat,this.dateTimeFormatToDisplay=this.currentUserService.displayDateTimeFormat})}add(){this.onAdd.emit(null)}changePassword(e){this.onChangePasswordRow.emit(e)}view(e){this.onViewRow.emit(e)}edit(e){this.onEditRow.emit(e)}delete(e){this.onDeleteRow.emit(e)}refresh(){this.onRefresh.emit(null)}static{this.\u0275fac=function(w){return new(w||i)}}static{this.\u0275dir=n({type:i,inputs:{items:"items"},outputs:{onEditRow:"onEditRow",onDeleteRow:"onDeleteRow"},features:[a]})}}return i})()});export{A as a,S as b};
