import{Sd as c,g as s,na as a,ta as o}from"./chunk-XJKQ67XR.js";import{e as i}from"./chunk-SZAW6ZBW.js";var f,n=i(()=>{a();c();f=(()=>{class t{constructor(){this.showRequest=new s,this.showRequestObservable=this.showRequest.asObservable()}show(e){this.showRequest.next(e)}static{this.\u0275fac=function(r){return new(r||t)}}static{this.\u0275prov=o({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{f as a,n as b};
