import{Sd as n,Tb as r,g as e,na as s}from"./chunk-XJKQ67XR.js";import{e as o}from"./chunk-SZAW6ZBW.js";var f,c=o(()=>{s();n();f=(()=>{class t{constructor(){this.destroyed$=new e}ngOnDestroy(){this.destroyed$.next(!0)}static{this.\u0275fac=function(i){return new(i||t)}}static{this.\u0275dir=r({type:t})}}return t})()});export{f as a,c as b};
