import{a as n,b as g}from"./chunk-ZHWSFSWA.js";import{D as p,E as u}from"./chunk-INYFUTXB.js";import{Gb as c,Sd as d,Ub as m}from"./chunk-XJKQ67XR.js";import{e as S}from"./chunk-SZAW6ZBW.js";var A,y,I=S(()=>{g();d();u();A={icon:"dvl-icon-account-owner",color:null},y=(()=>{class t{constructor(e){this.serverInformationService=e}transform(e,i=24,o=[],f="di dvl-icon-entry-SampleUser"){let l=r=>this.serverInformationService.privateServerInformation.dataSourceOwner===r.id,a=r=>r.name===e.replace(/^(?:.*?\\)?(.+)$/,"$1"),v=r=>r.filter(a).map(s=>n.fromUserListItem(s,i,null,l(s)?A:null)).find(Boolean);return o.some(a)?v(o):new n({username:e,name:e,size:24,icon:f})}static{this.\u0275fac=function(i){return new(i||t)(c(p,16))}}static{this.\u0275pipe=m({name:"getUserAvatarSettings",type:t,pure:!0})}}return t})()});export{A as a,y as b,I as c};
