import{a as e,b as p}from"./chunk-A3DB3QKG.js";import{Ac as s,Bc as f,ic as o,jc as r}from"./chunk-2GUKPKKM.js";import{e as c}from"./chunk-SZAW6ZBW.js";var i,C=c(()=>{r();p();i=class extends e{constructor(t,n,u,T){let a=o[t]+(n?`.${n}`:"");super("Session",a,u,T),this.groupLabel="ConnectionType_Session"}}});var m,x=c(()=>{r();p();f();m=class extends e{constructor(t,n){super(o[o.Sync],s[t],n),this.groupLabel="ConnectionType_Sync"}}});export{i as a,C as b,m as c,x as d};
