import{a as v,b as N}from"./chunk-4RFRWT4W.js";import"./chunk-O5AT6KUC.js";import"./chunk-FT2MXX5O.js";import{a as k,b as M}from"./chunk-4UZNKZHD.js";import"./chunk-AOND3IEX.js";import"./chunk-F546IYFH.js";import"./chunk-M6QXLIAS.js";import"./chunk-TCZMMRHQ.js";import"./chunk-ZQDV6VKJ.js";import"./chunk-UKCSD7AA.js";import"./chunk-JQWJUO4E.js";import"./chunk-VI2ZMLGW.js";import"./chunk-KT65IHFF.js";import"./chunk-2JML2BCE.js";import"./chunk-6VT4TDQD.js";import"./chunk-X72GXCHF.js";import"./chunk-V5K74FRB.js";import"./chunk-DOBP56NJ.js";import{a as f,b as w}from"./chunk-MBLP4DHH.js";import"./chunk-NXE5NMPT.js";import"./chunk-757BDCJT.js";import"./chunk-CU4Z4462.js";import"./chunk-FNWXX5ZV.js";import"./chunk-OS2MCOAN.js";import"./chunk-5NV2OBBJ.js";import"./chunk-R62MQIIU.js";import"./chunk-75IA4M7J.js";import"./chunk-DWUSH2HZ.js";import"./chunk-SNKEPRYU.js";import"./chunk-POVJA64S.js";import"./chunk-XS2WT5DI.js";import"./chunk-V5JCR6DH.js";import"./chunk-BXBF54ET.js";import"./chunk-TO7PC7FF.js";import"./chunk-GKFLU7NZ.js";import"./chunk-3RHWAFOM.js";import"./chunk-AJRIRLVM.js";import"./chunk-OBBCOWJF.js";import"./chunk-XFDUJZRZ.js";import"./chunk-NE65SHFM.js";import"./chunk-TMVLLXX5.js";import"./chunk-ZSHAELBT.js";import"./chunk-LEYQOZ37.js";import"./chunk-DM2XHT3H.js";import"./chunk-K7KAWBRR.js";import"./chunk-BH5IBE7S.js";import"./chunk-2Z3VA76N.js";import"./chunk-XUUJ7NBF.js";import"./chunk-QH7CG3K4.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-CMGJXTS5.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import{P as g,Q as S}from"./chunk-INYFUTXB.js";import{o as u,v as H}from"./chunk-FNS7DN6T.js";import"./chunk-IS75LVR4.js";import"./chunk-LGDTZDAH.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import{a as b,b as _}from"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Gb as a,Hc as l,J as r,Rb as d,Sd as y,Vb as p,ia as t,kc as c,oa as D,x as s,xc as m,yc as h}from"./chunk-XJKQ67XR.js";import{e as C}from"./chunk-SZAW6ZBW.js";var K,T=C(()=>{D();M();_();N();y();H();S();w();K=(()=>{class i extends b{constructor(e,n,o){super(),this.route=e,this.displayModeService=n,this.connectionHandbookNavigationService=o,this.permissions=new k,this.displayModeService.isKioskMode=!0}ngOnInit(){this.route.params.pipe(t(this.destroyed$),r(e=>!!e.id)).subscribe(e=>this.handbookId=e.id),this.route.queryParams.pipe(t(this.destroyed$)).subscribe(e=>this.pageId=e.page||this.handbookId),this.route.data.pipe(t(this.destroyed$),s(e=>e.permissions)).subscribe(e=>this.permissions=e)}onPageChange(e){this.connectionHandbookNavigationService.navigateToDocumentation(this.handbookId,e,!1)}static{this.\u0275fac=function(n){return new(n||i)(a(u),a(g),a(f))}}static{this.\u0275cmp=d({type:i,selectors:[["ng-component"]],features:[p],decls:1,vars:5,consts:[[3,"pageChange","handbookId","pageId","canEditHandbook","canDeleteHandbook","allowOpenInNewTab"]],template:function(n,o){n&1&&(m(0,"dvls-doc-handbook-dashboard",0),l("pageChange",function(I){return o.onPageChange(I)}),h()),n&2&&c("handbookId",o.handbookId)("pageId",o.pageId)("canEditHandbook",o.permissions==null?null:o.permissions.canEditHandbook)("canDeleteHandbook",o.permissions==null?null:o.permissions.canDeleteHandbook)("allowOpenInNewTab",!1)},dependencies:[v],encapsulation:2})}}return i})()});T();export{K as ConnectionDocumentationComponent};
