import{Rb as t,Sd as a}from"./chunk-XJKQ67XR.js";import{e as i}from"./chunk-SZAW6ZBW.js";var m,p=i(()=>{a();m=(()=>{class e{static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275cmp=t({type:e,selectors:[["ng-component"]],decls:0,vars:0,template:function(n,c){},encapsulation:2})}}return e})()});p();export{m as LandingPageComponent};
