import{Sd as a,Ub as i}from"./chunk-XJKQ67XR.js";import{e as s}from"./chunk-SZAW6ZBW.js";var o,n=s(()=>{a();o=(()=>{class e{transform(t){return t=t.replace(/(?!^)[A-Z]/g,r=>`-${r.toLowerCase()}`),t.replace(/[\s\.]/g,"-")}static{this.\u0275fac=function(r){return new(r||e)}}static{this.\u0275pipe=i({name:"sanitizeCSSClass",type:e,pure:!0})}}return e})()});export{o as a,n as b};
