import{i as e,j as t}from"./chunk-2JML2BCE.js";import{e as o}from"./chunk-SZAW6ZBW.js";var l=o(()=>{t();e.setLanguage(["ko","ko-KR"],{Markdown:"\uB9C8\uD06C\uB2E4\uC6B4",WYSIWYG:"\uC704\uC9C0\uC705",Write:"\uD3B8\uC9D1\uD558\uAE30",Preview:"\uBBF8\uB9AC\uBCF4\uAE30",Headings:"\uC81C\uBAA9\uD06C\uAE30",Paragraph:"\uBCF8\uBB38",Bold:"\uAD75\uAC8C",Italic:"\uAE30\uC6B8\uC784\uAF34",Strike:"\uCDE8\uC18C\uC120",Code:"\uC778\uB77C\uC778 \uCF54\uB4DC",Line:"\uBB38\uB2E8\uB098\uB214",Blockquote:"\uC778\uC6A9\uAD6C","Unordered list":"\uAE00\uBA38\uB9AC \uAE30\uD638","Ordered list":"\uBC88\uD638 \uB9E4\uAE30\uAE30",Task:"\uCCB4\uD06C\uBC15\uC2A4",Indent:"\uB4E4\uC5EC\uC4F0\uAE30",Outdent:"\uB0B4\uC5B4\uC4F0\uAE30","Insert link":"\uB9C1\uD06C \uC0BD\uC785","Insert CodeBlock":"\uCF54\uB4DC\uBE14\uB7ED \uC0BD\uC785","Insert table":"\uD45C \uC0BD\uC785","Insert image":"\uC774\uBBF8\uC9C0 \uC0BD\uC785",Heading:"\uC81C\uBAA9","Image URL":"\uC774\uBBF8\uC9C0 \uC8FC\uC18C","Select image file":"\uC774\uBBF8\uC9C0 \uD30C\uC77C\uC744 \uC120\uD0DD\uD558\uC138\uC694.","Choose a file":"\uD30C\uC77C \uC120\uD0DD","No file":"\uC120\uD0DD\uB41C \uD30C\uC77C \uC5C6\uC74C",Description:"\uC124\uBA85",OK:"\uD655\uC778",More:"\uB354 \uBCF4\uAE30",Cancel:"\uCDE8\uC18C",File:"\uD30C\uC77C",URL:"\uC8FC\uC18C","Link text":"\uB9C1\uD06C \uD14D\uC2A4\uD2B8","Add row to up":"\uC704\uC5D0 \uD589 \uCD94\uAC00","Add row to down":"\uC544\uB798\uC5D0 \uD589 \uCD94\uAC00","Add column to left":"\uC67C\uCABD\uC5D0 \uC5F4 \uCD94\uAC00","Add column to right":"\uC624\uB978\uCABD\uC5D0 \uC5F4 \uCD94\uAC00","Remove row":"\uD589 \uC0AD\uC81C","Remove column":"\uC5F4 \uC0AD\uC81C","Align column to left":"\uC5F4 \uC67C\uCABD \uC815\uB82C","Align column to center":"\uC5F4 \uAC00\uC6B4\uB370 \uC815\uB82C","Align column to right":"\uC5F4 \uC624\uB978\uCABD \uC815\uB82C","Remove table":"\uD45C \uC0AD\uC81C","Would you like to paste as table?":"\uD45C\uD615\uD0DC\uB85C \uBD99\uC5EC \uB123\uACA0\uC2B5\uB2C8\uAE4C?","Text color":"\uAE00\uC790 \uC0C9\uC0C1","Auto scroll enabled":"\uC790\uB3D9 \uC2A4\uD06C\uB864 \uCF1C\uC9D0","Auto scroll disabled":"\uC790\uB3D9 \uC2A4\uD06C\uB864 \uAEBC\uC9D0","Choose language":"\uC5B8\uC5B4 \uC120\uD0DD"})});l();
