import{b as n,c}from"./chunk-IBUDDWQO.js";import{Gb as i,Sd as o,Ub as a}from"./chunk-XJKQ67XR.js";import{e as s}from"./chunk-SZAW6ZBW.js";var l,m=s(()=>{o();c();l=(()=>{class t{constructor(r){this.translatorService=r}transform(r,e){return this.translatorService.translateEnum(e,r)}static{this.\u0275fac=function(e){return new(e||t)(i(n,16))}}static{this.\u0275pipe=a({name:"translateEnum",type:t,pure:!0})}}return t})()});export{l as a,m as b};
