import{e as o}from"./chunk-SZAW6ZBW.js";function l(e=0){return function(a,r,t){let n=t.value;return t.value=function(...u){setTimeout(()=>{n.apply(this,u)},e)},t}}var i=o(()=>{});export{l as a,i as b};
