import{Sd as i,ta as r}from"./chunk-XJKQ67XR.js";import{e as o}from"./chunk-SZAW6ZBW.js";var f,a=o(()=>{i();f=(()=>{class t{static{this.\u0275fac=function(e){return new(e||t)}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{f as a,a as b};
