import{V as a,W as u}from"./chunk-INYFUTXB.js";import{Gb as t,Sd as s,Ub as i}from"./chunk-XJKQ67XR.js";import{e as n}from"./chunk-SZAW6ZBW.js";var m,o=n(()=>{s();u();m=(()=>{class e{constructor(r){this.currentUserService=r}transform(r){switch(this.currentUserService.currentUserSnapshot.culture){case"fr":return r+"?l=fr ";default:return r}}static{this.\u0275fac=function(c){return new(c||e)(t(a,16))}}static{this.\u0275pipe=i({name:"languageQueryParam",type:e,pure:!0})}}return e})()});export{m as a,o as b};
