import{Rb as n,Sd as c,xc as i,yc as o,zc as d}from"./chunk-XJKQ67XR.js";import{e as l}from"./chunk-SZAW6ZBW.js";var p,a=l(()=>{c();p=(()=>{class e{static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275cmp=n({type:e,selectors:[["dvl-redirecting"]],decls:3,vars:0,consts:[["id","dvl-loading",1,"loader"],[1,"loader-10x"],[1,"di","dvl-icon-logo-dvls"]],template:function(t,s){t&1&&(i(0,"div",0),d(1,"div",1)(2,"i",2),o())},encapsulation:2})}}return e})()});a();export{p as RedirectingComponent};
