import{d as q,e as V,i as L,j as z}from"./chunk-QH7CG3K4.js";import{Ha as O,Ia as _,La as A}from"./chunk-5YPL7P3G.js";import{D as N,K as $,m as x,p as E,s as k,t as w}from"./chunk-HJI5FNS6.js";import{Bb as i,Cc as y,Ic as g,Jc as Q,Kc as B,Oa as T,Rb as I,Rc as m,Sb as M,Sd as j,Tc as f,Uc as u,Vb as S,Xb as c,jc as p,kc as r,kd as F,oc as D,ta as v,ua as h,xc as s,yc as d,za as C}from"./chunk-XJKQ67XR.js";import{e as P}from"./chunk-SZAW6ZBW.js";function U(e,l){e&1&&y(0)}function W(e,l){if(e&1&&(s(0,"div",4),c(1,U,1,0,"ng-container",5),d()),e&2){let t=g();p("data-pc-section","start"),i(),r("ngTemplateOutlet",t.startTemplate||t._startTemplate)}}function X(e,l){e&1&&y(0)}function Y(e,l){if(e&1&&(s(0,"div",6),c(1,X,1,0,"ng-container",5),d()),e&2){let t=g();p("data-pc-section","center"),i(),r("ngTemplateOutlet",t.centerTemplate||t._centerTemplate)}}function Z(e,l){e&1&&y(0)}function ee(e,l){if(e&1&&(s(0,"div",7),c(1,Z,1,0,"ng-container",5),d()),e&2){let t=g();p("data-pc-section","end"),i(),r("ngTemplateOutlet",t.endTemplate||t._endTemplate)}}var G,H,J,K,te,ne,R,ae,ve,oe=P(()=>{$();$();j();j();A();z();V();G=["start"],H=["end"],J=["center"],K=["*"];te=({dt:e})=>`
.p-toolbar {
    display: flex;
    align-items: center;
    justify-content: space-between;
    flex-wrap: wrap;
    padding: ${e("toolbar.padding")};
    background: ${e("toolbar.background")};
    border: 1px solid ${e("toolbar.border.color")};
    color: ${e("toolbar.color")};
    border-radius: ${e("toolbar.border.radius")};
    gap: ${e("toolbar.gap")};
}

.p-toolbar-start,
.p-toolbar-center,
.p-toolbar-end {
    display: flex;
    align-items: center;
}
`,ne={root:"p-toolbar p-component",start:"p-toolbar-start",center:"p-toolbar-center",end:"p-toolbar-end"},R=(()=>{class e extends q{name="toolbar";theme=te;classes=ne;static \u0275fac=(()=>{let t;return function(n){return(t||(t=T(e)))(n||e)}})();static \u0275prov=v({token:e,factory:e.\u0275fac})}return e})(),ae=(()=>{class e extends L{style;styleClass;ariaLabelledBy;_componentStyle=C(R);getBlockableElement(){return this.el.nativeElement.children[0]}startTemplate;endTemplate;centerTemplate;templates;_startTemplate;_endTemplate;_centerTemplate;ngAfterContentInit(){this.templates.forEach(t=>{switch(t.getType()){case"start":case"left":this._startTemplate=t.template;break;case"end":case"right":this._endTemplate=t.template;break;case"center":this._centerTemplate=t.template;break}})}static \u0275fac=(()=>{let t;return function(n){return(t||(t=T(e)))(n||e)}})();static \u0275cmp=I({type:e,selectors:[["p-toolbar"]],contentQueries:function(a,n,b){if(a&1&&(m(b,G,4),m(b,H,4),m(b,J,4),m(b,O,4)),a&2){let o;f(o=u())&&(n.startTemplate=o.first),f(o=u())&&(n.endTemplate=o.first),f(o=u())&&(n.centerTemplate=o.first),f(o=u())&&(n.templates=o)}},inputs:{style:"style",styleClass:"styleClass",ariaLabelledBy:"ariaLabelledBy"},features:[F([R]),S],ngContentSelectors:K,decls:5,vars:9,consts:[["role","toolbar",3,"ngClass","ngStyle"],["class","p-toolbar-start",4,"ngIf"],["class","p-toolbar-center",4,"ngIf"],["class","p-toolbar-end",4,"ngIf"],[1,"p-toolbar-start"],[4,"ngTemplateOutlet"],[1,"p-toolbar-center"],[1,"p-toolbar-end"]],template:function(a,n){a&1&&(Q(),s(0,"div",0),B(1),c(2,W,2,2,"div",1)(3,Y,2,2,"div",2)(4,ee,2,2,"div",3),d()),a&2&&(D(n.styleClass),r("ngClass","p-toolbar p-component")("ngStyle",n.style),p("aria-labelledby",n.ariaLabelledBy)("data-pc-name","toolbar"),i(2),r("ngIf",n.startTemplate||n._startTemplate),i(),r("ngIf",n.centerTemplate||n._centerTemplate),i(),r("ngIf",n.endTemplate||n._endTemplate))},dependencies:[N,x,E,w,k,_],encapsulation:2,changeDetection:0})}return e})(),ve=(()=>{class e{static \u0275fac=function(a){return new(a||e)};static \u0275mod=M({type:e});static \u0275inj=h({imports:[ae,_,_]})}return e})()});export{ae as a,ve as b,oe as c};
