import{e as t,j as e}from"./chunk-SZAW6ZBW.js";var r,l=t(()=>{r=class{static load(){return e(this,null,function*(){try{let n=(yield import("./chunk-CG7KKPP5.js")).RecordingPlayerComponent;return n||(console.error("RecordingPlayerComponent not found in the loaded module"),null)}catch(o){return console.error("Failed to load RecordingPlayerComponent:",o),null}})}}});export{r as a,l as b};
