import{a as d,b as P}from"./chunk-4FQJH7SK.js";import{a as c,b as f}from"./chunk-GKFLU7NZ.js";import{V as u,W as b}from"./chunk-INYFUTXB.js";import{o as n,v as l}from"./chunk-FNS7DN6T.js";import{Gb as e,Sd as g,Tb as o,Vb as a,ia as s,oa as p}from"./chunk-XJKQ67XR.js";import{e as m}from"./chunk-SZAW6ZBW.js";var C,S=m(()=>{P();p();g();b();l();f();C=(()=>{class t extends d{constructor(i,r,h){super(),this.currentUserService=i,this.route=r,this.longOperationIndicatorService=h,this.defaultSortOrder=1}loadList(){this.subscription&&(this.subscription.unsubscribe(),this.subscription=null),this.longOperationIndicatorService.showIndicator(),this.getList()}buildParamsFromQueryParams(i){this.parameters=this.buildQueryParams(i),this.paging=this.buildPagingFromQueryParams(i)}subscribeToQueryParams(){this.route.queryParams.pipe(s(this.destroyed$)).subscribe(i=>{this.buildParamsFromQueryParams(i),this.loadList()})}buildPagingFromQueryParams(i){return{pageNumber:i.pagenumber&&(!this.totalCount||this.paging.pageSize<this.totalCount)?+i.pagenumber:1,pageSize:+i.pagesize||this.currentUserService.webSettingsSnapshot.defaultPageSize,sortOrder:i&&i.sortorder?+i.sortorder:this.defaultSortOrder,sortField:i.sortfield}}static{this.\u0275fac=function(r){return new(r||t)(e(u),e(n),e(c))}}static{this.\u0275dir=o({type:t,features:[a]})}}return t})()});export{C as a,S as b};
