import{a as y,b as q}from"./chunk-WWBMXDWO.js";import{a as O,b as Q}from"./chunk-R2P5HLZV.js";import"./chunk-AOND3IEX.js";import"./chunk-F546IYFH.js";import{a as I,b as w}from"./chunk-4FQJH7SK.js";import"./chunk-UKCSD7AA.js";import{a as S,b as k,c as _}from"./chunk-FEY6ZSY7.js";import"./chunk-5NV2OBBJ.js";import"./chunk-EXSN6QWV.js";import{a as b,b as j}from"./chunk-BH5IBE7S.js";import"./chunk-QH7CG3K4.js";import{a as P,b as B}from"./chunk-CMGJXTS5.js";import"./chunk-INYFUTXB.js";import{q as C,v as T}from"./chunk-FNS7DN6T.js";import"./chunk-IS75LVR4.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import{b as h,c as A}from"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Bb as o,Cc as f,Gb as r,Hc as p,Lc as s,Rb as d,Sd as x,Vb as g,kc as u,vd as m,wd as c,xc as i,yc as l,zc as v}from"./chunk-XJKQ67XR.js";import{e as D}from"./chunk-SZAW6ZBW.js";var N,z=D(()=>{w();Q();_();B();j();x();A();q();T();_();N=(()=>{class n extends I{constructor(a,e,t){super(),this.translatorService=a,this.pamOnboardingSharedDataService=e,this.router=t,this.titleHeader=[{title:this.translatorService.translate("lblAdministration"),link:"/administration/"},{title:this.translatorService.translate("lblPrivilegedAccessManagement"),link:"/administration/pam/"},{title:this.translatorService.translate("lblPamQuickStart")}]}getComponentTitle(){return"lblPamQuickStart"}ngOnInit(){super.ngOnInit(),this.pamOnboardingSharedDataService.clear()}onContinueClick(){this.router.navigate(["/pam/quickstart/setup"])}static{this.\u0275fac=function(e){return new(e||n)(r(h),r(y),r(C))}}static{this.\u0275cmp=d({type:n,selectors:[["ng-component"]],features:[g],decls:9,vars:7,consts:[[3,"titlePage"],["header-right",""],["severity","info",3,"text"],[1,"page-container"],[1,"di","dvl-icon-quick-start","onboarding-icon",3,"click"],["size","md","variant","primary",3,"onClick","label"]],template:function(e,t){e&1&&(i(0,"dvl-page-wrapper-v2",0),f(1,1),i(2,"div"),v(3,"p-message",2),m(4,"translate"),i(5,"div",3)(6,"i",4),p("click",function(){return t.onContinueClick()}),l(),i(7,"dvl-button",5),m(8,"translate"),p("onClick",function(){return t.onContinueClick()}),l()()()()),e&2&&(u("titlePage",t.titleHeader),o(3),s("text",c(4,3,"PAMQuickStart_Intro_Details")),o(4),s("label",c(8,5,"lblContinue")))},dependencies:[O,k,S,b,P],styles:[".page-container[_ngcontent-%COMP%]{padding-top:20px;display:flex;flex-direction:column;align-items:center}.onboarding-icon[_ngcontent-%COMP%]{font-size:300px}[_nghost-%COMP%]     .continue-button{width:200px!important;height:60px!important}"]})}}return n})()});z();export{N as PamOnboardingIntroPageComponent};
