import{K as S,m as P}from"./chunk-HJI5FNS6.js";import{$c as y,Bb as i,Ic as s,Rb as M,Sd as h,Xb as c,_c as C,ad as O,kc as g,nd as u,sc as m,uc as p,vc as _,wc as d,xc as o,yc as a,zc as v}from"./chunk-XJKQ67XR.js";import{e as x}from"./chunk-SZAW6ZBW.js";var l,b=x(()=>{l=function(e){return e[e.Error=0]="Error",e[e.Warning=1]="Warning",e}(l||{})});function w(e,r){if(e&1&&(o(0,"li",3),C(1),a()),e&2){let n=r.$implicit,t=s(4);g("ngClass",u(2,E,t.isWarning,t.isError)),i(),O(" ",n," ")}}function F(e,r){if(e&1&&(o(0,"span")(1,"ul"),_(2,w,2,5,"li",3,p),a()()),e&2){let n=s().$implicit;i(2),d(n.subMessages)}}function k(e,r){if(e&1&&(o(0,"span")(1,"p",0),v(2,"i",1),o(3,"label",2),C(4),a()(),c(5,F,4,0,"span"),a()),e&2){let n=r.$implicit,t=s(2);i(),g("ngClass",u(3,E,t.isWarning,t.isError)),i(3),y(n.message),i(),m(n.subMessages.length>0?5:-1)}}function W(e,r){if(e&1&&(o(0,"span"),_(1,k,6,6,"span",null,p),a()),e&2){let n=s();i(),d(n.messages)}}var E,N,I=x(()=>{b();S();h();E=(e,r)=>({warning:e,error:r});N=(()=>{class e{constructor(){this.messages=[],this.type=l.Error}get isWarning(){return this.type===l.Warning}get isError(){return this.type===l.Error}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275cmp=M({type:e,selectors:[["sd-message-summary"]],inputs:{messages:"messages",type:"type"},decls:1,vars:1,consts:[[1,"dvl-form-message-summary",3,"ngClass"],[1,"di","dvl-icon-notification"],[1,"message-label"],[3,"ngClass"]],template:function(t,f){t&1&&c(0,W,3,0,"span"),t&2&&m((f.messages==null?null:f.messages.length)>0?0:-1)},dependencies:[P],styles:["ul[_ngcontent-%COMP%]{margin-left:15px;padding-left:20px;margin-top:5px}.message-label[_ngcontent-%COMP%]{align-self:center;margin-left:5px}li[_ngcontent-%COMP%]{line-height:1.5em}li.error[_ngcontent-%COMP%]{color:rgba(var(--status-banner-error-color-rgb),1)}li.warning[_ngcontent-%COMP%]{color:orange}p[_ngcontent-%COMP%]{padding:10px 16px;margin:0;display:flex}p.error[_ngcontent-%COMP%]{background:var(--message-error-background-color)}p.error[_ngcontent-%COMP%]   .message-label[_ngcontent-%COMP%], p.error[_ngcontent-%COMP%]   .dvl-icon[_ngcontent-%COMP%]{color:var(--message-error-color)}p.warning[_ngcontent-%COMP%]{background:var(--message-warn-background-color)}p.warning[_ngcontent-%COMP%]   .message-label[_ngcontent-%COMP%], p.warning[_ngcontent-%COMP%]   .dvl-icon[_ngcontent-%COMP%]{color:var(--message-warn-color)}"]})}}return e})()});export{N as a,I as b};
