import{c as S,d as w}from"./chunk-VXWUYKXC.js";import{Wj as f,Xj as k,a as h,b}from"./chunk-2GUKPKKM.js";import{g as d,h as T}from"./chunk-CUIDVWNH.js";import{Da as u,La as j}from"./chunk-5YPL7P3G.js";import{b as m,c as G}from"./chunk-IBUDDWQO.js";import{Sd as A,ka as c,oa as y,ta as p,x as a,ya as s}from"./chunk-XJKQ67XR.js";import{e as I}from"./chunk-SZAW6ZBW.js";var O,x=I(()=>{y();w();b();A();T();G();k();j();O=(()=>{class i extends h{constructor(r,e,t,l){super(e,r),this.assetServices=t,this.messageService=l}getAzureRoles(r,e){return this.assetServices.GetRoleTemplates(r||null,e.clientId||null,e.tenantId||null,e.secretClientKey||null).pipe(c(t=>this.handleGetApiResult(t)),a(t=>this.returnWhenSuccessful(t,()=>t.data)))}getPagedAzureGroups(r,e,t,l,v,g){let n=new S;return n.pageSize=l,n.nextPageToken=v||null,n.filter=g||null,this.assetServices.GetGroupsPaged(n,t,r||null,e.clientId||null,e.tenantId||null,e.secretClientKey||null,e.linkedCredentialId||null).pipe(c(o=>this.executeBasedOnResult(o,()=>void 0,()=>this.showErrorToast())),a(o=>this.returnWhenSuccessful(o,()=>o.data)))}showErrorToast(){this.messageService.add({key:"globalToast",severity:"error",summary:this.translatorService.translate("Failed"),detail:this.translatorService.translate("ConnectionFailed")})}static{this.\u0275fac=function(e){return new(e||i)(s(d),s(m),s(f),s(u))}}static{this.\u0275prov=p({token:i,factory:i.\u0275fac,providedIn:"root"})}}return i})()});export{O as a,x as b};
