import{V as o,W as f}from"./chunk-INYFUTXB.js";import{q as c,v as u}from"./chunk-FNS7DN6T.js";import{Sd as a,ta as s,ya as i}from"./chunk-XJKQ67XR.js";import{e as n}from"./chunk-SZAW6ZBW.js";var d,m=n(()=>{a();f();u();d=(()=>{class t{constructor(e,r){this.currentUserService=e,this.router=r}canActivate(e,r){return!this.currentUserService.isAdministrator&&!this.currentUserService.accessSnapshot.hasAccessPam?(this.router.navigate(["/connections"]),!1):!0}static{this.\u0275fac=function(r){return new(r||t)(i(o),i(c))}}static{this.\u0275prov=s({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{d as a,m as b};
