import{e as s}from"./chunk-SZAW6ZBW.js";var m,C=s(()=>{m=function(n){return n[n.All=0]="All",n[n.InRange=1]="InRange",n[n.OutOfRange=2]="OutOfRange",n}(m||{});(function(n){function f(){return"LastLoginDateFilter"}n.getClass=f;function c(l){let u="";return typeof n.getClass=="function"&&(u=n.getClass()),u+"_"+n[l]}n.getEnumKey=c})(m||(m={}))});var r,O=s(()=>{r=function(n){return n[n.All=0]="All",n[n.InRange=1]="InRange",n[n.OutOfRange=2]="OutOfRange",n}(r||{});(function(n){function f(){return"DateRangeType"}n.getClass=f;function c(l){let u="";return typeof n.getClass=="function"&&(u=n.getClass()),u+"_"+n[l]}n.getEnumKey=c})(r||(r={}))});export{m as a,C as b,r as c,O as d};
