import{Sd as n,g as s,na as c,ta as i}from"./chunk-XJKQ67XR.js";import{e as o}from"./chunk-SZAW6ZBW.js";var h,a=o(()=>{c();n();h=(()=>{class t{constructor(){this.responseRequestSource=new s,this.createRequestSource=new s,this.viewPermissionSetDetailsRequestSource=new s,this.responseRequest=this.responseRequestSource.asObservable(),this.createRequest=this.createRequestSource.asObservable(),this.viewPermissionSetDetailsRequest=this.viewPermissionSetDetailsRequestSource.asObservable()}editResponse(e){this.responseRequestSource.next(e)}create(e){this.createRequestSource.next(e)}viewPermissionSetDetails(e){this.viewPermissionSetDetailsRequestSource.next(e)}static{this.\u0275fac=function(r){return new(r||t)}}static{this.\u0275prov=i({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{h as a,a as b};
