import{e as r}from"./chunk-SZAW6ZBW.js";var e,c=r(()=>{e=class t{constructor(){this.message="",this.subMessages=[]}static setMessageSummary(a){let s=new t;return s.message=a,s}}});export{e as a,c as b};
