import{V as n,W as a}from"./chunk-INYFUTXB.js";import{Sd as s,ta as i,ya as c}from"./chunk-XJKQ67XR.js";import{e as o}from"./chunk-SZAW6ZBW.js";var p,f=o(()=>{s();a();p=(()=>{class t{constructor(r){this.currentUserService=r}canActivate(r,e){return this.currentUserService.isAdministrator}static{this.\u0275fac=function(e){return new(e||t)(c(n))}}static{this.\u0275prov=i({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{p as a,f as b};
