import{b as o,c as h}from"./chunk-IBUDDWQO.js";import{Gb as a,Sd as u,Ub as n,g as i,ia as s,na as p,oa as c}from"./chunk-XJKQ67XR.js";import{e as l}from"./chunk-SZAW6ZBW.js";var b,f=l(()=>{p();c();u();h();b=(()=>{class r{constructor(t){this.translatorService=t,this.destroy$=new i,this.translatorService.updatedLang$.pipe(s(this.destroy$)).subscribe(()=>{this.translate()})}ngOnDestroy(){this.destroy$.next()}transform(t){return this.defaultOptions=JSON.parse(JSON.stringify(t)),this.options=t,this.translate()}translate(){return this.options?this.options.map(t=>(Object.prototype.hasOwnProperty.call(t,"label")&&(t.label=this.translatorService.translate(this.defaultOptions.find(e=>e.value===t.value).label)),t)):[]}static{this.\u0275fac=function(e){return new(e||r)(a(o,16))}}static{this.\u0275pipe=n({name:"translateOptions",type:r,pure:!0})}}return r})()});export{b as a,f as b};
