import{a as C,b as F}from"./chunk-CMGJXTS5.js";import{a as v,b as D}from"./chunk-3AUKOF45.js";import{K as b,m as u}from"./chunk-HJI5FNS6.js";import{$c as r,Bb as i,Oa as m,Rb as c,Sd as y,Vb as g,_c as l,kc as h,md as f,vd as p,wd as d,xc as o,yc as s}from"./chunk-XJKQ67XR.js";import{e as I}from"./chunk-SZAW6ZBW.js";var B,N,x=I(()=>{D();b();F();y();B=e=>({"data-label-highlight":e}),N=(()=>{class e extends v{constructor(){super(...arguments),this.highlight=!1,this.divider=!1,this.translate=!1}static{this.\u0275fac=(()=>{let n;return function(t){return(n||(n=m(e)))(t||e)}})()}static{this.\u0275cmp=c({type:e,selectors:[["dvls-diagnostic-information"]],inputs:{label:"label",value:"value",highlight:"highlight",divider:"divider",translate:"translate"},features:[g],decls:9,vars:9,consts:[[1,"grid","grid-cols-12","gap-4"],[1,"col-span-12","md:col-span-5"],[3,"ngClass"],[1,"col-span-12","md:col-span-7"]],template:function(a,t){a&1&&(o(0,"div",0)(1,"div",1)(2,"label",2),l(3),p(4,"translate"),s()(),o(5,"div",3)(6,"span"),l(7),p(8,"translate"),s()()()),a&2&&(i(2),h("ngClass",f(7,B,t.highlight)),i(),r(d(4,3,t.label)),i(4),r(t.translate?d(8,5,t.value):t.value))},dependencies:[u,C],encapsulation:2})}}return e})()});export{N as a,x as b};
