import{i as e,j as o}from"./chunk-2JML2BCE.js";import{e as a}from"./chunk-SZAW6ZBW.js";var i=a(()=>{o();e.setLanguage(["pt","pt-BR"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Escrever",Preview:"Pr\xE9-visualizar",Headings:"Cabe\xE7alhos",Paragraph:"Par\xE1grafo",Bold:"Negrito",Italic:"It\xE1lico",Strike:"Tra\xE7ado",Code:"C\xF3digo",Line:"Linha",Blockquote:"Bloco de cita\xE7\xE3o","Unordered list":"Lista n\xE3o ordenada","Ordered list":"Lista ordenada",Task:"Tarefa",Indent:"Recuo \xE0 esquerda",Outdent:"Recuo \xE0 direita","Insert link":"Inserir link","Insert CodeBlock":"Inserir bloco de c\xF3digo","Insert table":"Inserir tabela","Insert image":"Inserir imagem",Heading:"T\xEDtulo","Image URL":"URL da imagem","Select image file":"Selecione um arquivo de imagem","Choose a file":"Escolha um arquivo","No file":"Nenhum arquivo",Description:"Descri\xE7\xE3o",OK:"OK",More:"Mais",Cancel:"Cancelar",File:"Arquivo",URL:"URL","Link text":"Link de texto","Add row to up":"Adicionar linha para cima","Add row to down":"Adicionar linha para baixo","Add column to left":"Adicionar coluna \xE0 esquerda","Add column to right":"Adicionar coluna \xE0 direita","Remove row":"Remover linha","Remove column":"Remover coluna","Align column to left":"Alinhar \xE0 esquerda","Align column to center":"Alinhar ao centro","Align column to right":"Alinhar \xE0 direita","Remove table":"Remover tabela","Would you like to paste as table?":"Voc\xEA gostaria de colar como mesa?","Text color":"Cor do texto","Auto scroll enabled":"Rolagem autom\xE1tica habilitada","Auto scroll disabled":"Rolagem autom\xE1tica desabilitada","Choose language":"Escolher linguagem"})});i();
