import{a as s,b as l}from"./chunk-WWBMXDWO.js";import{p as a,v as u}from"./chunk-FNS7DN6T.js";import{a as m,b as f}from"./chunk-3AUKOF45.js";import"./chunk-Z2WWNJIF.js";import"./chunk-HJI5FNS6.js";import{Rb as o,Sd as d,Vb as n,kd as i,zc as r}from"./chunk-XJKQ67XR.js";import{e as c}from"./chunk-SZAW6ZBW.js";var C,g=c(()=>{f();l();u();d();C=(()=>{class e extends m{constructor(){super()}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275cmp=o({type:e,selectors:[["ng-component"]],features:[i([s]),n],decls:1,vars:0,template:function(t,h){t&1&&r(0,"router-outlet")},dependencies:[a],encapsulation:2})}}return e})()});g();export{C as PamOnboardingDashboardPageComponent};
