import{a as _,b as R}from"./chunk-BH5IBE7S.js";import{d as f,e as v,f as C}from"./chunk-2Z3VA76N.js";import{Hc as r,Rb as c,Sd as b,Ua as s,_c as p,ed as u,fd as d,gd as g,kc as m,xc as i,yc as o}from"./chunk-XJKQ67XR.js";import{e as y}from"./chunk-SZAW6ZBW.js";var j,w=y(()=>{b();C();R();b();C();j=(()=>{class t{constructor(){this.visible=!1,this.visibleChange=new s,this.onResult=new s}onConfirm(){this.onResult.emit(!0),this.visibleChange.emit(!1)}onReject(){this.onResult.emit(!1),this.visibleChange.emit(!1)}static{this.\u0275fac=function(n){return new(n||t)}}static{this.\u0275cmp=c({type:t,selectors:[["app-request-recording-permission-dialog"]],inputs:{visible:"visible"},outputs:{visibleChange:"visibleChange",onResult:"onResult"},decls:7,vars:5,consts:[[1,"dialog-box",3,"visibleChange","visible","resizable","modal","closable","showHeader"],[1,"dialog-content"],[1,"button-group"],["variant","primary","type","button","label","Yes",3,"click"],["variant","secondary","type","button","label","No",3,"click"]],template:function(n,e){n&1&&(i(0,"p-dialog",0),g("visibleChange",function(l){return d(e.visible,l)||(e.visible=l),l}),i(1,"div",1)(2,"p"),p(3,"Do you want to record your session?"),o(),i(4,"div",2)(5,"dvl-button",3),r("click",function(){return e.onConfirm()}),o(),i(6,"dvl-button",4),r("click",function(){return e.onReject()}),o()()()()),n&2&&(u("visible",e.visible),m("resizable",!1)("modal",!1)("closable",!1)("showHeader",!1))},dependencies:[v,f,_],styles:[".dialog-box[_ngcontent-%COMP%]{width:350px;background:var(--base-background-color);color:var(--text-color--primary)}.dialog-content[_ngcontent-%COMP%]{text-align:center;margin:1rem;display:flex;flex-direction:column;justify-content:space-between;gap:2rem}.button-group[_ngcontent-%COMP%]{display:flex;flex-direction:row;justify-content:end;gap:10px}"]})}}return t})()});export{j as a,w as b};
