import{a as u,b as o}from"./chunk-XIO7RKDL.js";import{E as d,c as s}from"./chunk-K7KAWBRR.js";import{Sd as l,Tb as r,kd as a,sa as t}from"./chunk-XJKQ67XR.js";import{e as m}from"./chunk-SZAW6ZBW.js";var w,c=m(()=>{d();l();o();l();w=(()=>{class e{constructor(){this.emailHelper=new u}validate(i){return!i.value||i.value.length===0||this.emailHelper.validate(i.value)?null:{email:{valid:!1}}}static{this.\u0275fac=function(n){return new(n||e)}}static{this.\u0275dir=r({type:e,selectors:[["sd-input-email"],["","sdValidateEmail","","ngModel",""]],features:[a([{provide:s,useExisting:t(()=>e),multi:!0}])]})}}return e})()});export{w as a,c as b};
