import{i as e,j as n}from"./chunk-2JML2BCE.js";import{e as t}from"./chunk-SZAW6ZBW.js";var r=t(()=>{n();e.setLanguage(["nb","nb-NO"],{Markdown:"Funksjonaliteter",WYSIWYG:"WYSIWYG",Write:"Skriv",Preview:"Forh\xE5ndsvisning",Headings:"Overskrift",Paragraph:"Paragraf",Bold:"Fet skrift",Italic:"Italic",Strike:"Strike",Code:"Kode",Line:"Linje",Blockquote:"Blokksitat","Unordered list":"Usortert liste","Ordered list":"Sortert liste",Task:"Task",Indent:"Indent",Outdent:"Outdent","Insert link":"Sett inn lenke","Insert CodeBlock":"Sett inn CodeStreng","Insert table":"Sett inn diagram","Insert image":"Sett inn bilde",Heading:"Overskrift","Image URL":"BildeURL","Select image file":"Velg bildefil","Choose a file":"Velg en fil","No file":"Ingen fil",Description:"Beskrivelse",OK:"OK",More:"Mer",Cancel:"Angre",File:"Fil",URL:"URL","Link text":"Lenketekst","Add row to up":"Legg rad til opp","Add row to down":"Legg rad til ned","Add column to left":"Legg til kolonne til venstre","Add column to right":"Legg til kolonne til h\xF8yre","Remove row":"Fjern rad","Remove column":"Fjern kolonne","Align column to left":"Venstreorienter","Align column to center":"Senterorienter","Align column to right":"H\xF8yreorienter","Remove table":"Fjern diagram","Would you like to paste as table?":"\xD8nsker du \xE5 lime inn som et diagram?","Text color":"Tekstfarge","Auto scroll enabled":"Auto-scroll aktivert","Auto scroll disabled":"Auto-scroll deaktivert","Choose language":"Velg spr\xE5l"})});r();
