import{i as a,j as o}from"./chunk-2JML2BCE.js";import{e}from"./chunk-SZAW6ZBW.js";var i=e(()=>{o();a.setLanguage(["es","es-ES"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Escribir",Preview:"Vista previa",Headings:"Encabezados",Paragraph:"P\xE1rrafo",Bold:"Negrita",Italic:"It\xE1lica",Strike:"Tachado",Code:"C\xF3digo",Line:"L\xEDnea",Blockquote:"Cita","Unordered list":"Lista desordenada","Ordered list":"Lista ordenada",Task:"Tarea",Indent:"Sangr\xEDa",Outdent:"Saliendo","Insert link":"Insertar enlace","Insert CodeBlock":"Insertar bloque de c\xF3digo","Insert table":"Insertar tabla","Insert image":"Insertar imagen",Heading:"Encabezado","Image URL":"URL de la imagen","Select image file":"Seleccionar archivo de imagen","Choose a file":"Escoge un archivo","No file":"Ning\xFAn archivo",Description:"Descripci\xF3n",OK:"Aceptar",More:"M\xE1s",Cancel:"Cancelar",File:"Archivo",URL:"URL","Link text":"Texto del enlace","Add row to up":"Agregar fila para subir","Add row to down":"Agregar fila hacia abajo","Add column to left":"Agregar columna a la izquierda","Add column to right":"Agregar columna a la derecha","Remove row":"Eliminar fila","Remove column":"Eliminar columna","Align column to left":"Alinear a la izquierda","Align column to center":"Centrar","Align column to right":"Alinear a la derecha","Remove table":"Eliminar tabla","Would you like to paste as table?":"\xBFDesea pegar como tabla?","Text color":"Color del texto","Auto scroll enabled":"Desplazamiento autom\xE1tico habilitado","Auto scroll disabled":"Desplazamiento autom\xE1tico deshabilitado","Choose language":"Elegir idioma"})});i();
