import{a as o,b as d}from"./chunk-BJZVALMW.js";import{wc as e,xc as s}from"./chunk-2GUKPKKM.js";import{Sd as a,ta as c}from"./chunk-XJKQ67XR.js";import{e as i}from"./chunk-SZAW6ZBW.js";var m,l=i(()=>{d();s();a();m=(()=>{class r{getByCredentialType(t,n){switch(t){case e.DomainUser:return n===o.Provider?"dvl-icon-entry-domain":"dvl-icon-credentials-bold";case e.LocalUser:return n===o.Provider?"dvl-icon-entry-session-ssh":"dvl-icon-logo-account";case e.Certificate:return"dvl-icon-entry-certificate";case e.SqlServer:return"dvl-icon-sql";case e.WindowsLocalAccount:return"dvl-icon-windows";case e.AzureADUser:return"dvl-icon-logo-office-365";case e.AWSIAMUser:return"dvl-icon-aws";case e.Standalone:return"dvl-icon-cal";case e.SshKey:return"dvl-icon-ssh-key-dashboard";case e.PostgreSQL:return"dvl-icon-entry-PgAdminIII";case e.MongoDb:return"dvl-icon-mongodb";default:return"dvl-icon-credentials-bold"}}static{this.\u0275fac=function(n){return new(n||r)}}static{this.\u0275prov=c({token:r,factory:r.\u0275fac,providedIn:"root"})}}return r})()});export{m as a,l as b};
