import{a as M,b as T,c as V}from"./chunk-S4UXPA7L.js";import{a as O,b as R}from"./chunk-BH5IBE7S.js";import{a as B,b as F}from"./chunk-CMGJXTS5.js";import{Ha as P,La as E}from"./chunk-5YPL7P3G.js";import{Bb as v,Dc as u,Hc as g,Ic as l,Ka as p,La as s,Rb as D,Sc as w,Sd as Q,Tc as x,Uc as k,Xb as f,Yc as m,kc as r,ld as _,nc as h,sc as d,vd as C,wd as y,xc as a,yc as c}from"./chunk-XJKQ67XR.js";import{e as j}from"./chunk-SZAW6ZBW.js";function q(t,b){if(t&1){let e=u();a(0,"dvl-button",5),C(1,"translate"),g("onClick",function(){p(e);let i=l(2);return s(i.accept())}),c()}if(t&2){let e,n,i;l(2);let o=m(1);r("danger",(e=o.confirmation==null||o.confirmation.acceptButtonProps==null?null:o.confirmation.acceptButtonProps.danger)!==null&&e!==void 0?e:!1)("warning",(n=o.confirmation==null||o.confirmation.acceptButtonProps==null?null:o.confirmation.acceptButtonProps.warning)!==null&&n!==void 0?n:!1)("label",y(1,3,(i=o.confirmation==null?null:o.confirmation.acceptLabel)!==null&&i!==void 0?i:"lblOk"))}}function z(t,b){if(t&1){let e=u();a(0,"dvl-button",6),C(1,"translate"),g("onClick",function(){p(e);let i=l(2);return s(i.reject())}),c()}if(t&2){let e;l(2);let n=m(1);r("label",y(1,1,(e=n.confirmation==null?null:n.confirmation.rejectLabel)!==null&&e!==void 0?e:"lblClose"))}}function I(t,b){if(t&1&&f(0,q,2,5,"dvl-button",3)(1,z,2,3,"dvl-button",4),t&2){l();let e=m(1);d((e.confirmation==null?null:e.confirmation.acceptVisible)!==!1?0:-1),v(),d((e.confirmation==null?null:e.confirmation.rejectVisible)!==!1?1:-1)}}var S,A,L,X,G=j(()=>{V();F();R();Q();V();E();S=["cd"],A=()=>({width:"32rem"}),L=()=>({"1199px":"75vw","575px":"90vw"});X=(()=>{class t{constructor(){this.key="confirm-dialog",this.cancelable=!0}accept(){this.confirmDialog&&this.confirmDialog.onAccept&&this.confirmDialog.onAccept()}reject(){this.confirmDialog&&this.confirmDialog.onReject&&this.confirmDialog.onReject()}static{this.\u0275fac=function(n){return new(n||t)}}static{this.\u0275cmp=D({type:t,selectors:[["sd-confirm-dialog"]],viewQuery:function(n,i){if(n&1&&w(S,5),n&2){let o;x(o=k())&&(i.confirmDialog=o.first)}},inputs:{key:"key",cancelable:"cancelable"},decls:3,vars:7,consts:[["cd",""],[3,"breakpoints","key","styleClass"],["pTemplate","footer"],["variant","primary","type","button",3,"danger","warning","label"],["variant","tertiary","type","button",3,"label"],["variant","primary","type","button",3,"onClick","danger","warning","label"],["variant","tertiary","type","button",3,"onClick","label"]],template:function(n,i){n&1&&(a(0,"p-confirmDialog",1,0),f(2,I,2,2,"ng-template",2),c()),n&2&&(h(_(5,A)),r("breakpoints",_(6,L))("key",i.key)("styleClass",i.key))},dependencies:[T,M,P,B,O],styles:['[_nghost-%COMP%]     .p-confirm-dialog-message{white-space:pre-line}[_nghost-%COMP%]     .p-dialog-mask{z-index:9999}.confirm-dialog[_ngcontent-%COMP%]     .p-dialog .p-dialog-header{padding:21px;margin:0;color:var(--message-font-color);border-bottom:none}.confirm-dialog[_ngcontent-%COMP%]     .p-dialog .ui-confirmdialog-message{padding:0;display:inline;color:var(--message-font-color)}.confirm-dialog[_ngcontent-%COMP%]     .p-dialog .pi{margin-right:auto}.close-icon[_ngcontent-%COMP%]     .p-dialog>.ui-widget-header .p-dialog-header-icon .pi-times:before{font-family:devolutions-icons;content:"\\e93e";color:var(--message-font-color)}']})}}return t})()});export{X as a,G as b};
