import{E as g,i as f}from"./chunk-K7KAWBRR.js";import{a as v,b as k}from"./chunk-BH5IBE7S.js";import{Ac as m,Bb as l,Bc as u,Gb as a,Hc as C,Rb as r,Sd as d,Ua as i,kc as c,xc as p,yc as s}from"./chunk-XJKQ67XR.js";import{e as h}from"./chunk-SZAW6ZBW.js";var j,y=h(()=>{d();k();d();g();j=(()=>{class e{constructor(t){this.cancel=new i,this.onClick=new i,t&&(this.parent=t)}triggerCancel(t){this.cancel.emit(t),this.onClick.emit(t),this.parent&&this.parent.resetForm()}static{this.\u0275fac=function(n){return new(n||e)(a(f,9))}}static{this.\u0275cmp=r({type:e,selectors:[["sd-cancel-button"]],inputs:{parent:"parent",label:"label"},outputs:{cancel:"cancel",onClick:"onClick"},decls:2,vars:1,consts:[["type","button","variant","tertiary",3,"onClick","label"]],template:function(n,o){n&1&&(m(0),p(1,"dvl-button",0),C("onClick",function(b){return o.triggerCancel(b)}),s(),u()),n&2&&(l(),c("label",o.label))},dependencies:[v],encapsulation:2})}}return e})()});export{j as a,y as b};
