import{Sd as a,ta as c}from"./chunk-XJKQ67XR.js";import{e as n}from"./chunk-SZAW6ZBW.js";var d,r=n(()=>{a();d=(()=>{class t{copy(o){let e=document.createElement("textarea");e.style.position="absolute",e.style.left="10000px",document.body.appendChild(e),e.value=o,e.select(),document.execCommand("copy"),document.body.removeChild(e)}static{this.\u0275fac=function(e){return new(e||t)}}static{this.\u0275prov=c({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{d as a,r as b};
