import{e as z}from"./chunk-SZAW6ZBW.js";var r,v=z(()=>{r=function(s){return s[s.en_US=0]="en_US",s[s.fr=1]="fr",s[s.de=2]="de",s[s.es=3]="es",s[s.hu=4]="hu",s[s.it=5]="it",s[s.nl=6]="nl",s[s.pl=7]="pl",s[s.ru=8]="ru",s[s.sv=9]="sv",s[s.uk=10]="uk",s[s.zh_CHS=11]="zh_CHS",s[s.zh_TW=12]="zh_TW",s[s.cs=13]="cs",s}(r||{});(function(s){function l(){return"Language"}s.getClass=l;function h(f){let t="";return typeof s.getClass=="function"&&(t=s.getClass()),t+"_"+s[f]}s.getEnumKey=h;function e(f,t){let i=t;return f===1?i="fr":f===2?i="de":f===3?i="es":f===4?i="hu":f===5?i="it":f===6?i="nl":f===7?i="pl":f===8?i="ru":f===9?i="sv":f===10?i="uk":f===11?i="zh-CHS":f===12?i="zh-TW":f===13&&(i="cs"),i}s.valueOf=e;function c(f){let t=s.en_US;return f==="fr"?t=s.fr:f==="de"?t=s.de:f==="es"?t=s.es:f==="hu"?t=s.hu:f==="it"?t=s.it:f==="nl"?t=s.nl:f==="pl"?t=s.pl:f==="ru"?t=s.ru:f==="sv"?t=s.sv:f==="uk"?t=s.uk:f==="zh-CHS"?t=s.zh_CHS:f==="zh-TW"?t=s.zh_TW:f==="cs"&&(t=s.cs),t}s.fromValue=c;function o(f){return e(f,"en_US")}s.getLanguageFilename=o;function S(f){return e(f,"en-US")}s.getSettingsValue=S;function C(f,t){let i=t;return f===1?i="fr-FR":f===2?i="de-DE":f===3?i="es-ES":f===4?i=t:f===5?i="it-IT":f===6?i="nl-NL":f===7?i="pl-PL":f===8?i="ru-RU":f===9?i="sv-SE":f===10?i="uk-UA":f===11?i="zh-CN":f===12?i="zh-TW":f===13&&(i="cs-CZ"),i}s.getBCP47Value=C})(r||(r={}))});export{r as a,v as b};
