import{e as f}from"./chunk-SZAW6ZBW.js";var t,a=f(()=>{t=function(n){return n[n.DSA=0]="DSA",n[n.RSA=1]="RSA",n[n.ECDSA=2]="ECDSA",n[n.ED25519=3]="ED25519",n[n.Unknown=4]="Unknown",n}(t||{});(function(n){function o(){return"SSHKeyType"}n.getClass=o;function r(s){let u="";return typeof n.getClass=="function"&&(u=n.getClass()),u+"_"+n[s]}n.getEnumKey=r})(t||(t={}))});export{t as a,a as b};
