import{a as r,b as h}from"./chunk-ESBDAYYU.js";import"./chunk-5QS6IFSE.js";import{a as o,b as p}from"./chunk-JGUUVYPT.js";import"./chunk-MUDU7EZV.js";import"./chunk-6WF6MHOA.js";import"./chunk-FWGS4OLA.js";import"./chunk-V5K74FRB.js";import{a as t,b as e}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-4XZGH33S.js";import"./chunk-WG6KMJAB.js";import"./chunk-2T3UB3DC.js";import"./chunk-BX7D5OPW.js";import"./chunk-TL2YKB4P.js";import"./chunk-BXBF54ET.js";import"./chunk-5DATZV72.js";import"./chunk-3RHWAFOM.js";import"./chunk-JSWQM3QO.js";import"./chunk-TMVLLXX5.js";import"./chunk-ZSHAELBT.js";import"./chunk-DM2XHT3H.js";import"./chunk-K7KAWBRR.js";import"./chunk-BH5IBE7S.js";import"./chunk-2Z3VA76N.js";import"./chunk-XUUJ7NBF.js";import"./chunk-QH7CG3K4.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-CMGJXTS5.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import"./chunk-IS75LVR4.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import"./chunk-XJKQ67XR.js";import{e as m}from"./chunk-SZAW6ZBW.js";var s,i=m(()=>{e();p();h();s=[{canActivate:[t,o],providers:[r],path:"",pathMatch:"full",loadComponent:()=>import("./chunk-MXTTIE5B.js").then(a=>a.SystemDashboardComponent)}]});i();export{s as SYSDASHBOARD_ROUTES};
