import{a as g,b as M}from"./chunk-UXH2HBKA.js";import"./chunk-GKFLU7NZ.js";import{a as f,b as I}from"./chunk-EM7RCG2I.js";import{o as h,v as S}from"./chunk-FNS7DN6T.js";import"./chunk-CUIDVWNH.js";import{a as u,b as C}from"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Gb as o,J as r,Rb as d,Sd as m,T as a,Vb as p,ia as c,oa as b,x as s,za as l}from"./chunk-XJKQ67XR.js";import{e as v}from"./chunk-SZAW6ZBW.js";var W,w=v(()=>{m();b();C();I();m();S();M();W=(()=>{class t extends u{constructor(i,e){super(),this.activatedRoute=i,this.navigationService=e,this.webSessionIcon="",this.jetUrl="",this.webClientSessionService=l(f)}ngAfterViewInit(){this.subscribeToRoute()}subscribeToRoute(){this.activatedRoute.paramMap.pipe(a(1)).subscribe(i=>{this.sessionId=i.get("sessionId"),this.sessionId||this.initializePlaceholderSession(this.sessionId)})}initializePlaceholderSession(i){this.webClientSessionService.getAllWebSessions().pipe(c(this.destroyed$),s(e=>e.filter(n=>n.sessionId===i)),r(e=>e.length===0)).subscribe(()=>{this.navigationService.navigateToRoot()})}getComponentTitle(){return"lblWebApplication"}static{this.\u0275fac=function(e){return new(e||t)(o(h),o(g))}}static{this.\u0275cmp=d({type:t,selectors:[["ng-component"]],features:[p],decls:0,vars:0,template:function(e,n){},styles:['[_nghost-%COMP%]{background:var(--connection-view-default-background-color) url("./media/session_background-PMMLASQV.png");display:flex;height:100%;width:100%;flex-direction:column}.session-container[_ngcontent-%COMP%]{display:flex;flex-direction:column;height:100%}']})}}return t})()});w();export{W as WebClientPlaceholderComponent};
