import{a as n,b as e}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import"./chunk-XJKQ67XR.js";import{e as t}from"./chunk-SZAW6ZBW.js";var m,p=t(()=>{e();m=[{path:":repositoryId/:connectionId",canActivate:[n],loadComponent:()=>import("./chunk-COUEQ6PB.js").then(o=>o.WebClientComponent),children:[{path:":sessionId",loadComponent:()=>import("./chunk-26U75J4H.js").then(o=>o.WebClientPlaceholderComponent)},{path:"ard",loadComponent:()=>import("./chunk-7SJMVAI6.js").then(o=>o.WebClientArdComponent)},{path:"rdp",loadComponent:()=>import("./chunk-6PY4QK4G.js").then(o=>o.WebClientRdpComponent)},{path:"vnc",loadComponent:()=>import("./chunk-7VKEHXAP.js").then(o=>o.WebClientVncComponent)},{path:"ssh",loadComponent:()=>import("./chunk-HAEEA2IY.js").then(o=>o.WebClientSshShellComponent)},{path:"telnet",loadComponent:()=>import("./chunk-O3VMVO6R.js").then(o=>o.WebClientTelnetComponent)},{path:"powershell",loadComponent:()=>import("./chunk-7YVXGPSH.js").then(o=>o.WebClientPowerShellComponent)}]}]});p();export{m as WEB_CLIENT_ROUTES};
