import{a as n,c as s}from"./chunk-NUDGNJBM.js";import{a as t,b as r}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-ZSHAELBT.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import{Yb as i,Zb as e}from"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import"./chunk-XJKQ67XR.js";import{e as a}from"./chunk-SZAW6ZBW.js";var h,p=a(()=>{r();s();e();h=[{path:"",canActivate:[t],loadComponent:()=>import("./chunk-PVIW5PFN.js").then(o=>o.NotificationsComponent),children:[{path:"user-subscriptions",canActivate:[t,n],data:{permissions:[i.NotificationsUserView]},loadComponent:()=>import("./chunk-G4EFDMUV.js").then(o=>o.UserSubscriptionDashboardComponent)},{path:"subscriptions",canActivate:[t,n],data:{permissions:[i.NotificationsSubscriptionsView]},loadComponent:()=>import("./chunk-MHF32JW3.js").then(o=>o.SubscriptionDashboardComponent)},{path:"groups",canActivate:[t,n],data:{permissions:[i.NotificationsSubscriberGroupView]},loadComponent:()=>import("./chunk-FHKO6T7Z.js").then(o=>o.SubscriberGroupDashboardComponent)},{path:"subscribers",canActivate:[t,n],data:{permissions:[i.NotificationsSubscriberView]},loadComponent:()=>import("./chunk-TSXI7GUX.js").then(o=>o.SubscriberDashboardComponent)},{path:"notification-preferences-dashboard",loadComponent:()=>import("./chunk-FKM3YW4G.js").then(o=>o.NotificationPreferencesDashboardComponent)},{path:"",loadComponent:()=>import("./chunk-OAY33QIP.js").then(o=>o.NotificationsDashboardComponent)}]}]});p();export{h as NOTIFICATIONS_ROUTES};
