import{Wj as g,Xj as I}from"./chunk-2GUKPKKM.js";import{c as n,d as A}from"./chunk-CUIDVWNH.js";import{b as v,c as b}from"./chunk-IBUDDWQO.js";import{L as d,Sd as p,g as o,na as f,ta as u,ya as m}from"./chunk-XJKQ67XR.js";import{a as c,b as h,e as l}from"./chunk-SZAW6ZBW.js";var G,V=l(()=>{f();p();b();I();G=(()=>{class e{constructor(t,i){this.translatorService=t,this.assetServices=i,this.assignmentSelectionSource=new o,this.assignmentSelection=this.assignmentSelectionSource.asObservable()}assign(t){let i=this.getAdvanceParams(t);this.assignmentSelectionSource.next(i)}getAdvanceParams(t){switch(t.mode){case"applications":return h(c({},t),{title:this.translatorService.translate("lblApplications"),columns:[{field:"name",header:this.translatorService.translate("lblName"),sortable:!0,alwaysVisible:!0},{field:"description",header:this.translatorService.translate("lblDescription"),sortable:!0,alwaysVisible:!0},{field:"isMember",header:"",alwaysVisible:!0}],get:i=>this.assetServices.GetRepositoryApplications(i),save:(i,a)=>this.assetServices.SaveRepositoryApplications(i,a)});case"usersRoles":return h(c({},t),{title:this.translatorService.translate("lblUsersRoles"),columns:[{field:"name",header:this.translatorService.translate("lblName"),sortable:!0,alwaysVisible:!0},{field:"description",header:this.translatorService.translate("lblDescription"),sortable:!0,alwaysVisible:!0},{field:"isAdministrator",header:this.translatorService.translate("lblIsAdministrator"),sortable:!0,alwaysVisible:!0},{field:"isRole",header:this.translatorService.translate("lblType"),sortable:!0,alwaysVisible:!0},{field:"isMember",header:"",alwaysVisible:!0}],getUsersRoles:i=>d(this.assetServices.GetRepositoryUsers(i),this.assetServices.GetRepositoryRoles(i)),save:(i,a)=>this.assetServices.SaveRepositoryUsersAndRoles(i,a)})}}static{this.\u0275fac=function(i){return new(i||e)(m(v),m(g))}}static{this.\u0275prov=u({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var O,w=l(()=>{f();p();O=(()=>{class e{constructor(){this.displayDialogSource=new o,this.displayDialog=this.displayDialogSource.asObservable()}show(t){this.displayDialogSource.next(t)}static{this.\u0275fac=function(i){return new(i||e)}}static{this.\u0275prov=u({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});var r,S=l(()=>{r=function(e){return e[e.Default=0]="Default",e[e.v1=1]="v1",e[e.v2=2]="v2",e}(r||{});(function(e){function s(){return"RDMExportVersion"}e.getClass=s;function t(i){let a="";return typeof e.getClass=="function"&&(a=e.getClass()),a+"_"+e[i]}e.getEnumKey=t})(r||(r={}))});var y,D=l(()=>{A();S();y=class e{toQueryString(){let s=[];if(this.entries)for(let t=0;t<this.entries.length;t++)this.entries[t]!==null&&s.push("entries="+encodeURIComponent(this.entries[t].toString()));if(this.includeAttachments!==null&&this.includeAttachments.toString()&&s.push("includeAttachments="+encodeURIComponent(this.includeAttachments.toString())),this.includeCredentials!==null&&this.includeCredentials.toString()&&s.push("includeCredentials="+encodeURIComponent(this.includeCredentials.toString())),this.includeDocumentation!==null&&this.includeDocumentation.toString()&&s.push("includeDocumentation="+encodeURIComponent(this.includeDocumentation.toString())),this.isMultiVaults!==null&&this.isMultiVaults.toString()&&s.push("isMultiVaults="+encodeURIComponent(this.isMultiVaults.toString())),this.masterKey!==null&&this.masterKey.toString()&&s.push("masterKey="+encodeURIComponent(this.masterKey.toString())),this.removeAllGroups!==null&&this.removeAllGroups.toString()&&s.push("removeAllGroups="+encodeURIComponent(this.removeAllGroups.toString())),this.vaultIds)for(let t=0;t<this.vaultIds.length;t++)this.vaultIds[t]!==null&&s.push("vaultIds="+encodeURIComponent(this.vaultIds[t].toString()));return this.version!==null&&this.version.toString()&&s.push("version="+encodeURIComponent(this.version.toString())),s.join("&")}constructor(s=null){this.clone=()=>new e(this),this.entries=n.manageDefaultArrayValue(s,"entries",void 0),this.includeAttachments=n.manageDefaultValue(s,"includeAttachments",!1),this.includeCredentials=n.manageDefaultValue(s,"includeCredentials",!1),this.includeDocumentation=n.manageDefaultValue(s,"includeDocumentation",!1),this.isMultiVaults=n.manageDefaultValue(s,"isMultiVaults",!1),this.masterKey=n.manageDefaultValue(s,"masterKey",""),this.removeAllGroups=n.manageDefaultValue(s,"removeAllGroups",!1),this.vaultIds=n.manageDefaultArrayValue(s,"vaultIds",void 0),this.version=n.manageDefaultValue(s,"version",r.v2)}get versionKey(){return r.getEnumKey(this.version)}}});export{G as a,V as b,O as c,w as d,y as e,D as f};
