import{a as y,b}from"./chunk-XK4SWUVR.js";import{a as U,b as _}from"./chunk-IUCR25IG.js";import{a as D,b as O}from"./chunk-3HUVXGAM.js";import{wa as h,xa as k,ya as T}from"./chunk-QH7CG3K4.js";import{Bb as i,Hc as d,Ja as r,Lc as v,Rb as m,Sd as g,Ua as l,_c as f,ad as C,kc as s,pc as p,xc as a,yc as u,zc as c}from"./chunk-XJKQ67XR.js";import{e as I}from"./chunk-SZAW6ZBW.js";var q,M=I(()=>{g();O();T();_();b();g();T();q=(()=>{class n{constructor(){this.usernameClick=new l}ngOnChanges(o){let t=e=>/^\s+$/.test(e.fullName)?"":e.fullName;o.user&&this.user&&(this.username=t(this.user)||this.user.name,this.avatarSettings={gravatar:this.user.iconUrl,size:24,icon:"",name:this.user.fullName?.trim()?t(this.user):this.user.name})}onUsernameClick(){this.enableUsernameClick&&this.usernameClick&&this.usernameClick.emit(this.outputData)}static{this.\u0275fac=function(t){return new(t||n)}}static{this.\u0275cmp=m({type:n,selectors:[["dvls-col-username"]],inputs:{avatarSettings:"avatarSettings",username:"username",tooltip:"tooltip",outputData:"outputData",enableUsernameClick:"enableUsernameClick",user:"user"},outputs:{usernameClick:"usernameClick"},features:[r],decls:4,vars:7,consts:[[1,"username-container"],[3,"settings"],["tooltipPosition","bottom","dvlsTextOverflowTooltipDirective","","dvlsTooltipPositionAdjustment","",2,"padding-left","5px",3,"click","elementIsContainedInADiv","pTooltip"]],template:function(t,e){t&1&&(a(0,"div",0),c(1,"sd-avatar",1),a(2,"span",2),d("click",function(){return e.onUsernameClick()}),f(3),u()()),t&2&&(i(),s("settings",e.avatarSettings),i(),p("username-label ",e.enableUsernameClick?"cursor-pointer no-underline text-[var(--text-action)]":"",""),v("pTooltip",e.tooltip||e.username),s("elementIsContainedInADiv",!0),i(),C(" ",e.username," "))},dependencies:[D,k,h,U,y],styles:[".username-container[_ngcontent-%COMP%]{display:flex;align-items:center}.username-container[_ngcontent-%COMP%]   .username-label[_ngcontent-%COMP%]{overflow:hidden;text-overflow:ellipsis}"]})}}return n})()});export{q as a,M as b};
