import{a as R,b as rt}from"./chunk-ZTZZCUEG.js";import"./chunk-SXJPV3EP.js";import"./chunk-O3DVJVUS.js";import{a as G,b as it}from"./chunk-R2P5HLZV.js";import"./chunk-AOND3IEX.js";import"./chunk-F546IYFH.js";import"./chunk-SN34CLEC.js";import"./chunk-NMBI62AY.js";import"./chunk-FWGS4OLA.js";import"./chunk-45OT6L2A.js";import"./chunk-3Z3FF5DY.js";import{a as q,b as et}from"./chunk-4FQJH7SK.js";import"./chunk-ZQDV6VKJ.js";import"./chunk-UKCSD7AA.js";import"./chunk-IO5W55QR.js";import{a as H,b as O,c as $}from"./chunk-GUKSOC5A.js";import"./chunk-6VT4TDQD.js";import"./chunk-X72GXCHF.js";import"./chunk-V5K74FRB.js";import"./chunk-6CN6AVDY.js";import"./chunk-5OVPVT45.js";import{a as z,b as tt}from"./chunk-43YDQPPD.js";import"./chunk-4MUEJ55U.js";import"./chunk-5NV2OBBJ.js";import"./chunk-SNKEPRYU.js";import"./chunk-POVJA64S.js";import"./chunk-XS2WT5DI.js";import"./chunk-TO7PC7FF.js";import"./chunk-GKFLU7NZ.js";import"./chunk-SS6ERTE2.js";import"./chunk-A3Y77RII.js";import"./chunk-5DATZV72.js";import"./chunk-3RHWAFOM.js";import"./chunk-EXSN6QWV.js";import"./chunk-OBBCOWJF.js";import"./chunk-TMVLLXX5.js";import"./chunk-ZSHAELBT.js";import"./chunk-LEYQOZ37.js";import"./chunk-DM2XHT3H.js";import{C as M,E,g as k,i as I,n as A}from"./chunk-K7KAWBRR.js";import{a as B,b as Z}from"./chunk-BH5IBE7S.js";import"./chunk-2Z3VA76N.js";import"./chunk-XUUJ7NBF.js";import{xa as j,ya as Y}from"./chunk-QH7CG3K4.js";import{a as Q,b as X}from"./chunk-CMGJXTS5.js";import"./chunk-QQZCALHS.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import"./chunk-IS75LVR4.js";import"./chunk-IGDIZLCB.js";import{Ga as L,Ha as x}from"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import{b as N,c as K}from"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Ac as S,Bb as l,Bc as C,Dc as b,Gb as p,Hc as F,J as a,Ka as c,La as g,Lc as y,Rb as f,Sc as P,Sd as J,Tc as D,Uc as w,Vb as h,Yc as _,ia as m,kc as o,oa as W,vd as T,wd as V,x as u,xc as s,yc as d,zc as v}from"./chunk-XJKQ67XR.js";import{e as U}from"./chunk-SZAW6ZBW.js";var nt,yt,ot=U(()=>{W();E();et();x();X();it();Y();$();rt();Z();J();K();tt();$();E();nt=["currentForm"],yt=(()=>{class n extends q{constructor(t,e){super(),this.translatorService=t,this.pamSettingsService=e,this.titleHeader=[{title:this.translatorService.translate("lblAdministration"),link:"/administration/"},{title:this.translatorService.translate("lblPrivilegedAccessManagement"),link:"/administration/pam/"},{title:this.translatorService.translate("lblDefaultSettings")}],this.settings=new L,this.isFormLoaded=!1}get isDirty(){return this.currentForm?this.isFormLoaded&&this.currentForm.dirty:!1}getComponentTitle(){return"lblDefaultSettings"}ngOnInit(){super.ngOnInit()}ngAfterViewInit(){this.getSettings()}save(){return this.onSubmit().subscribe(()=>this.setFormPristine())}cancelChange(){return!1}rejectChange(){return!0}validateChange(){return this.onSubmit()}onSubmit(){return this.pamSettingsService.setSettings(this.settings).pipe(m(this.destroyed$),a(t=>!!t),u(t=>!!t))}getSettings(){this.pamSettingsService.getSettings(!0).pipe(m(this.destroyed$),a(t=>!!t)).subscribe(t=>{this.settings=t,this.setFormPristine(),this.isFormLoaded=!0})}setFormPristine(){setTimeout(()=>{this.currentForm?.form?.markAsPristine(),this.currentForm?.form?.updateValueAndValidity()})}static{this.\u0275fac=function(e){return new(e||n)(p(N),p(z))}}static{this.\u0275cmp=f({type:n,selectors:[["dvls-pam-settings-dashboard"]],viewQuery:function(e,i){if(e&1&&P(nt,5),e&2){let r;D(r=w())&&(i.currentForm=r.first)}},features:[h],decls:8,vars:8,consts:[["currentForm","ngForm"],[3,"titlePage"],["header-right",""],["variant","icon","showDelay","500","tooltipPosition","bottom","icon","di dvl-icon-save",3,"onClick","dvlTooltip","disabled"],["autocomplete","off","novalidate","",1,"full-height"],[3,"settings","form","mdColSize"]],template:function(e,i){if(e&1){let r=b();s(0,"dvl-page-wrapper-v2",1),S(1,2),s(2,"dvl-button",3),T(3,"translate"),F("onClick",function(){return c(r),g(i.save())}),d(),C(),s(4,"form",4,0)(6,"p-fieldset"),v(7,"dps-pam-settings",5),d()()()}if(e&2){let r=_(5);o("titlePage",i.titleHeader),l(2),y("dvlTooltip",V(3,6,"lblSave")),o("disabled",!i.isDirty),l(5),o("settings",i.settings)("form",r)("mdColSize",12)}},dependencies:[Q,G,B,j,O,H,M,A,k,I,R],encapsulation:2})}}return n})()});ot();export{yt as PamSettingsDashboardComponent};
