import{a as p}from"./chunk-IBUDDWQO.js";import{Sd as n,Ub as o}from"./chunk-XJKQ67XR.js";import{e as m,h as f}from"./chunk-SZAW6ZBW.js";var e,c,a=m(()=>{e=f(p());n();c=(()=>{class r{transform(t){return t?e.default.utc(t).local().fromNow():""}static{this.\u0275fac=function(i){return new(i||r)}}static{this.\u0275pipe=o({name:"fromNow",type:r,pure:!1})}}return r})()});export{c as a,a as b};
