import{a as l,b as J}from"./chunk-GWI5PXEB.js";import{a as S,b as I,c as P,d as O}from"./chunk-ROAZQWCV.js";import{c as h,d as R}from"./chunk-2NOZKQ2D.js";import{a as n,c as w}from"./chunk-NUDGNJBM.js";import"./chunk-SN34CLEC.js";import"./chunk-NMBI62AY.js";import{a as f,b as g}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-SS6ERTE2.js";import"./chunk-A3Y77RII.js";import"./chunk-5DATZV72.js";import"./chunk-QST7UV2H.js";import"./chunk-OBBCOWJF.js";import"./chunk-ZSHAELBT.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import{Yb as s,Zb as A}from"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Sd as y,ha as d,ka as v,oa as D,ta as u,x as p,ya as a}from"./chunk-XJKQ67XR.js";import{e as m}from"./chunk-SZAW6ZBW.js";var c,b=m(()=>{D();y();R();I();c=(()=>{class t{constructor(e,i){this.networkDiscoveryService=e,this.discoveryJobService=i}resolve(e,i){let C=e.paramMap.get("id"),r;return this.discoveryJobService.getJob(C).pipe(v(o=>r=o),d(()=>this.networkDiscoveryService.get(r.scanConfigurationID)),p(o=>({scanJob:r,scanConfiguration:o})))}static{this.\u0275fac=function(i){return new(i||t)(a(h),a(S))}}static{this.\u0275prov=u({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});var Y,j=m(()=>{g();J();b();O();w();A();Y=[{path:"",canActivate:[f,l],children:[{path:"scan-list",loadComponent:()=>import("./chunk-3ELJWQUG.js").then(t=>t.ScansDashboardComponent),canActivate:[n],data:{permissions:[s.PamProvidersScanView],isPamProviderDependant:!0}},{path:"scan-result/:id",loadComponent:()=>import("./chunk-IEQR6ICV.js").then(t=>t.ScanResultListPageComponent),resolve:{scanResultElements:c},providers:[P,c],canActivate:[n],data:{permissions:[s.PamProvidersScanResultsView],isPamProviderDependant:!0}},{path:"",redirectTo:"scan-list",pathMatch:"full"}]}]});j();export{Y as ACCOUNT_DISCOVERY_ROUTES};
