import{a as F,b as q}from"./chunk-AOND3IEX.js";import{a as V,b as W,c as B}from"./chunk-F546IYFH.js";import{a as j,b as N}from"./chunk-UKCSD7AA.js";import{a as R,b as D}from"./chunk-CMGJXTS5.js";import{r as I,u as k,v as O}from"./chunk-FNS7DN6T.js";import{Ha as E,La as L}from"./chunk-5YPL7P3G.js";import{K as M,m as w}from"./chunk-HJI5FNS6.js";import{Bb as n,Gb as _,Ic as d,Ja as v,Jc as y,Kc as o,Rb as C,Sd as z,Xb as m,_c as g,ad as h,kc as p,md as T,sc as c,uc as x,vc as b,vd as f,wc as P,wd as u,xc as a,yc as i,zc as S}from"./chunk-XJKQ67XR.js";import{e as $}from"./chunk-SZAW6ZBW.js";function J(e,r){if(e&1&&(a(0,"a",11),g(1),f(2,"translate"),i()),e&2){let t=d().$implicit;p("routerLink",t.link),n(),h(" ",u(2,2,t.title)," ")}}function K(e,r){if(e&1&&(a(0,"span",12),g(1),f(2,"translate"),i()),e&2){let t=d().$implicit;n(),h(" ",u(2,1,t.title)," ")}}function Q(e,r){e&1&&S(0,"span",13)}function U(e,r){if(e&1&&(a(0,"span"),m(1,J,3,4,"a",11)(2,K,3,3,"span",12)(3,Q,1,0,"span",13),i()),e&2){let t=r.$implicit,l=r.$index,s=r.$count;n(),c(t.link?1:-1),n(),c(t.link?-1:2),n(),c(l!==s-1?3:-1)}}function X(e,r){if(e&1&&(a(0,"h2",4),b(1,U,4,3,"span",null,x),i()),e&2){let t=d();p("ngClass",T(1,H,t.toolbarRightIsOverlayPanel)),n(),P(t.titles)}}function Y(e,r){e&1&&o(0,5)}var A,G,H,ce,Z=$(()=>{M();q();B();O();D();z();N();B();L();O();A=[[["","header-left",""]],[["","header",""]],[["","header-right",""]],[["","content",""]],"*",[["","footer",""]]],G=["[header-left]","[header]","[header-right]","[content]","*","[footer]"],H=e=>({"mobile-display":e});ce=(()=>{class e{constructor(t){this.screenSizeService=t,this.titles=[],this.toolbarRightIsOverlayPanel=!1}ngOnInit(){this.rightToolbarBreakpoint==="small"?(this.screenSizeService.lessThanSmall(!0).subscribe(()=>{this.toolbarRightIsOverlayPanel=!0}),this.screenSizeService.greaterThanSmall().subscribe(()=>{this.toolbarRightIsOverlayPanel=!1})):this.rightToolbarBreakpoint==="medium"&&(this.screenSizeService.lessThanMedium(!0).subscribe(()=>{this.toolbarRightIsOverlayPanel=!0}),this.screenSizeService.greaterThanMedium().subscribe(()=>{this.toolbarRightIsOverlayPanel=!1}))}ngOnChanges(t){if(!this.titlePage){this.titles=[];return}this.titlePage instanceof String||typeof this.titlePage=="string"?this.titles=[{title:this.titlePage}]:this.titles=[...this.titlePage]}static{this.\u0275fac=function(l){return new(l||e)(_(j))}}static{this.\u0275cmp=C({type:e,selectors:[["dvl-page-wrapper-v2"]],inputs:{styleClass:"styleClass",titlePage:"titlePage",rightToolbarBreakpoint:"rightToolbarBreakpoint"},features:[v],ngContentSelectors:G,decls:16,vars:4,consts:[[1,"page-content-wrapper","grid","gap-4",2,"grid-template-columns","repeat(12, minmax(0, 1fr))","grid-template-rows","auto 1fr","min-height","0",3,"ngClass"],[1,"wrapper-header",2,"grid-column","1 / -1"],[1,"page-header"],[1,"toolbar-group-left",2,"white-space","nowrap"],[3,"ngClass"],[1,"flex"],[1,"toolbar-group-right","flex",2,"white-space","nowrap"],["switchClass","button-action","switchIcon","di dvl-icon-more-vert",3,"isPanel","switchLabel"],[1,"flex","items-center"],[1,"wrapper-content",2,"grid-column","1 / -1","min-height","0"],["pTemplate","footer"],[1,"text-2xl","!text-[var(--text-300)]",3,"routerLink"],[1,"text-2xl","text-[var(--text-300)]"],[1,"text-[var(--text-300)]","di","text-2xl","dvl-icon-arrow-right-medium","arrow","page-header-arrow"]],template:function(l,s){l&1&&(y(A),a(0,"div",0)(1,"div",1)(2,"p-toolbar",2)(3,"div",3),m(4,X,3,3,"h2",4),a(5,"div",5),o(6),i(),o(7,1),i(),a(8,"div",6)(9,"dvl-overlay-panel-toggle",7)(10,"div",8),o(11,2),i()()()()(),a(12,"div",9),o(13,3),o(14,4),i(),m(15,Y,1,0,"ng-template",10),i()),l&2&&(p("ngClass",s.styleClass),n(4),c(s.titles.length>0?4:-1),n(5),p("isPanel",s.toolbarRightIsOverlayPanel)("switchLabel",null))},dependencies:[w,F,W,V,E,k,I,R],encapsulation:2})}}return e})()});export{ce as a,Z as b};
