import{a as f,b as D}from"./chunk-P26FBUCV.js";import{a as u,b as C}from"./chunk-R2P5HLZV.js";import"./chunk-AOND3IEX.js";import"./chunk-F546IYFH.js";import"./chunk-IUCR25IG.js";import{a as b,b as g}from"./chunk-3Z3FF5DY.js";import"./chunk-4FQJH7SK.js";import"./chunk-UKCSD7AA.js";import"./chunk-E6LEVHML.js";import"./chunk-5NV2OBBJ.js";import"./chunk-EXSN6QWV.js";import"./chunk-BH5IBE7S.js";import"./chunk-QH7CG3K4.js";import"./chunk-CMGJXTS5.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import"./chunk-IS75LVR4.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import{b as d,c as S}from"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Bb as s,Gb as r,Rb as a,Sd as h,Vb as l,kc as o,xc as c,yc as p,zc as m}from"./chunk-XJKQ67XR.js";import{e as v}from"./chunk-SZAW6ZBW.js";var P,T=v(()=>{g();C();D();h();S();P=(()=>{class n extends b{constructor(e){super(),this.translateService=e,this.titleHeader=[{title:"lblAdministration",link:"/administration"},{title:"lblNotifications"}],this.sections=[]}ngOnInit(){super.ngOnInit(),this.createSectionsTemplate();let e=this.sectionsTemplate.filter(t=>t.items.some(i=>i.condition()));e.forEach(t=>t.items=t.items.filter(i=>i.condition())),this.sections=e}getComponentTitle(){return"lblNotifications"}createSectionsTemplate(){this.sectionsTemplate=[{label:this.translateService.translate("lblNotifications"),items:[{label:this.translateService.translate("lblSubscribers"),icon:"dvl-icon-notifications-subscriber",link:"subscribers",condition:()=>!0},{label:this.translateService.translate("lblSubscriberGroups"),icon:"dvl-icon-notifications-subscriber-group",link:"groups",condition:()=>!0},{label:this.translateService.translate("lblSubscriptions"),icon:"dvl-icon-notifications-bold",link:"subscriptions",condition:()=>!0},{label:this.translateService.translate("lblUserSubscriptions"),icon:"dvl-icon-user-bold",link:"user-subscriptions",condition:()=>!0}]}]}static{this.\u0275fac=function(t){return new(t||n)(r(d))}}static{this.\u0275cmp=a({type:n,selectors:[["ng-component"]],features:[l],decls:2,vars:2,consts:[[3,"titlePage"],[3,"sections"]],template:function(t,i){t&1&&(c(0,"dvl-page-wrapper-v2",0),m(1,"dvl-menu-dashboard",1),p()),t&2&&(o("titlePage",i.titleHeader),s(),o("sections",i.sections))},dependencies:[u,f],encapsulation:2})}}return n})()});T();export{P as NotificationsDashboardComponent};
