import{a as l,b as d}from"./chunk-EMEMSPZE.js";import{Sd as p,Ub as m,za as s}from"./chunk-XJKQ67XR.js";import{a as n,b as o,e as h}from"./chunk-SZAW6ZBW.js";var y,E=h(()=>{p();d();p();y=(()=>{class e{constructor(){this.helper=s(l)}transform(r,t=0,u=[],a=[],c=!1,f=null){return this.helper.initComboEnum(r,u,c,!1,t,f).map(i=>o(n({},i),{disabled:a.includes(i.value)}))}static{this.\u0275fac=function(t){return new(t||e)}}static{this.\u0275pipe=m({name:"getSelectItemsFromEnum",type:e,pure:!0})}}return e})()});export{y as a,E as b};
