import{a,b as h}from"./chunk-EXSN6QWV.js";import{n as r,o as F,p as o,q as d}from"./chunk-INYFUTXB.js";import{a as l}from"./chunk-IBUDDWQO.js";import{e as f,h as c}from"./chunk-SZAW6ZBW.js";var t,i,p=f(()=>{t=c(l());d();F();h();i=class{constructor(m,s,D){let e=a.injector.get(o);this.dateFilter=m||r.Last7Days,this.fromDate=e.getStartDateFromDateFilter(this.dateFilter,s,!0),this.toDate=e.getEndDateFromDateFilter(this.dateFilter,D,!0)}formattedFromDate(){return(0,t.default)(this.fromDate).format()}formattedToDate(){return(0,t.default)(this.toDate).format()}}});export{i as a,p as b};
