import{c as e,d as r}from"./chunk-CUIDVWNH.js";import{e as g}from"./chunk-SZAW6ZBW.js";var i,f=g(()=>{r();i=class u{constructor(a=null){this.gravatar=e.manageDefaultValue(a,"gravatar",null),this.icon=e.manageDefaultValue(a,"icon",null),this.name=e.manageDefaultValue(a,"name",""),this.size=e.manageDefaultValue(a,"size",35),this.imageSrc=e.manageDefaultValue(a,"imageSrc",null),this.globalImageId=e.manageDefaultValue(a,"globalImageId",null),this.imageIcon=e.manageDefaultValue(a,"imageIcon",null),this.username=e.manageDefaultValue(a,"username",null),this.badge=e.manageDefaultValue(a,"badge",null)}static fromUserListItem(a,l=24,n,t){return new u({gravatar:e.manageDefaultValue(a,"iconUrl",null),username:e.manageDefaultValue(a,"name",null),name:[a.fullName?.trim(),a.name?.trim()].find(Boolean),size:l,icon:n,badge:t})}static fromUserListItemReport(a,l=24,n,t){return new u({gravatar:e.manageDefaultValue(a,"iconUrl",null),username:e.manageDefaultValue(a,"name",null),name:[a.fullName?.trim(),a.name?.trim()].find(Boolean),size:l,icon:n,badge:t})}static fromBaseConnectionInfo(a,l=24,n,t){return new u({gravatar:e.manageDefaultValue(a,"iconUrl",null),name:e.manageDefaultValue(a,"name",null),size:l,icon:n,badge:t})}static fromRepositoryEntity(a,l=24,n,t){let m=a?.imageBytes||a?.image;return new u({name:e.manageDefaultValue(a,"name",null),imageSrc:m?`data:image/png;base64,${m}`:null,size:l,icon:n,badge:t})}}});export{i as a,f as b};
