import{a as S,b as F}from"./chunk-CMGJXTS5.js";import{c as B,d as h,g as f,h as b,i as v,s as D}from"./chunk-IS75LVR4.js";import{$c as H,Bb as s,Hc as l,Jc as p,Kc as r,Lb as y,Rb as m,Sd as u,Vc as A,Xc as w,Ya as g,_c as k,kc as x,mc as c,vd as P,wd as M,xc as n,yc as d,za as _}from"./chunk-XJKQ67XR.js";import{e as I}from"./chunk-SZAW6ZBW.js";var Q,T,E=I(()=>{u();j();u();Q=["*"],T=(()=>{class i{constructor(){this.items=y(C),this.isFused=g(!1),this.multiple=g(!0)}setActiveItem(t){!this.multiple()&&t.isItemHidden?this.items().forEach(e=>e.setItemHidden(e!==t)):t.isItemHidden=!t.isItemHidden}setActiveItemByIndex(t){let e=this.items();t>=0&&t<e.length&&this.setActiveItem(e[t])}static{this.\u0275fac=function(e){return new(e||i)}}static{this.\u0275cmp=m({type:i,selectors:[["dvl-accordion"]],contentQueries:function(e,o,a){e&1&&A(a,o.items,C,4),e&2&&w()},hostVars:2,hostBindings:function(e,o){e&2&&c("fused",o.isFused())},inputs:{isFused:[1,"isFused"],multiple:[1,"multiple"]},ngContentSelectors:Q,decls:1,vars:0,template:function(e,o){e&1&&(p(),r(0))},styles:["[_nghost-%COMP%]{display:block;margin-bottom:1.5em}.fused[_nghost-%COMP%]     .accordion-item{border:1px solid var(--border-brand-400);border-bottom:1px solid var(--border-200);box-shadow:none}.fused[_nghost-%COMP%]     .accordion-item:first-child{border-radius:8px 8px 0 0}.fused[_nghost-%COMP%]     .accordion-item:last-child{border-radius:0 0 8px 8px}.fused[_nghost-%COMP%]     .accordion-item .accordion-item-header{padding:.57em .86em}.fused[_nghost-%COMP%]     .accordion-item+.accordion-item{margin:0;border-radius:0;border-top:0 none}.fused[_nghost-%COMP%]     .accordion-item+.accordion-item:last-child{border-radius:0 0 8px 8px}"]})}}return i})()});var V,q,C,j=I(()=>{u();D();F();E();u();V=[[["",8,"title-action"]],[["",8,"action-buttons"]],"*",[["footer",8,"inner-footer"]],[["footer"]]],q=[".title-action",".action-buttons","*","footer.inner-footer","footer"],C=(()=>{class i{constructor(){this.accordion=_(T,{optional:!0,host:!0}),this.isAccordionItem=!0,this.isItemHidden=!0}toggleItem(){this.accordion?this.accordion.setActiveItem(this):this.isItemHidden=!this.isItemHidden}setItemHidden(t){this.isItemHidden=t}static{this.\u0275fac=function(e){return new(e||i)}}static{this.\u0275cmp=m({type:i,selectors:[["dvl-accordion-item"]],hostVars:2,hostBindings:function(e,o){e&2&&c("accordion-item",o.isAccordionItem)},inputs:{isItemHidden:"isItemHidden",title:"title"},ngContentSelectors:q,decls:15,vars:6,consts:[[1,"accordion-item-header","flex","items-center",3,"click"],[1,"header-title-action",3,"click"],[1,"header-title"],[1,"header-actions","ml-auto"],["tabindex","0",1,"di","dvl-icon-arrow-down-medium","toggle-button",3,"keydown.enter"],[1,"accordion-item-content"],[1,"accordion-item-footer"]],template:function(e,o){e&1&&(p(V),n(0,"div",0),l("click",function(){return o.toggleItem()}),n(1,"div",1),l("click",function(O){return O.stopPropagation()}),r(2),d(),n(3,"div",2)(4,"span"),k(5),P(6,"translate"),d()(),n(7,"div",3),r(8,1),n(9,"i",4),l("keydown.enter",function(){return o.toggleItem()}),d()()(),n(10,"div",5),r(11,2),r(12,3),d(),n(13,"div",6),r(14,4),d()),e&2&&(s(5),H(M(6,4,o.title)),s(4),c("toggle-up",!o.isItemHidden),s(),x("@openClose",o.isItemHidden?"closed":"open"))},dependencies:[S],styles:[`.accordion-item{margin-top:1.4em;display:inline-block;width:100%;background-color:var(--bg-100);border-radius:.714em}.accordion-item:first-child{margin-top:.714em}.accordion-item-header{cursor:pointer;width:100%;padding:1.071em .714em 1.071em 1.428em;gap:var(--spacing-3, 12px)}.accordion-item-header .header-actions{display:flex;gap:5px;align-items:center}.accordion-item-header .header-actions .action-buttons{display:inline-flex;white-space:nowrap}.accordion-item-header .header-actions .action-buttons:hover{background-color:var(--accent-brand-100)}.accordion-item-header .header-actions .action-buttons ::ng-deep .p-button{height:auto;width:auto}.header-title{color:var(--text-300);font-size:1.286em;font-weight:600;line-height:1.2;flex:1 auto}.accordion-item-content{color:var(--card-txt-color);font-size:1em;overflow:hidden;padding:.5em 1em 1em}.accordion-item-content>*:first-child{margin-top:0}.header-title-action{display:flex;align-items:center}.header-title-action:empty{display:none}.accordion-item-footer{border-top:1px solid var(--security-box-border-color);font-size:1em;font-weight:600;line-height:1}.accordion-item-footer:empty{display:none}.inner-footer{margin-top:10px}.toggle-button{transition:background-color .2s;border-radius:50%;font-size:1.4em;padding:.3em}.toggle-button:before{transition:transform ease-in-out .3s;display:inline-block}.toggle-button.toggle-up:before{transform:rotate(-180deg)}.accordion-item-header:hover:not(:has(.p-button:hover)) .toggle-button,.toggle-button:hover{background-color:var(--dropdown-item-hover-background-color)}
`],encapsulation:2,data:{animation:[B("openClose",[b("open",f({height:"*",paddingTop:"*",paddingBottom:"*",margin:"*"})),b("closed",f({height:"0px",paddingTop:0,paddingBottom:0,marginTop:0,marginBottom:0})),v("open => closed",[h(".2s ease-in-out")]),v("closed => open",[h(".3s ease-in-out")])])]}})}}return i})()});export{C as a,j as b,T as c,E as d};
