import{Sd as n,g as o,na as m,ta as i}from"./chunk-XJKQ67XR.js";import{e as s}from"./chunk-SZAW6ZBW.js";var c,p=s(()=>{m();n();c=(()=>{class e{constructor(){this.importRdmRequest=new o,this.importKeePassRequest=new o,this.importGoogleRequest=new o,this.importRdxRequest=new o}importRdm(t){this.importRdmRequest.next(t)}importKeePass(t){this.importKeePassRequest.next(t)}importGoogle(t){this.importGoogleRequest.next(t)}importRdx(t){this.importRdxRequest.next(t)}static{this.\u0275fac=function(r){return new(r||e)}}static{this.\u0275prov=i({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{c as a,p as b};
