import{c as o,d as n}from"./chunk-CUIDVWNH.js";import{e as r}from"./chunk-SZAW6ZBW.js";var e,s=r(()=>{n();e=class i{toQueryString(){let t=[];return this.group!==null&&this.group.toString()&&t.push("group="+encodeURIComponent(this.group.toString())),this.id!==null&&this.id.toString()&&t.push("id="+encodeURIComponent(this.id.toString())),t.join("&")}constructor(t=null){this.clone=()=>new i(this),this.group=o.manageDefaultValue(t,"group",""),this.id=o.manageDefaultValue(t,"id",void 0)}}});var u,l=r(()=>{n();s();u=class i{toQueryString(){let t=[];return this.repositoryID!==null&&this.repositoryID.toString()&&t.push("repositoryID="+encodeURIComponent(this.repositoryID.toString())),t.join("&")}constructor(t=null){this.clone=()=>new i(this),this.entries=o.manageDefaultArrayValueType(e,t,"entries"),this.repositoryID=o.manageDefaultValue(t,"repositoryID",void 0)}}});export{e as a,s as b,u as c,l as d};
