import{d as C,e as A,i as F,j as G}from"./chunk-QH7CG3K4.js";import{Ia as r,La as S}from"./chunk-5YPL7P3G.js";import{D as h,K as B}from"./chunk-HJI5FNS6.js";import{Jc as g,Kc as M,Oa as i,Rb as p,Sb as u,Sd as I,Vb as c,jc as l,kd as v,mc as f,nc as y,oc as m,ta as s,ua as a,za as d}from"./chunk-XJKQ67XR.js";import{e as j}from"./chunk-SZAW6ZBW.js";var b,x,D,H,O,N=j(()=>{B();I();I();S();G();A();b=["*"],x={root:"p-inputgroupaddon"},D=(()=>{class e extends C{name="inputgroupaddon";classes=x;static \u0275fac=(()=>{let t;return function(o){return(t||(t=i(e)))(o||e)}})();static \u0275prov=s({token:e,factory:e.\u0275fac})}return e})(),H=(()=>{class e extends F{style;styleClass;_componentStyle=d(D);get hostStyle(){return this.style}static \u0275fac=(()=>{let t;return function(o){return(t||(t=i(e)))(o||e)}})();static \u0275cmp=p({type:e,selectors:[["p-inputgroup-addon"],["p-inputGroupAddon"]],hostVars:7,hostBindings:function(n,o){n&2&&(l("data-pc-name","inputgroupaddon"),y(o.hostStyle),m(o.styleClass),f("p-inputgroupaddon",!0))},inputs:{style:"style",styleClass:"styleClass"},features:[v([D]),c],ngContentSelectors:b,decls:1,vars:0,template:function(n,o){n&1&&(g(),M(0))},dependencies:[h],encapsulation:2})}return e})(),O=(()=>{class e{static \u0275fac=function(n){return new(n||e)};static \u0275mod=u({type:e});static \u0275inj=a({imports:[H,r,r]})}return e})()});export{H as a,O as b,N as c};
