import{i as o,j as t}from"./chunk-2JML2BCE.js";import{e}from"./chunk-SZAW6ZBW.js";var a=e(()=>{t();o.setLanguage(["cs","cs-CZ"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Napsat",Preview:"N\xE1hled",Headings:"Nadpisy",Paragraph:"Odstavec",Bold:"Tu\u010Dn\xE9",Italic:"Kurz\xEDva",Strike:"P\u0159e\u0161krtnut\xE9",Code:"K\xF3d",Line:"Vodorovn\xE1 \u010D\xE1ra",Blockquote:"Citace","Unordered list":"Seznam s odr\xE1\u017Ekami","Ordered list":"\u010C\xEDslovan\xFD seznam",Task:"\xDAkol",Indent:"Zv\u011Bt\u0161it odsazen\xED",Outdent:"Zmen\u0161it odsazen\xED","Insert link":"Vlo\u017Eit odkaz","Insert CodeBlock":"Vlo\u017Eit blok k\xF3du","Insert table":"Vlo\u017Eit tabulku","Insert image":"Vlo\u017Eit obr\xE1zek",Heading:"Nadpis","Image URL":"URL obr\xE1zku","Select image file":"Vybrat obr\xE1zek","Choose a file":"Vyberte soubor","No file":"\u017D\xE1dn\xFD soubor",Description:"Popis",OK:"OK",More:"V\xEDce",Cancel:"Zru\u0161it",File:"Soubor",URL:"URL","Link text":"Text odkazu","Add row to up":"P\u0159idejte \u0159\xE1dek nahoru","Add row to down":"P\u0159idejte \u0159\xE1dek dol\u016F","Add column to left":"P\u0159idat sloupec vlevo","Add column to right":"P\u0159idat sloupec doprava","Remove row":"Odebrat \u0159\xE1dek","Remove column":"Odebrat sloupec","Align column to left":"Zarovnat vlevo","Align column to center":"Zarovnat na st\u0159ed","Align column to right":"Zarovnat vpravo","Remove table":"Odstranit tabulku","Would you like to paste as table?":"Chcete vlo\u017Eit jako tabulku?","Text color":"Barva textu","Auto scroll enabled":"Automatick\xE9 rolov\xE1n\xED zapnuto","Auto scroll disabled":"Automatick\xE9 rolov\xE1n\xED vypnuto","Choose language":"Vybrat jazyk"})});a();
