import{c as f,d as S}from"./chunk-GZ7NOJT2.js";import"./chunk-SEQ5B5J7.js";import"./chunk-CXXLKA4V.js";import"./chunk-PJG44MAK.js";import"./chunk-Q2RB6VUK.js";import"./chunk-QOEYQ4ZT.js";import"./chunk-FT2MXX5O.js";import"./chunk-AUIK75A7.js";import"./chunk-L23JU4OH.js";import"./chunk-F546IYFH.js";import{c as l,d as D}from"./chunk-L6SAMHX4.js";import"./chunk-VXW35KLU.js";import"./chunk-R6Q7WGZS.js";import"./chunk-I7QWXGCQ.js";import"./chunk-SN34CLEC.js";import"./chunk-NMBI62AY.js";import"./chunk-TCZMMRHQ.js";import"./chunk-FOSSYLCK.js";import"./chunk-6H26RJCS.js";import"./chunk-2JML2BCE.js";import"./chunk-W6WHIXFN.js";import"./chunk-V45IWNRH.js";import"./chunk-6VT4TDQD.js";import"./chunk-X72GXCHF.js";import"./chunk-V5K74FRB.js";import"./chunk-BHKVBL7P.js";import"./chunk-UK4K2VZ6.js";import"./chunk-757BDCJT.js";import"./chunk-AZ7BWNZR.js";import"./chunk-FNWXX5ZV.js";import"./chunk-4MUEJ55U.js";import"./chunk-ZHWSFSWA.js";import"./chunk-3HUVXGAM.js";import"./chunk-OV4EXBGV.js";import"./chunk-MWUYGNQS.js";import"./chunk-5NV2OBBJ.js";import"./chunk-R62MQIIU.js";import"./chunk-75IA4M7J.js";import"./chunk-DWUSH2HZ.js";import"./chunk-SNKEPRYU.js";import"./chunk-POVJA64S.js";import"./chunk-XS2WT5DI.js";import"./chunk-TO7PC7FF.js";import"./chunk-GKFLU7NZ.js";import"./chunk-SS6ERTE2.js";import"./chunk-A3Y77RII.js";import"./chunk-5DATZV72.js";import"./chunk-2JGOCWLT.js";import"./chunk-WX6FLFOB.js";import"./chunk-7JIUZH3F.js";import"./chunk-PHLM3WOL.js";import"./chunk-ALD4AWL4.js";import"./chunk-QST7UV2H.js";import"./chunk-LQXERWKD.js";import"./chunk-V36WA7UR.js";import"./chunk-VCRWDI6N.js";import"./chunk-EMEMSPZE.js";import"./chunk-AJRIRLVM.js";import"./chunk-OBBCOWJF.js";import"./chunk-XFDUJZRZ.js";import"./chunk-TMVLLXX5.js";import"./chunk-LEYQOZ37.js";import"./chunk-DM2XHT3H.js";import"./chunk-K7KAWBRR.js";import"./chunk-BH5IBE7S.js";import"./chunk-2Z3VA76N.js";import"./chunk-XUUJ7NBF.js";import"./chunk-QH7CG3K4.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-CMGJXTS5.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import{e as c,g as v,p as s,v as C}from"./chunk-FNS7DN6T.js";import"./chunk-IS75LVR4.js";import"./chunk-NCCWCQDA.js";import"./chunk-LGDTZDAH.js";import"./chunk-IGDIZLCB.js";import"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import{a as p,b as h}from"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import{b as a,c as g}from"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import{Gb as i,Rb as n,Sd as u,Vb as r,zc as m}from"./chunk-XJKQ67XR.js";import{e as d}from"./chunk-SZAW6ZBW.js";var W,T=d(()=>{h();C();D();S();u();v();g();W=(()=>{class t extends p{constructor(o,e){super(),this.titleService=o,this.translatorService=e}static{this.\u0275fac=function(e){return new(e||t)(i(c),i(a))}}static{this.\u0275cmp=n({type:t,selectors:[["ng-component"]],features:[r],decls:3,vars:0,template:function(e,b){e&1&&m(0,"router-outlet")(1,"dps-connection-comment")(2,"dvls-ticket-selector")},dependencies:[s,l,f],encapsulation:2})}}return t})()});T();export{W as WebClientComponent};
