import{a,b as u}from"./chunk-ZSHAELBT.js";import{q as c,v as l}from"./chunk-FNS7DN6T.js";import{Sd as m,ta as r,ya as n}from"./chunk-XJKQ67XR.js";import{e as f}from"./chunk-SZAW6ZBW.js";var j,p=f(()=>{u();m();l();j=(()=>{class t{constructor(e){this.router=e}navigateToDocumentation(e,o=null,s=!1){let i=`${a.location}connections/documentation/${e}`;s?(o&&(i+=`?page=${o}`),window.open(i)):this.router.navigate([i],{queryParams:{page:o}})}static{this.\u0275fac=function(o){return new(o||t)(n(c))}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{j as a,p as b};
