import{a as r,b as u,c}from"./chunk-MUDU7EZV.js";import{D as d,K as f}from"./chunk-HJI5FNS6.js";import{Ld as s,Rb as i,Sd as m,kc as p,nc as a,zc as o}from"./chunk-XJKQ67XR.js";import{e as v}from"./chunk-SZAW6ZBW.js";var D,h=v(()=>{m();f();c();m();c();D=(()=>{class n{constructor(){this.multiple=!1,this.transitionOptions="400ms cubic-bezier(0.86, 0, 0.07, 1)",this.tabindex=0}get computedStyleClass(){let t=["dvl-panel-menu"];return this.styleClass&&t.push(this.styleClass),t.join(" ")}static{this.\u0275fac=function(l){return new(l||n)}}static{this.\u0275cmp=i({type:n,selectors:[["dvl-panel-menu"]],hostAttrs:[1,"dvl-panel-menu-container","flex-shrink-0","w-[280px]","h-full","overflow-y-auto","overflow-x-hidden","border-r","px-2","py-3","bg-[var(--bg-100)]","border-[var(--border-100)]"],inputs:{model:"model",style:"style",styleClass:"styleClass",multiple:[2,"multiple","multiple",s],transitionOptions:"transitionOptions",id:"id",tabindex:"tabindex"},decls:1,vars:8,consts:[[3,"model","styleClass","multiple","transitionOptions","id","tabindex"]],template:function(l,e){l&1&&o(0,"p-panelMenu",0),l&2&&(a(e.style),p("model",e.model)("styleClass",e.computedStyleClass)("multiple",e.multiple)("transitionOptions",e.transitionOptions)("id",e.id)("tabindex",e.tabindex))},dependencies:[d,u,r],styles:[`.dvl-panel-menu .p-panelmenu .p-panelmenu-content{padding:0}.dvl-panel-menu .p-menuitem-icon{font-size:24px}.dvl-panel-menu .p-menuitem{padding-left:4px}.dvl-panel-menu .dvl-icon{font-size:12px}.dvl-panel-menu .p-panelmenu-header .p-panelmenu-header-content{display:flex;align-items:center;gap:.5rem}.dvl-panel-menu .p-panelmenu-header .p-panelmenu-header-content .p-panelmenu-header-action{order:1;flex:1 1 auto;padding:3px 4px;font-weight:700}.dvl-panel-menu .p-panelmenu-header .p-panelmenu-header-content chevrondownicon,.dvl-panel-menu .p-panelmenu-header .p-panelmenu-header-content .p-icon-wrapper{order:2;margin-left:auto;display:inline-flex}.dvl-panel-menu .p-menuitem.p-focus>.p-menuitem-content,.dvl-panel-menu .p-menuitem-content:hover{background:var(--bg-brand-300)}.dvl-panel-menu .p-menuitem.p-highlight>.p-menuitem-content{background:var(--bg-500);font-weight:600}
`],encapsulation:2})}}return n})()});export{D as a,h as b};
