import{d as R,e as K,i as V,j as L}from"./chunk-QH7CG3K4.js";import{Ha as P,Ia as g,La as J}from"./chunk-5YPL7P3G.js";import{D as q,K as A,m as E,p as N,t as O}from"./chunk-HJI5FNS6.js";import{$c as S,Ac as M,Bb as o,Bc as w,Ic as l,Jc as D,Kc as F,Ld as B,Oa as p,Rb as b,Rc as f,Sb as C,Sd as Q,Tc as m,Uc as y,Vb as $,Xb as r,_c as j,kc as i,kd as z,nc as k,oc as I,ta as v,ua as h,xc as u,yc as d,za as T,zc as x}from"./chunk-XJKQ67XR.js";import{e as G}from"./chunk-SZAW6ZBW.js";function X(e,c){if(e&1&&x(0,"span",4),e&2){let t=l(2);i("ngClass",t.icon)}}function Y(e,c){if(e&1&&(M(0),r(1,X,1,1,"span",3),w()),e&2){let t=l();o(),i("ngIf",t.icon)}}function Z(e,c){}function ee(e,c){e&1&&r(0,Z,0,0,"ng-template")}function te(e,c){if(e&1&&(u(0,"span",5),r(1,ee,1,0,null,6),d()),e&2){let t=l();o(),i("ngTemplateOutlet",t.iconTemplate||t._iconTemplate)}}var U,W,ne,ae,H,oe,be,ie=G(()=>{A();A();Q();Q();J();L();K();U=["icon"],W=["*"];ne=({dt:e})=>`
.p-tag {
    display: inline-flex;
    align-items: center;
    justify-content: center;
    background: ${e("tag.primary.background")};
    color: ${e("tag.primary.color")};
    font-size: ${e("tag.font.size")};
    font-weight: ${e("tag.font.weight")};
    padding: ${e("tag.padding")};
    border-radius: ${e("tag.border.radius")};
    gap: ${e("tag.gap")};
}

.p-tag-icon {
    font-size: ${e("tag.icon.size")};
    width: ${e("tag.icon.size")};
    height:${e("tag.icon.size")};
}

.p-tag-rounded {
    border-radius: ${e("tag.rounded.border.radius")};
}

.p-tag-success {
    background: ${e("tag.success.background")};
    color: ${e("tag.success.color")};
}

.p-tag-info {
    background: ${e("tag.info.background")};
    color: ${e("tag.info.color")};
}

.p-tag-warn {
    background: ${e("tag.warn.background")};
    color: ${e("tag.warn.color")};
}

.p-tag-danger {
    background: ${e("tag.danger.background")};
    color: ${e("tag.danger.color")};
}

.p-tag-secondary {
    background: ${e("tag.secondary.background")};
    color: ${e("tag.secondary.color")};
}

.p-tag-contrast {
    background: ${e("tag.contrast.background")};
    color: ${e("tag.contrast.color")};
}
`,ae={root:({props:e})=>["p-tag p-component",{"p-tag-info":e.severity==="info","p-tag-success":e.severity==="success","p-tag-warn":e.severity==="warn","p-tag-danger":e.severity==="danger","p-tag-secondary":e.severity==="secondary","p-tag-contrast":e.severity==="contrast","p-tag-rounded":e.rounded}],icon:"p-tag-icon",label:"p-tag-label"},H=(()=>{class e extends R{name="tag";theme=ne;classes=ae;static \u0275fac=(()=>{let t;return function(n){return(t||(t=p(e)))(n||e)}})();static \u0275prov=v({token:e,factory:e.\u0275fac})}return e})(),oe=(()=>{class e extends V{get style(){return this._style}set style(t){this._style=t,this.cd.markForCheck()}styleClass;severity;value;icon;rounded;iconTemplate;templates;_iconTemplate;_style;_componentStyle=T(H);ngAfterContentInit(){this.templates?.forEach(t=>{switch(t.getType()){case"icon":this._iconTemplate=t.template;break}})}containerClass(){let t="p-tag p-component";return this.severity&&(t+=` p-tag-${this.severity}`),this.rounded&&(t+=" p-tag-rounded"),this.styleClass&&(t+=` ${this.styleClass}`),t}static \u0275fac=(()=>{let t;return function(n){return(t||(t=p(e)))(n||e)}})();static \u0275cmp=b({type:e,selectors:[["p-tag"]],contentQueries:function(a,n,_){if(a&1&&(f(_,U,4),f(_,P,4)),a&2){let s;m(s=y())&&(n.iconTemplate=s.first),m(s=y())&&(n.templates=s)}},hostVars:4,hostBindings:function(a,n){a&2&&(k(n.style),I(n.containerClass()))},inputs:{style:"style",styleClass:"styleClass",severity:"severity",value:"value",icon:"icon",rounded:[2,"rounded","rounded",B]},features:[z([H]),$],ngContentSelectors:W,decls:5,vars:3,consts:[[4,"ngIf"],["class","p-tag-icon",4,"ngIf"],[1,"p-tag-label"],["class","p-tag-icon",3,"ngClass",4,"ngIf"],[1,"p-tag-icon",3,"ngClass"],[1,"p-tag-icon"],[4,"ngTemplateOutlet"]],template:function(a,n){a&1&&(D(),F(0),r(1,Y,2,1,"ng-container",0)(2,te,2,1,"span",1),u(3,"span",2),j(4),d()),a&2&&(o(),i("ngIf",!n.iconTemplate&&!n._iconTemplate),o(),i("ngIf",n.iconTemplate||n._iconTemplate),o(2),S(n.value))},dependencies:[q,E,N,O,g],encapsulation:2,changeDetection:0})}return e})(),be=(()=>{class e{static \u0275fac=function(a){return new(a||e)};static \u0275mod=C({type:e});static \u0275inj=h({imports:[oe,g,g]})}return e})()});export{oe as a,be as b,ie as c};
