import{Wj as o,Xj as d}from"./chunk-2GUKPKKM.js";import{Sd as c,ta as s,ya as i}from"./chunk-XJKQ67XR.js";import{e as a}from"./chunk-SZAW6ZBW.js";var p,u=a(()=>{c();d();p=(()=>{class e{constructor(t){this.assetServices=t}addLog(t,r=null,n=null){return this.assetServices.AddLogEntry(t,r,n)}updateLogEnd(t){return this.assetServices.UpdateLogEntryEnd(t)}static{this.\u0275fac=function(r){return new(r||e)(i(o))}}static{this.\u0275prov=s({token:e,factory:e.\u0275fac,providedIn:"root"})}}return e})()});export{p as a,u as b};
