import{a as e,b}from"./chunk-EXSN6QWV.js";import{V as s,W as F,j as T,k as D}from"./chunk-INYFUTXB.js";import{e as S,g as f}from"./chunk-FNS7DN6T.js";import{a as h,b as y}from"./chunk-3AUKOF45.js";import{b as g,c as v}from"./chunk-IBUDDWQO.js";import{J as a,Sd as p,Tb as c,Vb as l,ia as n,oa as u,za as m}from"./chunk-XJKQ67XR.js";import{e as d}from"./chunk-SZAW6ZBW.js";var x,C=d(()=>{p();f();b();D();v();y();u();F();p();x=(()=>{class r extends h{constructor(){super(),this.defaultPageSize=m(s).webSettingsSnapshot?.defaultPageSize??20,this.errorSummaryMessages=[],this.dateFormatToDisplay="",this.timeFormatToDisplay="",this.dateTimeFormatToDisplay="",this.BASE_TITLE="Devolutions Server - ",this.titleService=e.injector.get(S),this.storageService=e.injector.get(T)}ngOnInit(){e.injector.get(s).webSettingsChanged.pipe(n(this.destroyed$),a(o=>!!o)).subscribe(o=>this.onWebSettingsChanged(o));let t=e.injector.get(g).translate(this.getComponentTitle());this.setTitle(t)}initErrorSummary(){this.errorSummaryMessages=[]}onWebSettingsChanged(i){let t=e.injector.get(s);this.defaultPageSize=i.defaultPageSize,this.dateFormatToDisplay=t.displayDateFormat,this.timeFormatToDisplay=t.displayTimeFormat,this.dateTimeFormatToDisplay=t.displayDateTimeFormat}setTitle(i){this.titleService.setTitle(this.BASE_TITLE+i)}static{this.\u0275fac=function(t){return new(t||r)}}static{this.\u0275dir=c({type:r,features:[l]})}}return r})()});export{x as a,C as b};
