import{i as e,j as o}from"./chunk-2JML2BCE.js";import{e as a}from"./chunk-SZAW6ZBW.js";var t=a(()=>{o();e.setLanguage(["hr","hr-HR"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Pi\u0161i",Preview:"Pregled",Headings:"Naslovi",Paragraph:"Paragraf",Bold:"podebljano",Italic:"kurziv",Strike:"prcrtano",Code:"Uklopljeni k\xF4d",Line:"Linija",Blockquote:"Blok citat","Unordered list":"Neporedana lista","Ordered list":"Poredana lista",Task:"Task",Indent:"Pove\u0107aj uvlaku",Outdent:"Smanji uvlaku","Insert link":"Umetni link","Insert CodeBlock":"Umetni blok k\xF4da","Insert table":"Umetni tablicu","Insert image":"Umetni sliku",Heading:"Naslov","Image URL":"URL slike","Select image file":"Odaberi slikovnu datoteku","Choose a file":"Odaberite datoteka","No file":"Nema datoteka",Description:"Opis",OK:"OK",More:"Vi\u0161e",Cancel:"Odustani",File:"Datoteka",URL:"URL","Link text":"Tekst linka","Add row to up":"Dodaj redak prema gore","Add row to down":"Dodaj redak prema dolje","Add column to left":"Dodaj stupac s lijeve strane","Add column to right":"Dodajte stupac s desne strane","Remove row":"Ukloni redak","Remove column":"Remove stupac","Align column to left":"Poravnaj lijevo","Align column to center":"Poravnaj centrirano","Align column to right":"Poravnaj desno","Remove table":"Ukloni tablicu","Would you like to paste as table?":"Zalite li zalijepiti kao tablicu?","Text color":"Boja teksta","Auto scroll enabled":"Omogu\u0107i auto klizanje","Auto scroll disabled":"Onemogu\u0107i auto klizanje","Choose language":"Odabir jezika"})});t();
