import{e as s}from"./chunk-SZAW6ZBW.js";function e(){return e=Object.assign?Object.assign.bind():function(t){for(var n=1;n<arguments.length;n++){var a=arguments[n];for(var r in a)({}).hasOwnProperty.call(a,r)&&(t[r]=a[r])}return t},e.apply(null,arguments)}var u=s(()=>{});export{e as a,u as b};
