import{R as p,S as C}from"./chunk-INYFUTXB.js";import{a as h,b as v}from"./chunk-3AUKOF45.js";import{Bb as o,Gb as a,Rb as i,Sd as u,Vb as c,oc as l,pc as g,xc as s,yc as b,zc as d}from"./chunk-XJKQ67XR.js";import{e as m}from"./chunk-SZAW6ZBW.js";var f,M=m(()=>{v();u();C();f=(()=>{class r extends h{constructor(n){super(),this.themeService=n}ngOnInit(){this.theme=this.themeService.currentTheme==="light"?"light":"dark"}static{this.\u0275fac=function(e){return new(e||r)(a(p))}}static{this.\u0275cmp=i({type:r,selectors:[["dvls-risk-level"]],inputs:{riskLevel:"riskLevel"},features:[c],decls:5,vars:6,consts:[[1,"bar","high"],[1,"bar","mid"],[1,"bar","low"]],template:function(e,t){e&1&&(s(0,"div")(1,"div"),d(2,"span",0)(3,"span",1)(4,"span",2),b()()),e&2&&(l(t.theme),o(),g("bars ",t.riskLevel,""))},styles:["*[_ngcontent-%COMP%]{box-sizing:border-box}.light[_ngcontent-%COMP%], .dark[_ngcontent-%COMP%]{display:inline-block;padding:5px;margin-right:15px}.light[_ngcontent-%COMP%]{--neutral-color: 0, 0, 0;--border-color-tertiary: rgba(var(--neutral-color), .08);--bars-background-color: rgba(var(--neutral-color), .05);--bar-border-color: var(--border-color-tertiary);--bar-background-color: rgba(var(--neutral-color), .02)}.dark[_ngcontent-%COMP%]{--neutral-color: 255, 255, 255;--bars-background-color: rgba(var(--neutral-color), .15);--bar-border-color: var(--border-color-tertiary);--border-color-tertiary: rgba(var(--neutral-color), .1)}.bars[_ngcontent-%COMP%]{width:26px;height:26px;--bar-height: 6px;background:var(--bars-background-color);padding:3px 5px;border-radius:4px;display:flex;justify-content:space-between;align-items:flex-end;flex-direction:row-reverse}.bars.is-low[_ngcontent-%COMP%]   .low[_ngcontent-%COMP%], .bars.is-low[_ngcontent-%COMP%]   .low[_ngcontent-%COMP%] ~ .bar[_ngcontent-%COMP%]{--bar-background-color: #fde400}.bars.is-medium[_ngcontent-%COMP%]   .mid[_ngcontent-%COMP%], .bars.is-medium[_ngcontent-%COMP%]   .mid[_ngcontent-%COMP%] ~ .bar[_ngcontent-%COMP%]{--bar-background-color: #fd8900}.bars.is-high[_ngcontent-%COMP%]   .high[_ngcontent-%COMP%], .bars.is-high[_ngcontent-%COMP%]   .high[_ngcontent-%COMP%] ~ .bar[_ngcontent-%COMP%]{--bar-background-color: #fd0f00}.bar[_ngcontent-%COMP%]{width:4px;background-color:var(--bar-background-color);border:2px solid var(--bar-border-color);border-radius:2px}.bar.low[_ngcontent-%COMP%]{height:var(--bar-height)}.bar.mid[_ngcontent-%COMP%]{height:calc(var(--bar-height) * 2)}.bar.high[_ngcontent-%COMP%]{height:calc(var(--bar-height) * 3)}"]})}}return r})()});export{f as a,M as b};
