import{e as o}from"./chunk-SZAW6ZBW.js";var n,i=o(()=>{n=function(e){return e[e.Monday=1]="Monday",e[e.Tuesday=2]="Tuesday",e[e.Wednesday=4]="Wednesday",e[e.Thursday=8]="Thursday",e[e.Friday=16]="Friday",e[e.Saturday=32]="Saturday",e[e.Sunday=64]="Sunday",e}(n||{});(function(e){function u(){return"Days"}e.getClass=u;function d(r){let t="";return typeof e.getClass=="function"&&(t=e.getClass()),t+"_"+e[r]}e.getEnumKey=d})(n||(n={}))});export{n as a,i as b};
