import{d as w,e as R,i as D,j}from"./chunk-QH7CG3K4.js";import{Ia as a,La as I}from"./chunk-5YPL7P3G.js";import{D as C,K as M,m as v,s as S}from"./chunk-HJI5FNS6.js";import{Oa as r,Rb as m,Sb as p,Sd as b,Vb as u,jc as h,kc as f,kd as k,oc as y,ta as l,ua as c,za as d,zc as g}from"./chunk-XJKQ67XR.js";import{a as o,b as s,e as z}from"./chunk-SZAW6ZBW.js";var X,B,N,F,$,U,x=z(()=>{M();M();b();b();I();j();R();X=({dt:e})=>`
.p-skeleton {
    overflow: hidden;
    background: ${e("skeleton.background")};
    border-radius: ${e("skeleton.border.radius")};
}

.p-skeleton::after {
    content: "";
    animation: p-skeleton-animation 1.2s infinite;
    height: 100%;
    left: 0;
    position: absolute;
    right: 0;
    top: 0;
    transform: translateX(-100%);
    z-index: 1;
    background: linear-gradient(90deg, rgba(255, 255, 255, 0), ${e("skeleton.animation.background")}, rgba(255, 255, 255, 0));
}

[dir='rtl'] .p-skeleton::after {
    animation-name: p-skeleton-animation-rtl;
}

.p-skeleton-circle {
    border-radius: 50%;
}

.p-skeleton-animation-none::after {
    animation: none;
}

@keyframes p-skeleton-animation {
    from {
        transform: translateX(-100%);
    }
    to {
        transform: translateX(100%);
    }
}

@keyframes p-skeleton-animation-rtl {
    from {
        transform: translateX(100%);
    }
    to {
        transform: translateX(-100%);
    }
}
`,B={root:{position:"relative"}},N={root:({props:e})=>["p-skeleton p-component",{"p-skeleton-circle":e.shape==="circle","p-skeleton-animation-none":e.animation==="none"}]},F=(()=>{class e extends w{name="skeleton";theme=X;classes=N;inlineStyles=B;static \u0275fac=(()=>{let t;return function(i){return(t||(t=r(e)))(i||e)}})();static \u0275prov=l({token:e,factory:e.\u0275fac})}return e})(),$=(()=>{class e extends D{styleClass;style;shape="rectangle";animation="wave";borderRadius;size;width="100%";height="1rem";_componentStyle=d(F);containerClass(){return{"p-skeleton p-component":!0,"p-skeleton-circle":this.shape==="circle","p-skeleton-animation-none":this.animation==="none"}}get containerStyle(){let t=this._componentStyle?.inlineStyles.root,n;return this.size?n=s(o(o({},this.style),t),{width:this.size,height:this.size,borderRadius:this.borderRadius}):n=o(s(o({},t),{width:this.width,height:this.height,borderRadius:this.borderRadius}),this.style),n}static \u0275fac=(()=>{let t;return function(i){return(t||(t=r(e)))(i||e)}})();static \u0275cmp=m({type:e,selectors:[["p-skeleton"]],inputs:{styleClass:"styleClass",style:"style",shape:"shape",animation:"animation",borderRadius:"borderRadius",size:"size",width:"width",height:"height"},features:[k([F]),u],decls:1,vars:7,consts:[[3,"ngClass","ngStyle"]],template:function(n,i){n&1&&g(0,"div",0),n&2&&(y(i.styleClass),f("ngClass",i.containerClass())("ngStyle",i.containerStyle),h("data-pc-name","skeleton")("aria-hidden",!0)("data-pc-section","root"))},dependencies:[C,v,S,a],encapsulation:2,changeDetection:0})}return e})(),U=(()=>{class e{static \u0275fac=function(n){return new(n||e)};static \u0275mod=p({type:e});static \u0275inj=c({imports:[$,a,a]})}return e})()});export{$ as a,U as b,x as c};
