import{Sd as p,Ub as r}from"./chunk-XJKQ67XR.js";import{e as a}from"./chunk-SZAW6ZBW.js";var n,s=a(()=>{p();n=(()=>{class e{transform(t){return t==null?{}:{"transition-colors":!0,"text-sensitive":t>10&&t<=20,"animate-pulse":t<=10,"text-error":t<=10}}static{this.\u0275fac=function(i){return new(i||e)}}static{this.\u0275pipe=r({name:"animateOtp",type:e,pure:!0})}}return e})()});export{n as a,s as b};
