import{a as o,b as l}from"./chunk-UICD36F3.js";import{c as e,d as n}from"./chunk-CUIDVWNH.js";import{e as i}from"./chunk-SZAW6ZBW.js";var t,u=i(()=>{n();l();t=class s extends o{toQueryString(){let r=[];return r.push(super.toQueryString()),this.profilerLogs!==null&&this.profilerLogs.toString()&&r.push("profilerLogs="+encodeURIComponent(this.profilerLogs.toString())),r.join("&")}constructor(r=null){super(r),this.clone=()=>new s(this),this.data=e.manageDefaultValue(r,"data",null),this.profilerLogs=e.manageDefaultValue(r,"profilerLogs",[])}}});export{t as a,u as b};
