import{Sd as A,Ub as t}from"./chunk-XJKQ67XR.js";import{e as f}from"./chunk-SZAW6ZBW.js";function h(r,c,o="id"){if(c!==null){let p=new Map(c.map(a=>[a[o],a]));r=r.filter(a=>p.has(a[o])),p.forEach(a=>{r.some(e=>e[o]===a[o])||r.push(a)})}return r}function n(r){return r==null?[]:Array.isArray(r)?r:[r]}var u=f(()=>{});var y,i=f(()=>{u();A();y=(()=>{class r{transform(o){return n(o)}static{this.\u0275fac=function(p){return new(p||r)}}static{this.\u0275pipe=t({name:"toArray",type:r,pure:!0})}}return r})()});export{h as a,n as b,u as c,y as d,i as e};
