import{a as e,b as u}from"./chunk-A3DB3QKG.js";import{ic as o,jc as c,uc as r,vc as m}from"./chunk-2GUKPKKM.js";import{e as s}from"./chunk-SZAW6ZBW.js";var n,a=s(()=>{c();m();u();n=class extends e{constructor(p,t,i){super(o[o.Group],r[p],t,i),this.groupLabel="ConnectionType_Group"}}});export{n as a,a as b};
