import{i as e,j as t}from"./chunk-2JML2BCE.js";import{e as n}from"./chunk-SZAW6ZBW.js";var i=n(()=>{t();e.setLanguage(["de","de-DE"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Verfassen",Preview:"Vorschau",Headings:"\xDCberschriften",Paragraph:"Text",Bold:"Fett",Italic:"Kursiv",Strike:"Durchgestrichen",Code:"Code",Line:"Trennlinie",Blockquote:"Blocktext","Unordered list":"Aufz\xE4hlung","Ordered list":"Nummerierte Aufz\xE4hlung",Task:"Aufgabe",Indent:"Einr\xFCcken",Outdent:"Ausr\xFCcken","Insert link":"Link einf\xFCgen","Insert CodeBlock":"Codeblock einf\xFCgen","Insert table":"Tabelle einf\xFCgen","Insert image":"Grafik einf\xFCgen",Heading:"Titel","Image URL":"Bild URL","Select image file":"Grafik ausw\xE4hlen","Choose a file":"W\xE4hle eine Datei","No file":"Keine Datei",Description:"Beschreibung",OK:"OK",More:"Mehr",Cancel:"Abbrechen",File:"Datei",URL:"URL","Link text":"Anzuzeigender Text","Add row to up":"Zeile nach oben hinzuf\xFCgen","Add row to down":"Zeile nach unten hinzuf\xFCgen","Add column to left":"Spalte links hinzuf\xFCgen","Add column to right":"Spalte rechts hinzuf\xFCgen","Remove row":"Zeile entfernen","Remove column":"Spalte entfernen","Align column to left":"Links ausrichten","Align column to center":"Zentrieren","Align column to right":"Rechts ausrichten","Remove table":"Tabelle entfernen","Would you like to paste as table?":"M\xF6chten Sie eine Tabelle einf\xFCgen?","Text color":"Textfarbe","Auto scroll enabled":"Autoscrollen aktiviert","Auto scroll disabled":"Autoscrollen deaktiviert","Choose language":"Sprache ausw\xE4hlen"})});i();
