import{c as n,d as r}from"./chunk-CUIDVWNH.js";import{e as s}from"./chunk-SZAW6ZBW.js";var u,l=s(()=>{r();u=class t{toQueryString(){let e=[];if(this.connectionID!==null&&this.connectionID.toString()&&e.push("connectionID="+encodeURIComponent(this.connectionID.toString())),this.officeCacheSync!==null&&this.officeCacheSync.toString()&&e.push("officeCacheSync="+encodeURIComponent(this.officeCacheSync.toString())),this.offlineCacheSync!==null&&this.offlineCacheSync.toString()&&e.push("offlineCacheSync="+encodeURIComponent(this.offlineCacheSync.toString())),this.privateSubType!==null&&this.privateSubType.toString()&&e.push("privateSubType="+encodeURIComponent(this.privateSubType.toString())),this.unknownProperties)for(let i=0;i<this.unknownProperties.length;i++)this.unknownProperties[i]!==null&&e.push("unknownProperties="+encodeURIComponent(this.unknownProperties[i].toString()));return e.join("&")}constructor(e=null){this.clone=()=>new t(this),this.connectionID=n.manageDefaultValue(e,"connectionID",""),this.officeCacheSync=n.manageDefaultValue(e,"officeCacheSync",!1),this.offlineCacheSync=n.manageDefaultValue(e,"offlineCacheSync",!1),this.privateSubType=n.manageDefaultValue(e,"privateSubType",""),this.unknownProperties=n.manageDefaultArrayValue(e,"unknownProperties",null)}}});var o,a=s(()=>{o=class t{toQueryString(){return""}constructor(e=null){this.clone=()=>new t(this)}}});var c,h=s(()=>{r();a();c=class t extends o{toQueryString(){let e=[];return e.push(super.toQueryString()),this.private!==null&&this.private.toString()&&e.push("private="+encodeURIComponent(this.private.toString())),this.useSensitiveMode!==null&&this.useSensitiveMode.toString()&&e.push("useSensitiveMode="+encodeURIComponent(this.useSensitiveMode.toString())),e.join("&")}constructor(e=null){super(e),this.clone=()=>new t(this),this.private=n.manageDefaultValue(e,"private",!1),this.useSensitiveMode=n.manageDefaultValue(e,"useSensitiveMode",!1)}}});export{u as a,l as b,c,h as d};
