import{Wj as h,Xj as x,a as l,b as f}from"./chunk-2GUKPKKM.js";import{g as u,h as y}from"./chunk-CUIDVWNH.js";import{b as c,c as d}from"./chunk-IBUDDWQO.js";import{O as n,Sd as G,ka as i,na as S,oa as v,r,ta as a,x as s,ya as p}from"./chunk-XJKQ67XR.js";import{e as R}from"./chunk-SZAW6ZBW.js";var U,P=R(()=>{S();f();v();G();y();d();x();U=(()=>{class o extends l{constructor(t,e,m){super(e,t),this.assetService=m}getDataSourceLogReport(t){return this.assetService.GetDatasourceLogs(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportDataSourceLogReport(t){return this.assetService.GetDatasourceLogsExportFile(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getPasswordAnalyzer(t,e=!1){return this.assetService.GetPasswordStrengthAnalysis(t,e)}getSystemPermissionsReport(t){return this.assetService.GetSystemPermissionsReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportSystemPermissionsReport(t){return this.assetService.GetSystemPermissionsReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getUserCentricGatewayPermissionsReport(t){return this.assetService.GetGatewayPermissionsReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportUserCentricGatewayPermissionsReport(t){return this.assetService.GetGatewayPermissionsReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getAdministratorPermissionsReport(t){return this.assetService.GetAdministratorPermissionsReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportAdministratorPermissionsReport(t){return this.assetService.GetAdministratorPermissionsReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getVaultPermissionsReport(t){return t.repositoryIds.length===0?r(null):this.assetService.GetVaultPermissionsReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportVaultPermissionsReport(t){return this.assetService.GetVaultPermissionsReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getConnectionsPermissionsReport(t){return t.vaultIds.length===0?r(null):this.assetService.GetConnectionsPermissionsReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportConnectionsPermissionsReport(t){return this.assetService.GetConnectionsPermissionsReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getLicenseReport(t){return this.assetService.GetLicenseReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportLicenseReport(t){return this.assetService.GetLicenseReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}getLicensesByUsersReport(t){return this.assetService.GetLicensesByUsersReport(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}getLicensesByUsersLicenseUsers(t){return this.assetService.GetLicensesByUsersLicenseUsers(t).pipe(i(e=>this.handleGetApiResult(e)),s(e=>this.returnWhenSuccessful(e,()=>e.data)))}exportLicensesByUsersReport(t){return this.assetService.GetLicensesByUsersReportExport(t).pipe(s(e=>new Blob([e],{type:"text/csv"})),n(()=>r(null)))}static{this.\u0275fac=function(e){return new(e||o)(p(u),p(c),p(h))}}static{this.\u0275prov=a({token:o,factory:o.\u0275fac,providedIn:"root"})}}return o})()});export{U as a,P as b};
