import{a as n,c as m}from"./chunk-NUDGNJBM.js";import{a as o,b as s}from"./chunk-42YXKNUY.js";import"./chunk-QF7733B7.js";import"./chunk-ZSHAELBT.js";import"./chunk-7IASPOWU.js";import"./chunk-74IZ4RNT.js";import"./chunk-ZFMQAUJZ.js";import"./chunk-WNPXWF4E.js";import"./chunk-QQZCALHS.js";import"./chunk-UICD36F3.js";import"./chunk-VOLZZ3JP.js";import"./chunk-INYFUTXB.js";import"./chunk-FNS7DN6T.js";import{Yb as i,Zb as e}from"./chunk-2GUKPKKM.js";import"./chunk-CUIDVWNH.js";import"./chunk-3AUKOF45.js";import"./chunk-5YPL7P3G.js";import"./chunk-Z2WWNJIF.js";import"./chunk-IBUDDWQO.js";import"./chunk-HJI5FNS6.js";import"./chunk-XJKQ67XR.js";import{e as t}from"./chunk-SZAW6ZBW.js";var I,a=t(()=>{m();s();e();I=[{canActivate:[o,n],data:{permissions:[i.MonitoringPrivilegedSessionsView]},path:"",loadComponent:()=>import("./chunk-C3ZNURSI.js").then(r=>r.PrivilegedSessionMonitoringComponent)}]});a();export{I as PRIVILEGED_SESSIONS_MONITORING};
