import{Sd as s,h as o,na as n,ta as r}from"./chunk-XJKQ67XR.js";import{e as i}from"./chunk-SZAW6ZBW.js";var m,u=i(()=>{n();s();m=(()=>{class t{constructor(){this.modelSubject=new o(null)}getModelSubject(){return this.modelSubject}setModelSubject(e){this.modelSubject.next(e)}static{this.\u0275fac=function(c){return new(c||t)}}static{this.\u0275prov=r({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{m as a,u as b};
