import{e as d,f as M}from"./chunk-2JGOCWLT.js";import{K as l,L as A}from"./chunk-AJRIRLVM.js";import{Wj as f,Xj as g,a as m,b as T}from"./chunk-2GUKPKKM.js";import{g as p,h as C}from"./chunk-CUIDVWNH.js";import{b as c,c as j}from"./chunk-IBUDDWQO.js";import{Sd as S,ka as a,oa as v,ta as s,x as n,ya as o}from"./chunk-XJKQ67XR.js";import{e as u}from"./chunk-SZAW6ZBW.js";var E,N=u(()=>{T();v();A();S();C();j();M();g();E=(()=>{class i extends m{constructor(r,e,t,h){super(e,r),this.treeNodeConnectionManager=t,this.assetServices=h}get(r,e=null){return this.assetServices.GetPartialConnectionTree(r,e,!1).pipe(a(t=>this.handleGetApiResult(t)),n(t=>this.returnWhenSuccessful(t,()=>this.treeNodeConnectionManager.mapConnectionsToTreeNodes(new l(t.data),r,e))))}static{this.\u0275fac=function(e){return new(e||i)(o(p),o(c),o(d),o(f))}}static{this.\u0275prov=s({token:i,factory:i.\u0275fac,providedIn:"root"})}}return i})()});export{E as a,N as b};
