import{a as K,b as ie}from"./chunk-AO5QPNGJ.js";import{a as U,b as z,c as J}from"./chunk-DM2XHT3H.js";import{C as j,E as k,a as F,b as P,f as A,i as D,m as O,w as L,y as W}from"./chunk-K7KAWBRR.js";import{a as H,b as ee}from"./chunk-CMGJXTS5.js";import{b as G,c as $}from"./chunk-IBUDDWQO.js";import{$c as q,Bb as n,Dc as V,Gb as f,Hc as x,Ka as h,La as c,Rb as b,Sd as B,Xb as _,Yc as y,_c as p,ad as M,b as u,ed as S,fd as w,gd as R,jc as E,kc as v,kd as N,mc as g,na as Z,oc as T,sa as I,sc as C,vd as d,wd as m,xc as l,yc as o}from"./chunk-XJKQ67XR.js";import{e as Y}from"./chunk-SZAW6ZBW.js";function te(r,Q){r&1&&(l(0,"div",3),p(1),d(2,"translate"),o()),r&2&&(n(),M(" ",m(2,1,"TheEmailIsInvalid")," "))}function re(r,Q){r&1&&(l(0,"div",3),p(1),d(2,"translate"),o()),r&2&&(n(),M(" ",m(2,1,"TheEmailIsRequired")," "))}var Me,ne=Y(()=>{B();k();Z();J();ie();ee();B();k();$();J();Me=(()=>{class r{constructor(e,a){this.translatorService=a,this.labelName="lblEmail",this.isRequired=!1,this.styleClass="",this.disabled=!1,this.maxlength=null,this.invalidCharacterMessage=this.translatorService.translate("TheNameCanNotContainsTheInvalidCharacter","\\"),this.innerValue="",this.onModelChange=u,this.onModelTouched=u,this.parent=e}get value(){return this.innerValue}set value(e){e!==null&&e!==this.innerValue&&(this.writeValue(e),this.onModelChange(e))}isFormSubmitted(){return this.parent.submitted}onInputBlur(e){this.onModelTouched()}registerOnChange(e){this.onModelChange=e}registerOnTouched(e){this.onModelTouched=e}writeValue(e){this.innerValue=e}static{this.\u0275fac=function(a){return new(a||r)(f(D,9),f(G))}}static{this.\u0275cmp=b({type:r,selectors:[["sd-input-email"]],inputs:{labelName:"labelName",isRequired:"isRequired",styleClass:"styleClass",disabled:"disabled",maxlength:"maxlength",value:"value"},features:[N([{provide:F,useExisting:I(()=>r),multi:!0}])],decls:9,vars:18,consts:[["emailModel","ngModel"],[1,"block","text-xs","font-normal","py-1.5","pr-[3px]","text-left","text-[var(--text-400)]"],["pInputText","","name","email","sdValidateEmail","",3,"ngModelChange","blur","disabled","required","ngModel","maxlength"],[1,"dvl-form-error"]],template:function(a,t){if(a&1){let i=V();l(0,"div"),d(1,"translate"),l(2,"label",1),p(3),d(4,"translate"),o(),l(5,"input",2,0),R("ngModelChange",function(s){return h(i),w(t.value,s)||(t.value=s),c(s)}),x("blur",function(s){return h(i),c(t.onInputBlur(s))}),o(),_(7,te,3,3,"div",3)(8,re,3,3,"div",3),o()}if(a&2){let i=y(6);g("form-error",i.errors&&i.errors.email||i.invalid),E("data-error",i.errors&&i.errors.email?m(1,14,"TheEmailIsRequired"):t.invalidCharacterMessage),n(2),g("required",t.isRequired),n(),q(m(4,16,t.labelName)),n(2),T(t.styleClass),v("disabled",t.disabled)("required",t.isRequired),S("ngModel",t.value),v("maxlength",t.maxlength),n(2),C(i.errors&&i.errors.email?7:-1),n(),C(t.isRequired&&i.invalid&&(i.dirty||i.touched)&&(i!=null&&i.errors.required)?8:-1)}},dependencies:[j,P,A,L,W,O,z,U,K,H],encapsulation:2})}}return r})()});export{Me as a,ne as b};
