import{Da as c,La as f}from"./chunk-5YPL7P3G.js";import{b as a,c as u}from"./chunk-IBUDDWQO.js";import{Sd as l,ta as r,ya as o}from"./chunk-XJKQ67XR.js";import{e as n}from"./chunk-SZAW6ZBW.js";var v,d=n(()=>{l();f();u();v=(()=>{class s{constructor(t,e){this.toastService=t,this.translationService=e}successfulOperation(t,e){this.feedback("success",t,e)}failedOperation(t,e){this.feedback("error",t,e)}feedback(t,e,i){this.toastService.add({key:"globalToast",severity:t,summary:e,detail:i,closable:!0})}testConnectionFeedback(t,e,i){t?this.testConnectionSuccessful(e,i):this.testConnectionFailed(e,i)}testConnectionSuccessful(t,e){let i=this.translationService.translate("ConnectionSuccessful");t&&(i=`${t} ${i}`),this.successfulOperation(i,e)}testConnectionFailed(t,e){let i=this.translationService.translate("ConnectionFailed");t&&(i=`${t} ${i}`),this.failedOperation(i,e)}static{this.\u0275fac=function(e){return new(e||s)(o(c),o(a))}}static{this.\u0275prov=r({token:s,factory:s.\u0275fac,providedIn:"root"})}}return s})()});export{v as a,d as b};
