import{a as r,b as p}from"./chunk-BXBF54ET.js";import{Gb as u,Sd as l,Tb as c,Za as s,o,p as f}from"./chunk-XJKQ67XR.js";import{e as a}from"./chunk-SZAW6ZBW.js";var t,m=a(()=>{f();p();l();t=class e{constructor(i){this.el=i,this.autoFocus=!0}ngAfterViewInit(){this.autoFocus&&this.focusInput()}focusInput(){this.el.nativeElement.focus(),this.autoSelect&&this.el.nativeElement.select()}static{this.\u0275fac=function(n){return new(n||e)(u(s))}}static{this.\u0275dir=c({type:e,selectors:[["","sdAutoFocusInput",""]],inputs:{autoFocus:"autoFocus",autoSelect:"autoSelect"}})}};o([r()],t.prototype,"focusInput",null)});export{t as a,m as b};
