import{Of as i,Pf as p}from"./chunk-2GUKPKKM.js";import{e as m}from"./chunk-SZAW6ZBW.js";var I,u=m(()=>{p();I=(()=>{class e{static{this.TOOLBAR_SIZE=44}constructor(s,t,n,o,h="",c,a=i(),d,r){this.name="",this.icon="",this.name=s,this.component=t,this.data=n,this.sessionId=o,this.icon=h,this.tabIndex=c,this.id=a,this.status=d,this.desktopSize=r,this.createdDate=new Date}clone(s){let t=Object.assign(new e(this.name,this.component,this.data,this.sessionId,this.icon,this.tabIndex,this.id,this.status,this.desktopSize),this,s);return t.componentRef=this.componentRef,t}updatedTabIndex(s){let t=this.componentRef?.instance;return t&&(t.tabIndex=s),this.tabIndex=s,this}get connectionClosedObserver(){return this.componentRef?.instance?.connectionClosedObserver}}return e})()});export{I as a,u as b};
