import{e as l}from"./chunk-SZAW6ZBW.js";var r,u=l(()=>{r=function(n){return n[n.Received=0]="Received",n[n.Sent=1]="Sent",n}(r||{});(function(n){function t(){return"AccessRequestViewOption"}n.getClass=t;function a(g){let f="";return typeof n.getClass=="function"&&(f=n.getClass()),f+"_"+n[g]}n.getEnumKey=a})(r||(r={}))});export{r as a,u as b};
