import{e as a}from"./chunk-SZAW6ZBW.js";var n,m=a(()=>{n=function(t){return t[t.Group=0]="Group",t[t.Single=1]="Single",t[t.System=2]="System",t}(n||{});(function(t){function g(){return"SubscriberLinkType"}t.getClass=g;function f(l){let o="";return typeof t.getClass=="function"&&(o=t.getClass()),o+"_"+t[l]}t.getEnumKey=f})(n||(n={}))});export{n as a,m as b};
