import{a as l,b}from"./chunk-VOLZZ3JP.js";import{V as f,W as C}from"./chunk-INYFUTXB.js";import{Wj as p,Xj as A}from"./chunk-2GUKPKKM.js";import{g as m,h as j}from"./chunk-CUIDVWNH.js";import{b as n,c as d}from"./chunk-IBUDDWQO.js";import{Sd as S,ka as a,oa as v,ta as c,x as o,ya as r}from"./chunk-XJKQ67XR.js";import{e as u}from"./chunk-SZAW6ZBW.js";var w,g=u(()=>{v();b();S();A();C();d();j();w=(()=>{class t extends l{constructor(s,i,e,h){super(e,h,i),this.assetServices=s}getAllRoleSummaries(s=!1){let i=this.assetServices.GetRolesBasic().pipe(a(e=>this.handleGetApiResult(e)),o(e=>this.returnWhenSuccessful(e,()=>e.data)));return this.getCache(s,i)}static{this.\u0275fac=function(i){return new(i||t)(r(p),r(f),r(n),r(m))}}static{this.\u0275prov=c({token:t,factory:t.\u0275fac,providedIn:"root"})}}return t})()});export{w as a,g as b};
